/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20090126: alil, alil@users.sourceforge.net - fixed bug identified by cppcheck 
 *******************************************************************************/
#ifndef _DEVLOG_H_
#define _DEVLOG_H_

/*!\ingroup FORTE_HAL
 * \brief CDeviceLog is the entity that logs messages created by the FORTE Runtime system.
 * They can be printed to a console or archived somewhere (This is implementation dependent).
 *
 * @author SMR
 * @version 20070912/AZ - Added DEVLOG prefix to the DeviceLog macros
 * @version 20060717/SMR - revised version of the DeviceLog
 * @version 20051024/SMR - Initial creation of h-file for defining the interface
 *
*/
#include "datatype.h"
#include "../core/singlet.h"
#include "active_arch/sync.h"

// possible loglevels: NOLOG, LOGERROR, LOGWARNING, LOGINFO, LOGDEBUG
/* Meaning:
 * LOGERROR: log only error messages
 * LOGWARNING: log error and warning messages
 * LOGINFO: log error, warning, and info messages
 * LOGDEBUG: log all messages: error, warning, info, and debug
 * NOLOG: log no messages
 */

#if !(defined(NOLOG) || defined(LOGERROR) || defined(LOGWARNING) || defined(LOGINFO) || defined(LOGDEBUG))
/* Set default loglevel */
# define LOGDEBUG
#endif

#ifdef LOGDEBUG
  #define DEVLOG_ERROR(a,b,c) CDeviceLog::getInstance().addMsg(E_ERROR,a,b,c)
  #define DEVLOG_WARNING(a,b,c) CDeviceLog::getInstance().addMsg(E_WARNING,a,b,c)
  #define DEVLOG_INFO(a,b,c) CDeviceLog::getInstance().addMsg(E_INFO,a,b,c)
  #define DEVLOG_DEBUG(a,b,c) CDeviceLog::getInstance().addMsg(E_DEBUG,a,b,c)

  #define DEVLOG_ERROR_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d)
  #define DEVLOG_WARNING_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d)
  #define DEVLOG_INFO_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_INFO,a,b,c,d)
  #define DEVLOG_DEBUG_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_DEBUG,a,b,c,d)

  #define DEVLOG_ERROR_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d,e)
  #define DEVLOG_WARNING_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d,e)
  #define DEVLOG_INFO_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_INFO,a,b,c,d,e)
  #define DEVLOG_DEBUG_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_DEBUG,a,b,c,d,e)
#endif

#ifdef LOGERROR
  #define DEVLOG_ERROR(a,b,c) CDeviceLog::getInstance().addMsg(E_ERROR,a,b,c)
  #define DEVLOG_WARNING(a,b,c)
  #define DEVLOG_INFO(a,b,c)
  #define DEVLOG_DEBUG(a,b,c)

  #define DEVLOG_ERROR_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d)
  #define DEVLOG_WARNING_2(a,b,c,d)
  #define DEVLOG_INFO_2(a,b,c,d)
  #define DEVLOG_DEBUG_2(a,b,c,d)

  #define DEVLOG_ERROR_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d,e)
  #define DEVLOG_WARNING_3(a,b,c,d,e)
  #define DEVLOG_INFO_3(a,b,c,d,e)
  #define DEVLOG_DEBUG_3(a,b,c,d,e)
#endif

#ifdef LOGWARNING
  #define DEVLOG_ERROR(a,b,c) CDeviceLog::getInstance().addMsg(E_ERROR,a,b,c)
  #define DEVLOG_WARNING(a,b,c) CDeviceLog::getInstance().addMsg(E_WARNING,a,b,c)
  #define DEVLOG_INFO(a,b,c)
  #define DEVLOG_DEBUG(a,b,c)

  #define DEVLOG_ERROR_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d)
  #define DEVLOG_WARNING_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d)
  #define DEVLOG_INFO_2(a,b,c,d)
  #define DEVLOG_DEBUG_2(a,b,c,d)

  #define DEVLOG_ERROR_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d,e)
  #define DEVLOG_WARNING_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d,e)
  #define DEVLOG_INFO_3(a,b,c,d,e)
  #define DEVLOG_DEBUG_3(a,b,c,d,e)
#endif

#ifdef LOGINFO
 #define DEVLOG_ERROR(a,b,c) CDeviceLog::getInstance().addMsg(E_ERROR,a,b,c)
  #define DEVLOG_WARNING(a,b,c) CDeviceLog::getInstance().addMsg(E_WARNING,a,b,c)
  #define DEVLOG_INFO(a,b,c) CDeviceLog::getInstance().addMsg(E_INFO,a,b,c)
  #define DEVLOG_DEBUG(a,b,c)

  #define DEVLOG_ERROR_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d)
  #define DEVLOG_WARNING_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d)
  #define DEVLOG_INFO_2(a,b,c,d) CDeviceLog::getInstance().addVarMsg(E_INFO,a,b,c,d)
  #define DEVLOG_DEBUG_2(a,b,c,d)

  #define DEVLOG_ERROR_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_ERROR,a,b,c,d,e)
  #define DEVLOG_WARNING_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_WARNING,a,b,c,d,e)
  #define DEVLOG_INFO_3(a,b,c,d,e) CDeviceLog::getInstance().addVarMsg(E_INFO,a,b,c,d,e)
  #define DEVLOG_DEBUG_3(a,b,c,d,e)
#endif

#ifdef NOLOG
  #define DEVLOG_INFO(a,b,c)
  #define DEVLOG_WARNING(a,b,c)
  #define DEVLOG_ERROR(a,b,c)
  #define DEVLOG_DEBUG(a,b,c)

  #define DEVLOG_INFO_2(a,b,c,d)
  #define DEVLOG_WARNING_2(a,b,c,d)
  #define DEVLOG_ERROR_2(a,b,c,d)
  #define DEVLOG_DEBUG_2(a,b,c,d)

  #define DEVLOG_INFO_3(a,b,c,d,e)
  #define DEVLOG_WARNING_3(a,b,c,d,e)
  #define DEVLOG_ERROR_3(a,b,c,d,e)
  #define DEVLOG_DEBUG_3(a,b,c,d,e)
#endif

enum E_MsgLevel {E_INFO, E_WARNING, E_ERROR, E_DEBUG};

// here are the definitions for the Logbook Identifiers to be used by the classes
// TODO: Add numbers of new Classes here!
const TUINT8 cg_nID_CDevice                = 0x00;
const TUINT8 cg_nID_CResource              = 0x01;
const TUINT8 cg_nID_CFunctionBlock         = 0x02;
const TUINT8 cg_nID_CConnection            = 0x03;
const TUINT8 cg_nID_CDeviceAdministrator   = 0x04;
const TUINT8 cg_nID_CExtEventHandler       = 0x05;
const TUINT8 cg_nID_CThread                = 0x06;
const TUINT8 cg_nID_CSyncObject            = 0x07;
const TUINT8 cg_nID_CMemoryManager         = 0x08;
const TUINT8 cg_nID_CPersistantManager     = 0x09;
const TUINT8 cg_nID_CTimmerHandler         = 0x0A;
const TUINT8 cg_nID_CIEC_ANY               = 0x0B;
const TUINT8 cg_nID_CDeviceLog             = 0xFF;
const TUINT8 cg_nID_CTypeLib               = 0xFE;
const TUINT8 cg_nID_CSocketHandler			   = 0xFD;
const TUINT8 cg_nID_CNetCommFB             = 0xFC;
const TUINT8 cg_nID_Client                 = 0xFB;
const TUINT8 cg_nID_Server                 = 0xFB;
const TUINT8 cg_nID_Publish                = 0xF9;
const TUINT8 cg_nID_Subscribe              = 0xF9;


class CDeviceLog{
    SINGLETON(CDeviceLog)
  public:

    struct  SLogEntry{
      TUINT8 m_nSrcID;
      TUINT8 m_nSubID;
      E_MsgLevel m_enLevel;
      TUINT16 m_nMsg;
      TUINT32 m_nTimestamp;
    };

  private:
    static const int scm_nMsgBufSize;
    static char sm_acMsgBuf[100];   //!<Buffer for the messages created by the variable addMsg function
/*! \brief number of entries to be stored
 *
*/
    static const TUINT8 scm_nDepth = 100;
    SLogEntry m_stMsgList[scm_nDepth];
    int m_nPos;
    unsigned int m_nNum;


  protected:
    CSyncObject m_oSync;
  public:
    ~CDeviceLog() {};
/*! \brief Adds an Entry to the LogBook with an additional Text-Massage, that is written to a console
 *
 */
    bool addMsg(E_MsgLevel pa_enLevel, TUINT8 pa_nSrcID, TUINT8 pa_nSubID, const char *pa_acMessage);


    bool addVarMsg(E_MsgLevel pa_enLevel, TUINT8 pa_nSrcID, TUINT8 pa_nSubID, const char *pa_acMessage, ...);


/*! \brief Returns the last Message that was written to the LogBook in a structure
 *
 */
    SLogEntry &getMsg();
/*! \brief Returns the actual number of Entries stored in the Logbook
 *
 */
    unsigned int getMsgNumbers();
};

#endif //_DEVLOG_H_
