/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "thread.h"
#include <stdio.h>
const char *CTXThread::csm_cName = {"Test Name"};

const int CTXThread::scm_nThreadListSize = 27;
TCTXThreadPtr CTXThread::sm_aoThreadList[27] ={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};


bool CTXThread::create(void){
 // Get new Thread
 
  if (m_stThread.tx_thread_id != 0) return true;
  tx_thread_create(&m_stThread,
            "test",
            threadFunction,
            (ULONG)(this),
            m_cStack,
            m_nStackSize,
            scm_nThreadListSize + 3,
            scm_nThreadListSize + 3,   //disable preemption threshhold
            TX_NO_TIME_SLICE,  //disable timeslicing
            TX_AUTO_START);  // we start manually
  tx_thread_sleep(1);
 
  //TODO check return values 
  return true;         
}

void CTXThread::threadFunction(ULONG data){
 // Get pointer to CThread object out of void pointer
  CTXThread *pThread = (CTXThread*)(data);
  // if pointer is ok
  if (pThread){
    tx_mutex_get(&(pThread->m_stMutex), TX_WAIT_FOREVER);  
    pThread->run();  
    pThread->m_bAlive = false;
    tx_mutex_put(&(pThread->m_stMutex));
//TODO add error code e.g.: if (!pThread->run())
  }
}

CTXThread::CTXThread(long pa_nStackSize){
  m_bSuspended = true;
  m_bAlive     = true;
  m_nStackSize = pa_nStackSize;
  m_cStack = new unsigned char[m_nStackSize];
  m_stThread.tx_thread_id = 0;
  tx_mutex_create(&m_stMutex, "test", TX_NO_INHERIT);
  tx_semaphore_create(&m_stSemaphore, "hugo", 0);
  m_nDeadLine =  0;
  
}

CTXThread::~CTXThread(){
  end();
  tx_mutex_delete(&m_stMutex);
}

bool CTXThread::isAlive(void)      { return m_bAlive; }

void CTXThread::setDeadline(TUINT32 pa_nVal){
  int i, ii, error;
  m_nDeadLine = pa_nVal;
  //first of all check if this thread is allready in the list and remove it from the list
  for(i = 0; i < scm_nThreadListSize; i++){
    if(0 == sm_aoThreadList[i])
      break;
    else
      if(this == sm_aoThreadList[i]){
        for(ii = i; ii < scm_nThreadListSize - 1; ii++){
          if(0 == sm_aoThreadList[ii + 1])
            break;
          sm_aoThreadList[ii + 1]->setPriority(ii + 2);
          sm_aoThreadList[ii] = sm_aoThreadList[ii + 1];  
        }
        break;
      }  
  }

  if(0 == m_nDeadLine)
    setPriority( scm_nThreadListSize + 2);
  else{
    for(i = 0; i < scm_nThreadListSize; i++){
      if(0 == sm_aoThreadList[i]){
        sm_aoThreadList[i] = this;
        setPriority( i + 2);
        break;
      }  
      else
        if(m_nDeadLine < sm_aoThreadList[i]->getDeadline()){
          CTXThread *poRBuf, *poSBuf= sm_aoThreadList[i];
          sm_aoThreadList[i] =  this;
          setPriority( i + 2);
          for(ii = i + 1; ii < scm_nThreadListSize; ii++){
            poSBuf->setPriority(ii + 2);
            poRBuf = sm_aoThreadList[ii];
            sm_aoThreadList[ii] = poSBuf;
            if(0 == poRBuf)
              break;
            poSBuf = poRBuf;  
          }  
          break;
        }
    }      
  }   
}

TUINT32 CTXThread::getDeadline(void){
  return m_nDeadLine;
}

void CTXThread::start(void){
  m_bSuspended = false;
  create();
}

void CTXThread::selfSuspend(void){
    tx_semaphore_get(&m_stSemaphore, TX_WAIT_FOREVER);
}

void CTXThread::resumeSelfSuspend(void){
     // if the thread is not created yet, we do this here
    tx_semaphore_put(&m_stSemaphore);
}

void CTXThread::end(void){
  m_bAlive = false;
  resumeSelfSuspend();
  join();
}

void CTXThread::join(void){
  tx_mutex_get(&m_stMutex, TX_WAIT_FOREVER); 
  tx_mutex_put(&m_stMutex);
  tx_semaphore_delete(&m_stSemaphore);
  // TODO: check if delete works
  m_stThread.tx_thread_id = 0;
  tx_thread_delete(&m_stThread); 
}


// function for output of the semaphore status, for debugging
void CTXThread::printseminfo(TX_SEMAPHORE *pa_stSemaphore) {
CHAR *name;
ULONG current_value;
TX_THREAD *first_suspended;
ULONG suspended_count;
TX_SEMAPHORE *next_semaphore;
UINT status;
status = tx_semaphore_info_get(pa_stSemaphore, &name, &current_value, &first_suspended, &suspended_count, &next_semaphore);
printf("Seminfo: Name : %s, Value : %d, First Susp. Thread: %d, status : %d\n",\
     name, current_value, first_suspended->tx_thread_id, status);
}

