/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20081001: alil, alil@users.sourceforge.net - redesigned baudrate assignment framework to be easierly adaptable on each 
 *                                                 architecture
 *******************************************************************************/
#ifndef _TXSERHAN_H_
#define _TXSERHAN_H_

#include "../serhand.h"
#include "thread.h"

/*!\brief This class is the implementation of the Serial Handler for the NET+OS architecture.
 * \ingroup EXTEVHAND
 * \ingroup netos_hal
 * 
 * If the serial port to be used is currently defined as the standard output port,
 * the defines APP_DIALOG_PORT and APP_STDIO_PORT have to be disabled (commented)
 * in 'src/arch/threadx/appconf.h'. 
 *   
 * @author jbrunn
 * @version 20061214/jbrunn - Changed receiveDataOnSerial function and updated documentation
 * @version 20061204/jbrunn - Changed service types and fixed small send bug
 * @version 20061201/jbrunn - Added stopwatch functions and made data transfer more secure at low Baud rates
 * @version 20061130/jbrunn - Replaced old Serial Handler with this one
 * @version 20061128/jbrunn - Changed to work with renewed Serial Handler inherited from External Event Handler
 * @version 20061122/jbrunn - First working version
 * @version 20061116/jbrunn - Initial version
 */
class CTXSerialHandler : public CSerialHandler, public CThread{

  private:

//! The number of active serial ports.
  int m_nNumSerial;
/*!\brief Start time of the stopwatch.
 */     
  unsigned long m_nStartTime;
/*!\brief Hold time of the stopwatch.
 */       
  unsigned long m_nHoldTime;
/*!\brief Flag that indicates if the start time has been set.
 */   
  bool m_bStartFlag;
/*!\brief Flag that indicates if the hold time has been set.
 */
  bool m_bHoldFlag;

/*!\brief This function periodically checks for Traffic on the Serial port.
 */
  void selectRoutine(void);
/*!\brief Calculates the highest SerialID in use.
 *
 * \param pa_nSerialId The SerialID of a port that is opened or closed
 * \param pa_nMode 0 adds the SerialID, non-zero removes the SerialID
 * \return Status value
 */
  int calcHighestSerialID(int pa_nSerialId, int pa_nMode);

  int openSerialPort(CSerialHandler::SConnectionInformation &pa_stConnectionInformation);
  int closeSerialPort(SSerialHandlerListEntry &pa_stData);
  int setSerialPortOptions(int pa_nSerialID, CSerialHandler::SConnectionInformation &pa_stConnectionInformation);
  int sendDataOnSerial(SSerialHandlerListEntry &pa_stData);
  int receiveDataOnSerial(SSerialHandlerListEntry &pa_stData);
  void startStopWatch(void);
  void holdStopWatch(void);
  double readStopWatch(void);

protected:
//!The highest SerialID in use.
  int m_nHighestSerialID;

public:
  CTXSerialHandler();
  virtual ~CTXSerialHandler();
/*!\brief 
 */
  virtual bool run(void);
/*!\brief Enables this event source
 */
  virtual void enableHandler(void);  
/*!\brief Disable this event source
 */  
  virtual void disableHandler(void); 
/*!\brief Sets the priority of the event source  
 * 
 * \param pa_nPriority new priority of the event source
 */
  virtual void setPriority(int pa_nPriority);
/*!\brief Get the current priority of the event source
 * 
 * \return current priority
 */  
  virtual int getPriority(void);
};

#endif //_TXSERHAN_H_
