/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20080925: alil, alil@users.sourceforge.net - added patch from DResearch Digital Media Systems
 *******************************************************************************/
#include "txsocha.h"
#include "../../core/devexec.h"
#include "../devlog.h"

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sockapi.h>
#include <sys/wait.h>
#include <signal.h>
#include <fcntl.h>

CSocketHandler &CSocketHandler::getInstance(void) {
  static CTXSocketHandler oSocketHandler;
  return oSocketHandler;
}

int CTXSocketHandler::calcHighestSocketID(int pa_nSockId, int pa_nMode) {

  int maxid = 0;
  if (pa_nMode == 0) { // adding a new SocketID
    if (pa_nSockId > m_nHighestSocketID) {
      m_nHighestSocketID = pa_nSockId;
    }
  } else { // removing a SocketID
    std::list<SSocketHandlerListEntry>::iterator iter;
    for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++) {
      if ((maxid < iter->m_nSocketID) && (iter->m_nSocketID != pa_nSockId))
        maxid = iter->m_nSocketID;
      if ((maxid < iter->m_nAcceptID) && (iter->m_nAcceptID != pa_nSockId))
        maxid = iter->m_nAcceptID;
    }
    m_nHighestSocketID = maxid;
  }
  m_bSocketListChanged=true;
  return 0;
}

bool CTXSocketHandler::run(void) {
  m_eState = e_Run;

  SelectRoutine();
  return true;
}
;

void CTXSocketHandler::SelectRoutine(void) {
  fd_set socks;
  fd_set socks_save;
  struct timeval tv;
  int retval;
  int ret_checkreceived;
  //        int i;
  int nNotifiedSIFB;
  std::list<SSocketHandlerListEntry>::iterator iter;
  //        LOG(cg_nID_CExtEventHandler, 1, E_INFO, 0, "\nsockethander: select-thread started"); 
  //        printf("CTXSocketHandler:: select-thread started\n");
  DEVLOG_DEBUG(0xFD, 0x01, "CTXSocketHandler:: select-thread started");
  m_bSocketListChanged=true;
  while (1) {
    nNotifiedSIFB=0;
    m_oSync.lock();
    tv.tv_sec = 1; //TODO : To be set!
    tv.tv_usec = 25000;
    if (m_bSocketListChanged) {
      m_bSocketListChanged=false;
      FD_ZERO(&socks);
      DEVLOG_DEBUG(cg_nID_CSocketHandler, 0x03, "CTXSocketHandler: rebuilding socketlist");
      //            fprintf(stderr,"CTXSocketHandler:: Setting sockets that are listened:\n");
      for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++) {
        /*	        fprintf(stderr,"SocketID: %d\tAcceptID: %d\tBlocked: ",iter->m_nSocketID,iter->m_nAcceptID);
         if (iter->m_bLocked)
         fprintf(stderr,"true\n");
         else
         fprintf(stderr,"false\n");*/
        //  			if (iter->m_bLocked == false) {
        if (iter->m_nAcceptID!=-1) {
          FD_SET(iter->m_nAcceptID,&socks);
          //	              printf("AcceptID: %d\t",iter->m_nAcceptID);
        } else {
          FD_SET(iter->m_nSocketID,&socks);
          //                printf("SocketID: %d\t",iter->m_nSocketID);
        }
        //  	      }
      }
      //            printf("maxID: %i\n",m_nHighestSocketID);
      socks_save=socks;
    } else
      socks=socks_save;
    fprintf(stderr,".");
    //          printf("\n"); 
    m_oSync.unlock();
    sel_mark: retval = select(m_nHighestSocketID+1, &socks, NULL, NULL, &tv);
    //          retval = select(m_nHighestSocketID+1, &socks, NULL, NULL, NULL);
    if (retval == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x01, "Select failed: %s", strerror(errno) );

      goto sel_mark;
      //dirty, but works!
    }
    if (retval!=0) {
      m_oSync.lock();
      fprintf(stderr,":");

      //       fprintf(stderr,"CTXSocketHandler:: Checking Which Sockets received Data:\n");
      for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++) {
        ret_checkreceived=-1;
        fprintf(stderr,"CTXSocketHandler:: checking socket with SocketID: %d, AcceptID: %d, bLocked:", iter->m_nSocketID, iter->m_nAcceptID);
        if (iter->m_bLocked)
          fprintf(stderr,"true\n");
        else
          fprintf(stderr,"false\n");

        if (iter->m_nAcceptID != -1)
          if (FD_ISSET(iter->m_nAcceptID, &socks) != 0) {
            if (!iter->m_bLocked) {
              iter->m_bLocked=true;
              ret_checkreceived=checkReceiveData(*iter);
            }
          }
        if (iter->m_nAcceptID == -1)
          if (FD_ISSET(iter->m_nSocketID, &socks) != 0) {
            if (!iter->m_bLocked) {
              iter->m_bLocked=true;
              ret_checkreceived=checkReceiveData(*iter);
            }
          }
        if (ret_checkreceived==0) {
          nNotifiedSIFB++;
          //  			    printf("cr: %u\n",nNotifiedSIFB);
        }
      }
      m_oSync.unlock();
      while (nNotifiedSIFB>0) {
        tx_queue_receive(&tx_sockhan_queue, tx_queue_message, TX_WAIT_FOREVER);
        //	          tx_queue_receive(&tx_sockhan_queue, tx_queue_message,10000);
        //	          fprintf(stderr," Queue_Receive");
        nNotifiedSIFB--;
        //	          printf("QR: %u\n",nNotifiedSIFB);              
      }
    }
  }
}

int CTXSocketHandler::openTCPConnection(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  int sock_id;
  char ip[32];

  strlcpy(ip, pa_stConnectionInformation.m_pcConnectionID, 32); 
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));
  fprintf(stderr,"CTXSocketHandler:: Opening TCP-Connection at IP: %s\n",pa_stConnectionInformation.m_pcConnectionID);

  if ((sock_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket Creation failed: %s", strerror(errno));
    return -1;
  }
  // switch between client/server:
  if (pa_stConnectionInformation.m_enServiceType == e_Server) {
    sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);

    if (bind(sock_id, &sock_addr, sizeof(struct sockaddr))!= 0) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket bind() failed: %s", strerror(errno) );
      return -1;
    }
    if (listen(sock_id, 1) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket listen() failed: %s", strerror(errno));
      return -1;
    }
  } else { //e_client
    if (connect(sock_id, &sock_addr, sizeof(struct sockaddr)) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket connect() failed: %s", strerror(errno));
      return -1;
    }
  }
  //    fprintf(stderr,"CTXSocketHandler:: Adding: %s\n",pa_stConnectionInformation.m_pcConnectionID, sock_id);
  calcHighestSocketID(sock_id, 0);
  if (m_eState != e_Run) {
    start();
    m_eState = e_Run;
  }
  return sock_id;
}

int CTXSocketHandler::acceptTCPConnection(SSocketHandlerListEntry &pa_stData) {
  struct sockaddr_in client_addr;
  int retval=-1;
  DEVLOG_DEBUG(0xFD,0x04,"Accept Connection");
  //  int nReceiveID;
  int sin_size = sizeof(struct sockaddr_in);

  if (pa_stData.m_bAcceptedConnection==false) {
    if ((pa_stData.m_nAcceptID
        = accept(pa_stData.m_nSocketID, &client_addr, &sin_size)) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x03, "TCP-Socket accept() failed: %s", strerror(errno));
      return -1;
    } else {
      pa_stData.m_bAcceptedConnection = true;
      pa_stData.m_bLocked = false;
      DEVLOG_INFO(cg_nID_CSocketHandler, 0x03, "CTXSocketHandler:: Accepted TCP connection from IRGENDWAS\n");
      calcHighestSocketID(pa_stData.m_nAcceptID, 0);
      return 1;
    }
  } else {
    DEVLOG_ERROR(cg_nID_CSocketHandler, 0x03, "Connection exists");
    return 2;
  }
}

int CTXSocketHandler::receiveDataOnTCP(SSocketHandlerListEntry &pa_stData) {
  DEVLOG_DEBUG(cg_nID_CSocketHandler,0x04, "Receive Data on TCP");
  int nSockID = (pa_stData.m_bAcceptedConnection) ? pa_stData.m_nAcceptID : pa_stData.m_nSocketID;

  pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize
      =recv(nSockID, (char*)pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize, 0);

  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x04, "TCP-Socket recv() failed: %s", strerror(errno) );

    //TODO: smr: not sure if the following is the correct handling!
    //      pa_stData.m_bAcceptedConnection = false;
    //      pa_stData.m_nAcceptID = -1;
    //      pa_stData.m_bLocked = false;
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState
        = e_ClosedByClient;
    //      close(pa_stData.m_nAcceptID);
    //      calcHighestSocketID(pa_stData.m_nAcceptID, 1);

    return 0;
    //        return -1;
  } else if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize
      == 0) { // Connection Closed by client
    DEVLOG_ERROR(cg_nID_CSocketHandler, 0x04, "CTXSocketHandler::receiveDataOnTCP: connection closed by peer\n");
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState
        = e_ClosedByClient;
    return 0;
  } else {
    //      printf("CTXSocketHandler:: ReceiveDataOnTCP %d bytes\n",pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize);
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState
        =e_ReceivedData;
  }
  return 0;
}

int CTXSocketHandler::sendDataOnTCP(SSocketHandlerListEntry &pa_stData) {
  // This function sends all data in the buffer before it returns!
  int tosend = 0;
  int retval = 0;
  int sock_id = (pa_stData.m_bAcceptedConnection) ? pa_stData.m_nAcceptID : pa_stData.m_nSocketID;
  
  TBYTE *nSendBuf = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData;
  tosend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;
  //    printf("CTXSocketHandler:: SendDataOnTCP: sending %d bytes\n",pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize);
  while (tosend > 0) {
    retval = send(sock_id, (char*)nSendBuf, tosend, 0);
    //      printf("CTXSocketHandler::: send returned %i\n",retval);
    if (retval<=0)
      return retval;
    tosend -= retval;
    nSendBuf += retval;
  }
  return 0;
}

int CTXSocketHandler::openUDPSendPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  int sock_id;
  char ip[32];

  strlcpy(ip, pa_stConnectionInformation.m_pcConnectionID, 31);
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));
  
  DEVLOG_DEBUG_2(cg_nID_CSocketHandler, 0x05, "CTXSocketHandler:: Opening UDP-SendPort at IP: %s\n", ip);

  if ((sock_id = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x05, "UDP-Socket create failed: %s", strerror(errno) );
    return -1;
  }

  //   fprintf(stderr,"CTXSocketHandler:: Adding: %s\n",pa_stConnectionInformation.m_pcConnectionID, sock_id);

  return sock_id;
}

int CTXSocketHandler::openUDPRecvPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  struct ip_mreq mreq;
  int sock_id;
  char ip[32];
  //    printf("begin OpenUDPReceiveConnection\n");

  strlcpy(ip, pa_stConnectionInformation.m_pcConnectionID, 31);
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  // setting up multicast group
  mreq.imr_multiaddr.s_addr = fns_inet_addr(ip);
  mreq.imr_interface.s_addr= htonl(INADDR_ANY);

  DEVLOG_DEBUG_2(cg_nID_CSocketHandler, 0x06, "CTXSocketHandler:: Opening UDP-RecvPort at IP: %s\n", ip);

  if ((sock_id = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP-Socket socket() failed: %s", strerror(errno));
    return -1;
  }

  int value = 1;

  if (bind(sock_id, &sock_addr, sizeof(struct sockaddr))!= 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP-Socket bind() failed: %s", strerror(errno));
    return -1;
  }

  if (setsockopt(sock_id, SOL_SOCKET, SO_REUSEADDR, (char*)&value, sizeof(value)) < 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP-Socket setsockopt() REUSEADDR failed: %s", strerror(errno));
    return -1;
  }

  if (setsockopt(sock_id, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char*)&mreq, sizeof(mreq)) < 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP-Socket setsockopt() OP_ADD_MEMBERSHIP failed: %s", strerror(errno));
    return -1;
  }

  calcHighestSocketID(sock_id, 0);

  return sock_id;
}

int CTXSocketHandler::receiveDataOnUDP(SSocketHandlerListEntry &pa_stData) {
  pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize
      =recvfrom(pa_stData.m_nSocketID, (char*) pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize, 0, 
      NULL, 0);

  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x07, "UDP-Socket recfrom() failed: %s",  strerror(errno));

    // ??????
    pa_stData.m_bLocked=false;
    return (-1);
  } else {

    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState
        = e_ReceivedData;
  }
  return 0;
}

int CTXSocketHandler::sendDataOnUDP(SSocketHandlerListEntry &pa_stData) {
  struct sockaddr_in sock_addr;
  char ip[32];

  strlcpy(ip, pa_stData.m_stConnectionInformation.m_pcConnectionID, 31);
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = fns_inet_addr(ip);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  int tosend = 0;
  int retval = 0;
  TBYTE *nSendBuf = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData;
  tosend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;

  while (tosend > 0) {
    retval = sendto(pa_stData.m_nSocketID, (char*) nSendBuf, tosend, 0, &sock_addr, sizeof(struct sockaddr));
    if (retval<=0)
      return retval;
    tosend -= retval;
    nSendBuf += retval;
  }
  return 0;
}

int CTXSocketHandler::closeTCPConnection(SSocketHandlerListEntry &pa_stData) {

if (pa_stData.m_bAcceptedConnection) {
    closesocket(pa_stData.m_nAcceptID);
    calcHighestSocketID(pa_stData.m_nAcceptID, 1);
    pa_stData.m_nAcceptID = -1;
    pa_stData.m_bAcceptedConnection = false;
  }
  closesocket(pa_stData.m_nSocketID);
  calcHighestSocketID(pa_stData.m_nSocketID, 1);
  pa_stData.m_nSocketID = -1;
  return 0;
}

int CTXSocketHandler::closeUDPPort(SSocketHandlerListEntry &pa_stData) {
  close(pa_stData.m_nSocketID);
  calcHighestSocketID(pa_stData.m_nSocketID, 1);
  pa_stData.m_nSocketID = -1;
  return 0;
}

CTXSocketHandler::CTXSocketHandler() {
  m_nHighestSocketID=0;
  m_nNumSocket = 0;
  m_nExtEvHandID = sm_poDeviceExecution->registerExternalEventHandler(this);

  int status = tx_queue_create(&tx_sockhan_queue, "tx_sockhan_queue", TX_4_ULONG, (VOID *) 0x300000, 2000);
}

CTXSocketHandler::~CTXSocketHandler() {
  //TODO: try to close all connections before exiting.
  int status = tx_queue_delete(&tx_sockhan_queue);

}

int CTXSocketHandler::receivedData(CEventSourceFB *pa_poSocketFB) {
  list<SSocketHandlerListEntry>::iterator iter;

  m_oSync.lock();
  for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSocketFB==pa_poSocketFB) {
      DEVLOG_DEBUG(cg_nID_CSocketHandler,0x01,"receivedData - Queue_Send");
      tx_queue_send(&tx_sockhan_queue, tx_queue_message, TX_NO_WAIT);
      (*iter).m_bLocked = false;
      break;
    }
  m_oSync.unlock();
  return 0;
}

void CTXSocketHandler::enableHandler(void) {
}

void CTXSocketHandler::disableHandler(void) {
}

void CTXSocketHandler::setPriority(int pa_nPriority) {
}

int CTXSocketHandler::getPriority(void) {
}

/* 
 * Reopen TCP-Connection:
 * If Connection was closed by the client, the accept-socket will be closed
 * and the listener-socket will be "opened" (=put in list for select()) again 
 * */
int CTXSocketHandler::reopenSocketFB(CEventSourceFB *pa_poSocketFB, CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {

  list<SSocketHandlerListEntry>::iterator iter;
  int retval=-1;

  //printf("reopenSocketFB\n");
  m_oSync.lock();
  for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSocketFB==pa_poSocketFB) {
      retval=(*iter).m_nSocketID;
      (*iter).m_bAcceptedConnection = false;
      (*iter).m_nAcceptID = -1;
      (*iter).m_bLocked = false;
      closesocket((*iter).m_nAcceptID);
      calcHighestSocketID((*iter).m_nAcceptID, 1);
      break;
    }
  m_oSync.unlock();
  return retval;
}

