/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _TXSOCKETHA_H_
#define _TXSOCKETHA_H_

#include "../sochand.h"
#include "thread.h"

/*! \brief Handler for the socket network interface of NET+OS
 *  \ingroup EXTEVHAND
 *  \ingroup netos_hal
 *  
 * @author SMR
 * @version 20070912/AZ - moved printfs to DEVLOG
 * @version 20061130/IH  - implemented queue for synchronisation and other improvemtents
 * @version 20060522/SMR - first running version of the TXSockethandler
 * @version 20060504/SMR - created this out of the TXSockethandler
 */
class CTXSocketHandler : public CSocketHandler, public CThread{

private:

//! Queue for synchronizing buffer-reading
TX_QUEUE tx_sockhan_queue;
ULONG tx_queue_message[4];

//! The number of active sockets
int m_nNumSocket;

bool m_bSocketListChanged;

/*!\brief This function periodically checks for Traffic on the TCP/UDP Connections
 *
 * \param none
 * \return none
 */
void SelectRoutine(void);

/*!\brief Accepts an incomming Connection (for Client/Server only)
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int acceptTCPConnection(SSocketHandlerListEntry &pa_stData);

/*!\brief Receives Data on an Connection
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int receiveDataOnTCP(SSocketHandlerListEntry &pa_stData);

/*!\brief Sends Data on an Connection
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int sendDataOnTCP(SSocketHandlerListEntry &pa_stDtata);

int openTCPConnection(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);

int openUDPSendPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);

int sendDataOnUDP(SSocketHandlerListEntry &pa_stData);
int openUDPRecvPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);
int receiveDataOnUDP(SSocketHandlerListEntry &pa_stData);

int closeTCPConnection(SSocketHandlerListEntry &pa_stData);
int closeUDPPort(SSocketHandlerListEntry &pa_stData);

int calcHighestSocketID(int pa_nSockId, int pa_nMode);




protected:
 

/*!\brief 
 *
 * \param 
 * \return 
 */
int m_nHighestSocketID;

public:

 
/*!\brief 
 *
 */
CTXSocketHandler();

/*!\brief 
 *
 */
virtual ~CTXSocketHandler();

/*!\brief 
 *
*/
virtual bool run(void);

/*!\brief Enables this event source
 */
  virtual void enableHandler(void);  

/*!\brief Disable this event source
 */  
  virtual void disableHandler(void); 
/*!\brief Sets the priority of the event source  
 * 
 * \param pa_nPriority new priority of the event source
 */

  virtual void setPriority(int pa_nPriority);

/*!\brief Get the current priority of the event source
 * 
 * \return current priority
 */  
  virtual int getPriority(void);

/*!\brief Sends Data on a Socket via SHM
 *
 * \param pa_poSocketFB The FB that is sending
 * \return Status Value
 */
  int SendData(CEventSourceFB *pa_poSocketFB);

/*!\brief Called when a FB has read the SHM
 *
 * \param pa_poSocketFB The FB that is sending
 * \return Status Value
 */
  int receivedData(CEventSourceFB *pa_poSocketFB);

/*!\brief Called when Connection was closed by opposite communication partner
 * 
 * Close Accept-Port and listen to own Socket again
 * 
 * \param pa_poSocketFB The FB that is calling
 * \param pa_stConnectionInformation Connection Information (e.g. Address, Port)
 * \return socketid or -1 on error
 */
  int reopenSocketFB(CEventSourceFB *pa_poSocketFB,SConnectionInformation &pa_stConnectionInformation);
 
};

#endif //_TXSOCHA__H
