/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *  20081003: ih, hegny@acin.tuwien.ac.at - replaced getTicksPerMilliSecond with getTicksPerSecond
 *******************************************************************************/
#ifndef _TXTIMEHA_H_
#define _txTIMEHA_H_

#include <threadx/tx_api.h>
#include "../timerha.h"
#include "../devlog.h"

/*! \ingroup netos_hal 
 * \ingroup EXTEVHAND
 * \brief Timerhandler for NET+OS. 
 *  
 * @author AZ
 * @version 20051009/AZ - Initial implementation
 * @version 20081003/IH - replaced getTicksPerMilliSecond with getTicksPerSecond
 */
class CTXTimerHandler : public CTimerHandler{
  private:
  /*!\brief default time base of the used timer. in seconds.
   */
    static const TINT32 csm_nTicksPerSecond;
  /*!\brief callback function for the system timer
   */  
    static void timerHandlerFunc(ULONG data){
      sm_poFORTETimer->nextTick();
    };
    
    TX_TIMER m_stTimer;
    
  public:
    CTXTimerHandler(); 
    	
  	virtual ~CTXTimerHandler();
  
  /*!\brief Enables this event source
   * 
   */
    virtual void enableHandler(void);  
  /*!\brief Disable this event source
   */  
    virtual void disableHandler(void); 
  /*!\brief Sets the priority of the event source  
   * 
   * \param pa_nPriority new priority of the event source
   */
    virtual void setPriority(int pa_nPriority);
  /*!\brief Get the current priority of the event source
   * 
   * \return current priority
   */  
    virtual int getPriority(void);  
    
  /*! \brief Get the time base of the runtime
   * 
   * \return internal runtime ticks per millisecond
   */ 
    virtual TUINT32 getTicksPerSecond(void){ return csm_nTicksPerSecond; };
    
};

#endif /*PCTIMEHA_H_*/
