/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20080925: alil, alil@users.sourceforge.net - added patch from DResearch Digital Media Systems 
 *******************************************************************************/
#include "pcsochan.h"
#include "../../core/devexec.h"

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <fcntl.h>

CSocketHandler &CSocketHandler::getInstance(void) {
  static CPCSocketHandler oSocketHandler;
  return oSocketHandler;
}

bool CPCSocketHandler::run(void) {
  // deactivating the SIGALRM signal used by the Cygwin-timerhandler
  // check this when porting to other systems
  struct sigaction act;
  memset(&act, 0, sizeof(act));
  sigemptyset (&act.sa_mask);
  act.sa_handler = SIG_IGN;
  sigaction (SIGALRM, &act, NULL);
  m_eState = e_Run;

  int nHighestSocketID=0;

  fd_set socks;
  fd_set socks_save;
  struct timeval tv;
  int retval;
  //std::list<SSocketHandlerListEntry>::iterator iter;

  while(m_bAlive)
  {
    m_oSync.lock();
    tv.tv_sec = 1; //TODO : To be set!
    tv.tv_usec = 1000;

    if (m_bSocketListChanged) {
      m_bSocketListChanged=false;
      FD_ZERO(&socks);
      nHighestSocketID=0;
      for(std::list<SSocketHandlerListEntry>::iterator iter(m_lSocketFBList.begin()); iter != m_lSocketFBList.end(); ++iter) {
        if(iter->m_bAcceptedConnection) {
          FD_SET(iter->m_nAcceptID,&socks);
          if (iter->m_nAcceptID > nHighestSocketID) nHighestSocketID=iter->m_nAcceptID;
        }
        else {
          FD_SET(iter->m_nSocketID,&socks);
          if (iter->m_nSocketID > nHighestSocketID) nHighestSocketID=iter->m_nSocketID;
        }
      }
      socks_save = socks;
    }
    else {
      socks = socks_save;
    }
    m_oSync.unlock();
    retval = select(nHighestSocketID+1, &socks, NULL, NULL, &tv);
    if (retval == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x01, "Select failed: %s", strerror(errno));
    }
    if (retval!=0) {
      m_oSync.lock();
      for(std::list<SSocketHandlerListEntry>::iterator iter(m_lSocketFBList.begin()); iter != m_lSocketFBList.end(); ++iter) {
        if (iter->m_nAcceptID == -1) {
          if (FD_ISSET(iter->m_nSocketID, &socks) != 0) {
            if (!iter->m_bLocked) {
              iter->m_bLocked=true;
              checkReceiveData(*iter);
            }
          }
        }
        else {
          if (FD_ISSET(iter->m_nAcceptID, &socks) != 0) {
            if (!iter->m_bLocked) {
              iter->m_bLocked=true;
              checkReceiveData(*iter);
            }
          }
        }
      }
      m_oSync.unlock();
    }
    else{ 
      fprintf(stderr,".");
    }

  }
  return true; //TODO correct return value
}

int CPCSocketHandler::openTCPConnection(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  int sock_id;
  char ip[32];

  strncpy(ip, pa_stConnectionInformation.m_pcConnectionID, 31);
  ip[31] = '\0';
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  DEVLOG_INFO_2(cg_nID_CSocketHandler, 0x10, "CPCSocketHandler:: Opening TCP-Connection at IP: %s", pa_stConnectionInformation.m_pcConnectionID);

  if ((sock_id = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket Creation failed: %s", strerror(errno));
    return -1;
  }
  // switch between client/server:
  if (pa_stConnectionInformation.m_enServiceType == e_Server) {
    sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    if (bind(sock_id, (struct sockaddr *)&sock_addr, sizeof(struct sockaddr)) != 0) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket bind() failed: %s", strerror(errno) );
      return -1;
    }
    if (listen(sock_id, 1) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket listen() failed: %s", strerror(errno));
      return -1;
    }
  } else { //e_client
    sock_addr.sin_addr.s_addr = inet_addr(ip);
    if (connect(sock_id, (struct sockaddr *)&sock_addr, sizeof(struct sockaddr)) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x02, "TCP-Socket connect failed: %s", strerror(errno));
      return -1;
    }
  }

  m_bSocketListChanged=true;
  if (m_eState != e_Run) {
    start();
    m_eState = e_Run;
  }

  return sock_id;
}

int CPCSocketHandler::acceptTCPConnection(SSocketHandlerListEntry &pa_stData) {
  struct sockaddr_in client_addr;
  //  int nReceiveID;
  socklen_t sin_size = sizeof(struct sockaddr_in);

  if (pa_stData.m_bAcceptedConnection==false) {
    if ((pa_stData.m_nAcceptID = accept(pa_stData.m_nSocketID, (struct sockaddr *)&client_addr, &sin_size)) == -1) {
      DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x03, "TCP-Socket accept failed: %s", strerror(errno));
      return -1;
    } else {
      pa_stData.m_bAcceptedConnection = true;
      m_bSocketListChanged=true;
      return 1;
    }
  } else {
    DEVLOG_DEBUG(cg_nID_CSocketHandler, 0x03, "CPCSocketHandler:: Connection exists");
    return 0;
  }
}

int CPCSocketHandler::receiveDataOnTCP(SSocketHandlerListEntry &pa_stData) {
  DEVLOG_INFO(cg_nID_CSocketHandler, 0x04, "CPCSocketHandler:: Entering ReceiveDataOnTCP\n");

  int nSockID = (pa_stData.m_bAcceptedConnection) ? pa_stData.m_nAcceptID : pa_stData.m_nSocketID;
   
  receveagain: pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize
      =recv(nSockID, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize, 0);

  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize == -1) {
    if (errno == EINTR) {
      DEVLOG_DEBUG(cg_nID_CSocketHandler, 0x04, "CPCSocketHandler:: recv got interrupt / recieving again!!\n");
      goto receveagain;
    }

    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x04, "TCP-Socket recv() failed: %s", strerror(errno));

    //TODO: smr: not sure if the following is the correct handling!
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState = e_ClosedByClient;
    
    //close the accepted connection on error
    closeAcceptedConnection(pa_stData);
    
    return 0;
  } 
  else if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize == 0) { // Connection Closed by client
    DEVLOG_INFO(cg_nID_CSocketHandler, 0x04, "CPCSocketHandler::receiveDataOnTCP: connection closed by peer\n");
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState = e_ClosedByClient;
    
    //close the accepted connection on error
    closeAcceptedConnection(pa_stData);
    
    return 0;
  } 
  else {
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState =e_ReceivedData;
  }
  return 0;
}

int CPCSocketHandler::sendDataOnTCP(SSocketHandlerListEntry &pa_stData) {
  // This function sends all data in the buffer before it returns!
  int tosend = 0;
  int retval = 0;
  int sock_id = (pa_stData.m_bAcceptedConnection) ? pa_stData.m_nAcceptID : pa_stData.m_nSocketID;  
  
  TBYTE *nSendBuf = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData;
  tosend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;
  while (tosend > 0) {
    //TODO: check if open connection (socket might be closed by peer)
    retval = send(sock_id, nSendBuf, tosend, MSG_NOSIGNAL);
    if (retval <= 0)
      return retval;
    tosend -= retval;
    nSendBuf += retval;
  }
  return 0;
}

int CPCSocketHandler::openUDPSendPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  int sock_id;
  char ip[32];

  strncpy(ip, pa_stConnectionInformation.m_pcConnectionID, 31);
  ip[31] = '\0';
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  if ((sock_id = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x05, "UDP-Socket Creation failed: %s", strerror(errno));
    return -1;
  }

  return sock_id;
}

int CPCSocketHandler::openUDPRecvPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation) {
  struct sockaddr_in sock_addr;
  struct ip_mreq mreq;
  int sock_id;
  char ip[32];

  strncpy(ip, pa_stConnectionInformation.m_pcConnectionID, 31);
  ip[31] = '\0';
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  // setting up multicast group
  mreq.imr_multiaddr.s_addr=inet_addr(ip);
  mreq.imr_interface.s_addr= htonl(INADDR_ANY);

  if ((sock_id = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP-Socket Creation failed: %s",  strerror(errno));
    return -1;
  }

  int value = 1;
  if (setsockopt(sock_id, SOL_SOCKET, SO_REUSEADDR, &value, sizeof(value)) < 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "setsockopt() REUSEADDR failed: %s", strerror(errno));
    return -1;
  }

  if (bind(sock_id, (struct sockaddr *)&sock_addr, sizeof(struct sockaddr))!= 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP Bind failed: %s", strerror(errno));
    return -1;
  }

  if (setsockopt(sock_id, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq)) < 0) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x06, "UDP setsockopt, add membership failed: %s", strerror(errno));
    return -1;
  }

  m_bSocketListChanged=true;

  return sock_id;
}

int CPCSocketHandler::receiveDataOnUDP(SSocketHandlerListEntry &pa_stData) {

  pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize
      =recvfrom(pa_stData.m_nSocketID, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_pcData, pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nMaxSize, 0, 
      NULL, 0);

  if (pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_nFilledSize == -1) {
    DEVLOG_ERROR_2(cg_nID_CSocketHandler, 0x07, "UDP error in recvfrom: %s", strerror(errno));
    pa_stData.m_bLocked=false;
    return (-1);
  } else {
    pa_stData.m_stConnectionInformation.m_pstRecvBuffer->m_enConnectionState =e_ReceivedData;
  }
  return 0;
}

int CPCSocketHandler::sendDataOnUDP(SSocketHandlerListEntry &pa_stData) {
  struct sockaddr_in sock_addr;
  char ip[32];

  strncpy(ip, pa_stData.m_stConnectionInformation.m_pcConnectionID, 31);
  ip[31] = '\0';
  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(atoi(splitString(ip)));
  sock_addr.sin_addr.s_addr = inet_addr(ip);
  memset(&(sock_addr.sin_zero), '\0', sizeof(sock_addr.sin_zero));

  int tosend = 0;
  int retval = 0;
  TBYTE *nSendBuf = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_pcData;
  tosend = pa_stData.m_stConnectionInformation.m_pstSendBuffer->m_nFilledSize;

  while (tosend > 0) {
    retval
        = sendto(pa_stData.m_nSocketID, nSendBuf, tosend, 0, (struct sockaddr *)&sock_addr, sizeof(struct sockaddr));
    if (retval<=0)
      return retval;
    tosend -= retval;
    nSendBuf += retval;
  }
  return 0;
}

int CPCSocketHandler::closeTCPConnection(SSocketHandlerListEntry &pa_stData) {
  m_bSocketListChanged=true;
  if (pa_stData.m_bAcceptedConnection) {
    close(pa_stData.m_nAcceptID);
    pa_stData.m_nAcceptID = -1;
    pa_stData.m_bAcceptedConnection = false;
  }
  close(pa_stData.m_nSocketID);
  pa_stData.m_nSocketID = -1;
  return 0;
}

int CPCSocketHandler::closeUDPPort(SSocketHandlerListEntry &pa_stData) {
  close(pa_stData.m_nSocketID);
  pa_stData.m_nSocketID = -1;
  m_bSocketListChanged=true;
  return 0;
}

CPCSocketHandler::CPCSocketHandler() {
  m_nExtEvHandID = sm_poDeviceExecution->registerExternalEventHandler(this);

  //  LOG(cg_nID_CExtEventHandler, 1, E_INFO, 0, "sockethander: started"); 
}

CPCSocketHandler::~CPCSocketHandler() {
  //TODO: try to close all connections before exiting.
}

void CPCSocketHandler::enableHandler(void) {
}

void CPCSocketHandler::disableHandler(void) {
}

void CPCSocketHandler::setPriority(int ) {
}

int CPCSocketHandler::getPriority(void) {
  return 1;
}


void CPCSocketHandler::closeAcceptedConnection(SSocketHandlerListEntry &pa_stData){
  pa_stData.m_bAcceptedConnection = false;
  close(pa_stData.m_nAcceptID);
  pa_stData.m_nAcceptID = -1;
  pa_stData.m_bLocked = false;
  m_bSocketListChanged=true;  
}

