/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _SOCKETHA_H_
#define _SOCKETHA_H_

#include "../sochand.h"
#include "thread.h"

/*! \brief Socket handler for the BSD socket interface
 *  \ingroup EXTEVHAND
 * \ingroup pc_hal 
 *  
 * @author SMR
 * @version 20060427/SMR - various changes
 * @version 20060322/SMR - added Mutexes
 * @version 20060316/SMR - first usable version
 * @version 20051028/SMR - class created
 */
class CPCSocketHandler : public CSocketHandler, public CThread{

private:

//! Socket-List changed -> change sockets, select listens to
bool m_bSocketListChanged;

/*!\brief Accepts an incomming Connection (for Client/Server only)
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int acceptTCPConnection(SSocketHandlerListEntry &pa_stData);

/*!\brief Receives Data on an Connection
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int receiveDataOnTCP(SSocketHandlerListEntry &pa_stData);

/*!\brief Sends Data on an Connection
 *
 * \param Pointer to Data Structure
 * \return Status Value
 */
int sendDataOnTCP(SSocketHandlerListEntry &pa_stDtata);

int openTCPConnection(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);

int openUDPSendPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);

int sendDataOnUDP(SSocketHandlerListEntry &pa_stData);
int openUDPRecvPort(CSocketHandler::SConnectionInformation &pa_stConnectionInformation);
int receiveDataOnUDP(SSocketHandlerListEntry &pa_stData);

int closeTCPConnection(SSocketHandlerListEntry &pa_stData);
int closeUDPPort(SSocketHandlerListEntry &pa_stData);

void closeAcceptedConnection(SSocketHandlerListEntry &pa_stData);

public:
  CPCSocketHandler();
  virtual ~CPCSocketHandler();
  
  virtual bool run(void);

/*!\brief Enables this event source
 */
  virtual void enableHandler(void);  

/*!\brief Disable this event source
 */  
  virtual void disableHandler(void); 
/*!\brief Sets the priority of the event source  
 * 
 * \param pa_nPriority new priority of the event source
 */

  virtual void setPriority(int pa_nPriority);

/*!\brief Get the current priority of the event source
 * 
 * \return current priority
 */  
  virtual int getPriority(void);
};

#endif //_SOCKETHA__H
