/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "thread.h"
#include "../devlog.h"
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>

bool CPCThread::create(long pa_nStackSize) {
  // Get new Thread

  if (m_stThreadID != 0)
    return true;

  pthread_attr_t *tattr= NULL;

  //If the stackSize is not 0 we have to set it
  if (pa_nStackSize) {
    if (pthread_attr_init(tattr)) {
      DEVLOG_ERROR_2(cg_nID_CThread, 0x01, "Error could not get the default thread attributes! %s\n", strerror(errno));
      return false;
    }
    if (pthread_attr_setstacksize(tattr, pa_nStackSize)) {
      DEVLOG_ERROR_2(cg_nID_CThread, 0x01, "Error could not set the stacksize for the thread! %s\n", strerror(errno));
      return false;
    }
    if (pthread_create(&m_stThreadID, tattr, threadFunction, this)) {
      DEVLOG_ERROR_2(cg_nID_CThread,  0x01, "Error could not create the thread! %s\n", strerror(errno));
      return false;
    }
    if (pthread_attr_destroy(tattr)) {
      DEVLOG_ERROR_2(cg_nID_CThread,  0x01, "Error could not free the thread attributes! %s\n", strerror(errno));
      return false;
    }
  } else if (pthread_create(&m_stThreadID, NULL, threadFunction, this)) {
    DEVLOG_ERROR_2(cg_nID_CThread,  0x01, "Error could not create the thread! %s\n", strerror(errno));
    return false;
  }

  return true;
}

bool CPCThread::destroy(void) {
  if ( 0 != m_stThreadID) {
    end();
    m_stThreadID = 0;
  }
  return true;
}

void * CPCThread::threadFunction(void *arguments) {
  // Get pointer to CThread object out of void pointer
  CPCThread *pThread = (CPCThread *) arguments;

  // if pointer is ok
  if (0 != pThread) {
    if (!pThread->run())
      DEVLOG_ERROR(cg_nID_CThread, 0x02, "Error while running Thread!");
    pThread->m_bAlive =false;
    pThread->m_stThreadID = 0;
  } else{
    DEVLOG_ERROR(cg_nID_CThread, 0x02, "pThread pointer is 0!");
  }
  return 0;
}

CPCThread::CPCThread(long pa_nStackSize) {
  m_bSuspended = true;
  m_bAlive = true;
  m_stThreadID = 0;
  m_nStackSize = pa_nStackSize;
  sem_init(&s, 0, 0);
  m_nDeadline = 0;
}

CPCThread::~CPCThread() {
  destroy();
  sem_destroy(&s);
}

bool CPCThread::isAlive(void) {
  return m_bAlive;
}

void CPCThread::setDeadline(TUINT32 pa_nVal) {
  m_nDeadline = pa_nVal;
  //under the posix pthread implemention currently it makes no sense to set any priority. 
  //It will not be considered.  
}

TUINT32 CPCThread::getDeadline(void) {
  return m_nDeadline;
}

void CPCThread::start(void) {
  m_bSuspended = false;
  create(m_nStackSize);
}

void CPCThread::selfSuspend(void) {
  sem_wait(&s);
}

void CPCThread::resumeSelfSuspend(void) {
  sem_post(&s);
}

void CPCThread::end(void) {
  m_bAlive = false;
  resumeSelfSuspend();
  join();
}

void CPCThread::join(void) {
  if (0 == m_stThreadID)
    return;
  pthread_join(m_stThreadID, NULL);
  m_stThreadID = 0;
}
