/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/ 
#include <string.h>
#include <stdlib.h>
#include "serhand.h"

int CSerialHandler::registerSerialFB(SConnectionInformation &pa_stConnectionInformation){
    int nRetVal = -1;
    if (m_eState == e_Error) 
      return -1;
    m_oSync.lock();
    //Check if an FB with the same serial port is already registered with the Serial Handler
    list<SSerialHandlerListEntry>::iterator iter;
    for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++)
      if (!strcmp((*iter).m_stConnectionInformation.m_pcConnectionID, pa_stConnectionInformation.m_pcConnectionID)){
        DEVLOG_DEBUG_2(cg_nID_CExtEventHandler, 0x01, "CSerialHandler:: An FB with port %s is already registered!\n", pa_stConnectionInformation.m_pcConnectionID);
        m_oSync.unlock();
        return -1;
      }        
    switch (pa_stConnectionInformation.m_enServiceType) {
      case e_NoService:
        nRetVal = -1;
        break;
      case e_SerComm:
      case e_Special:
        nRetVal = openSerialPort(pa_stConnectionInformation);
        break;
      default: 
        break;
    } 
    if(-1 != nRetVal){
      SSerialHandlerListEntry stNewSerialFB;
      stNewSerialFB.m_stConnectionInformation = pa_stConnectionInformation;
      stNewSerialFB.m_nSerialID = nRetVal;
      stNewSerialFB.m_bLocked = false;
      m_lSerialFBList.push_back(stNewSerialFB);
    }   
    m_oSync.unlock();
    return nRetVal;
}

int CSerialHandler::unregisterSerialFB(CEventSourceFB *pa_poSerialFB){  
  list<SSerialHandlerListEntry>::iterator iter;
  m_oSync.lock();
  for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++) {
      if ((*iter).m_stConnectionInformation.m_poSerialFB == pa_poSerialFB) {
        SSerialHandlerListEntry &stDataEntry(*iter);
        closeSerialPort(stDataEntry);
        m_lSerialFBList.erase(iter);
        break;
      }
   }
  m_oSync.unlock();
  return 0;
}

int CSerialHandler::receivedData(CEventSourceFB *pa_poSerialFB){
  list<SSerialHandlerListEntry>::iterator iter;
  m_oSync.lock();
  for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSerialFB == pa_poSerialFB){
      (*iter).m_bLocked = false;
      break;
  }
  m_oSync.unlock();
  // TODO: Error check, if not in List!
  return 0;
};

int CSerialHandler::sendData(CEventSourceFB *pa_poSerialFB){
  int nRetVal = -1;  
  list<SSerialHandlerListEntry>::iterator iter;
  m_oSync.lock();
  for(iter = m_lSerialFBList.begin(); iter != m_lSerialFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSerialFB == pa_poSerialFB){
      DEVLOG_DEBUG(cg_nID_CExtEventHandler,0x01,"CSerialHandler:: sendData: Found registered FB\n");
      switch ((*iter).m_stConnectionInformation.m_enServiceType) {
        case e_SerComm:
        case e_Special:
          nRetVal = sendDataOnSerial(*iter);
          break;
        case e_NoService:
          // TODO: this
        default:
          break;
      }
      break;
    }
  m_oSync.unlock();
  return nRetVal;
}

int CSerialHandler::checkReceiveData(SSerialHandlerListEntry &pa_stSerialListEntry){
  int result = -1;
  switch (pa_stSerialListEntry.m_stConnectionInformation.m_enServiceType) {
    case e_SerComm:
    case e_Special:
      result = receiveDataOnSerial(pa_stSerialListEntry);
      break;
    case e_NoService:
    default:
      break;
  }
  return result;
}

unsigned int CSerialHandler::extractPortNumber(const char *pa_Port){
  const unsigned int nPortLength = 4;
  unsigned int nPortNum;
  if((strlen(pa_Port) != nPortLength) || (strncmp(pa_Port,"COM",3) != 0) ||
      (isdigit(pa_Port[3]) == false)){
    DEVLOG_DEBUG_2(cg_nID_CExtEventHandler, 0x01, "CSerialHandler:: Invalid port descriptor %s\n", pa_Port);
    return 0;
  }
  nPortNum = atoi(&pa_Port[3]);
  if((nPortNum <= 9) && (nPortNum > 0))
    return nPortNum;
  else{
    DEVLOG_DEBUG_2(cg_nID_CExtEventHandler, 0x01, "CSerialHandler:: Invalid port number %s\n", pa_Port);
    return 0;
  }
}
