/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include <string.h>
#include "sochand.h"
#include "../core/devexec.h"

#include <stdio.h>

char *CSocketHandler::splitString(char *pa_psIn) {
  int cnt = 0;
  char *runner = pa_psIn;
  static char m_cEmpty = '\0';
  while (*runner != ':') {
    if (cnt++ == 16) {
      DEVLOG_DEBUG_2(cg_nID_CSocketHandler, 0x01, "CPCSocketHandler:: Invalid IP string: %s", pa_psIn);
      return &m_cEmpty;
    }
    runner++;
  }
  *runner = '\0';
  return (++runner);
}

int CSocketHandler::registerSocketFB(
  SConnectionInformation &pa_stConnectionInformation) {
  int nRetVal = -1;

  if (m_eState == e_Error)
    return -1;

  m_oSync.lock();
  // at first we check weather there is an existing connection between the requesting SIFB and the Sockethandler
  list<SSocketHandlerListEntry>::iterator iter;
  for (iter = m_lSocketFBList.begin(); iter != m_lSocketFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSocketFB
        == pa_stConnectionInformation.m_poSocketFB) {
      m_oSync.unlock();
      return -1;
    }

  switch (pa_stConnectionInformation.m_enServiceType) {
    case e_NoService:
      nRetVal = -1;
      break;
    case e_Server:
    case e_Client:
      nRetVal = openTCPConnection(pa_stConnectionInformation);
      break;
    case e_Publisher:
      nRetVal = openUDPSendPort(pa_stConnectionInformation);
      break;
    case e_Subscriber:
      nRetVal = openUDPRecvPort(pa_stConnectionInformation);
      break;
    default:
      break;
  }

  if (-1 != nRetVal) {
    SSocketHandlerListEntry stNewSocketFB;
    stNewSocketFB.m_stConnectionInformation = pa_stConnectionInformation;
    stNewSocketFB.m_nSocketID = nRetVal;
    stNewSocketFB.m_nAcceptID = -1;
    stNewSocketFB.m_bLocked = false;
    stNewSocketFB.m_bAcceptedConnection = false;
    m_lSocketFBList.push_back(stNewSocketFB);
  }

  m_oSync.unlock();
  return nRetVal;
}

int CSocketHandler::unregisterSocketFB(CEventSourceFB *pa_poSocketFB) {
  list<SSocketHandlerListEntry>::iterator iter;

  m_oSync.lock();
  for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++) {
    if ((*iter).m_stConnectionInformation.m_poSocketFB == pa_poSocketFB) {
      SSocketHandlerListEntry &stDataEntry(*iter);
      if ((stDataEntry.m_stConnectionInformation.m_enServiceType == e_Server)
          ||(stDataEntry.m_stConnectionInformation.m_enServiceType == e_Client))
        closeTCPConnection(stDataEntry);
      else
        closeUDPPort(stDataEntry);
      m_lSocketFBList.erase(iter);
      break;
    }
  }

  m_oSync.unlock();
  return 0;
}

int CSocketHandler::receivedData(CEventSourceFB *pa_poSocketFB) {
  list<SSocketHandlerListEntry>::iterator iter;

  m_oSync.lock();

  for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSocketFB==pa_poSocketFB) {
      (*iter).m_bLocked = false;
      break;
    }
  m_oSync.unlock();
  // TODO: Error check, if not in List!
  return 0;
}

int CSocketHandler::sendData(CEventSourceFB *pa_poSocketFB) {
  int nRetVal = -1;
  list<SSocketHandlerListEntry>::iterator iter;
  m_oSync.lock();
  for (iter=m_lSocketFBList.begin(); iter!=m_lSocketFBList.end(); iter++)
    if ((*iter).m_stConnectionInformation.m_poSocketFB==pa_poSocketFB) {
      switch ((*iter).m_stConnectionInformation.m_enServiceType) {
        case e_Server:
        case e_Client:
          nRetVal = sendDataOnTCP(*iter);
          break;
        case e_Publisher:
          nRetVal = sendDataOnUDP(*iter);
          break;
        case e_NoService:
          // TODO: this
        default:
          break;
      }
      break;
    }
  m_oSync.unlock();
  return nRetVal;
}

int CSocketHandler::checkReceiveData(
    SSocketHandlerListEntry &pa_stSocketListEntry) {
  int result = -1;
  switch (pa_stSocketListEntry.m_stConnectionInformation.m_enServiceType) {
    case e_Server:
      if (!pa_stSocketListEntry.m_bAcceptedConnection) {
        DEVLOG_DEBUG(cg_nID_CSocketHandler,0x02,"Start: Accept Connection");
        result = acceptTCPConnection(pa_stSocketListEntry);
        pa_stSocketListEntry.m_bLocked = false;
        return 1;
      } else {
        result = receiveDataOnTCP(pa_stSocketListEntry);
      }
      break;
    case e_Client:
      result = receiveDataOnTCP(pa_stSocketListEntry);
      break;
    case e_Subscriber:
      result = receiveDataOnUDP(pa_stSocketListEntry);
      break;
    case e_NoService:
    default:
      break;
  }
  if (result==0) {
    // informing SIFB if allowed
    if (sm_poDeviceExecution->extEvHandlerIsAllowed(m_nExtEvHandID)) {
      DEVLOG_INFO(cg_nID_CSocketHandler, 0x01, "Starting EC");
      sm_poDeviceExecution->startNewEventChain(pa_stSocketListEntry.m_stConnectionInformation.m_poSocketFB);
    } else {
      //TODO: handle this !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      DEVLOG_DEBUG(cg_nID_CSocketHandler, 0x01, "CTXSocketHandler:: Starting EC NOT ALLOWED !!!!!!!!!!!!!!!1\n");
    }
  }
  return result;
}
