/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *   20080418: IH, hegny@acin.tuwien.ac.at - initial version
 *******************************************************************************/ 
#ifndef _ADAPTER_H_
#define _ADAPTER_H_

#include "funcbloc.h"

class CAdapterConnection;

/*!\ingroup CORE\brief Class for handling adapters.
 * 
 * 
 * @author IH
 * @version 20080418/IH - Initial version
 */
class CAdapter : public CFunctionBlock {
protected:
 bool m_bIsPlug;
 CAdapter *m_poPeer; 
 TIEC_ANYPtr *m_apoLocalDIs;
 CAdapterConnection *m_poAdapterConn;
 
public:
/*!\brief The main constructor for an adapter instance.
 */
  //TODO: think of interface
  CAdapter(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
           const SFBInterfaceSpec *pa_pstInterfaceSpecSocket,
           TEventConnectionPtr *pa_apoEOConns, 
           TDataConnectionPtr *pa_apoDIConns,
           TIEC_ANYPtr *pa_apoDIs,  
           TDataConnectionPtr *pa_apoDOConns,  
           TIEC_ANYPtr *pa_apoDOs,
           const SFBInterfaceSpec *pa_pstInterfaceSpecPlug,
           TEventConnectionPtr *pa_apoEIConns,
           const char *pa_acAdapterInstanceName,
           bool pa_bIsPlug);
  virtual ~CAdapter();
  
  /*!\brief Returns if Adapter instance is a Plug
   */
  bool isPlug(){
    return m_bIsPlug;
  };
  
  /*!\brief Returns if Adapter instance is a Socket
   */
  bool isSocket(){
    return !m_bIsPlug;
  };

  /*! \brief Connects adapter to peer adapter
   * 
   *   With this command the adapter is connected to a complementary adapter instance (peer).
   *   The data output pointers are "redirected" to the data inputs of the peer.
   *   \param pa_poPeer  Pointer to the peer to connect to.
   *   \param pa_poAdConn Pointer to the connecting adapter connection.
   *   \return success of establishment of connection
   */
  bool connect(CAdapter *pa_poPeer, CAdapterConnection *pa_poAdConn);

  /*! \brief Disconnects adapter from peer adapter
     * 
     *   With this command the adapter is disconnected from a complementary adapter instance (peer).
     *   The redirection of data output pointers is removed.
     *   \param pa_poAdConn Pointer to the connecting adapter connection.
     *   \return success of disconnection
     */
  bool disconnect(CAdapterConnection *pa_poAdConn=0);
  
  /*! \brief Returns the compatibility of the adapter to another adapter
     * 
     *   This method will evaluate the compatibility of this adapter to another one. 
     *   Only instances of compatible adapter types may be interconnected.
     *   \param pa_poPeer Pointer to a potential peer, whose compatibility has to be checked.
     *   \return compatibility status
     */
  bool isCompatible(CAdapter *pa_poPeer){
    return (getFBTypeId() == pa_poPeer->getFBTypeId());
  };
  
  virtual void executeEvent(TEventID pa_nEIID){
    if (m_poPeer != 0) {
      m_poPeer->m_poInvokingExecEnv = m_poInvokingExecEnv;
      m_poPeer->sendOutputEvent(pa_nEIID);
    }
  };
};

typedef CAdapter *TAdapterPtr; 

#endif /*_ADAPTER_H_*/
