/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support - initial version
 *******************************************************************************/
#include "adapterconn.h"
#include "funcbloc.h"
#include "adapter.h"

CAdapterConnection::CAdapterConnection(TConnectionID pa_nSourceId) :
  CConnection(pa_nSourceId) {
  m_poPlug=0;
  m_poSocket=0;
}

CAdapterConnection::~CAdapterConnection() {
}

EMGMResponse CAdapterConnection::connect(TConnectionID pa_nDstId,
    CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB,
    const char *pa_sDstInput) {
  EMGMResponse retval = e_NO_SUCH_OBJECT;

  if ((m_poPlug!=0)|| (m_poSocket!=0))
    return e_NOT_READY; //connection already used!

  TAdapterPtr poPlug=pa_roSrcFB.getAdapter(getOutputName());
  TAdapterPtr poSocket=pa_roDstFB.getAdapter(pa_sDstInput);

  if ((poSocket != 0) && (poPlug != 0)) {
    if ((poSocket->isSocket()) &&(poPlug->isPlug())) {
      if (poSocket->isCompatible(poPlug)) {
        if (((poPlug->connect(poSocket, this)) && (poSocket->connect(poPlug,
            this)))) {
          m_poPlug=poPlug;
          m_poSocket=poSocket;
        } else {
          poPlug->disconnect();
          poSocket->disconnect();
          return e_NOT_READY; //connecting of socket or plug did not work
        }
      } else {
        return e_INVALID_DST; //socket and plug are not compatible
      }
    } else
      return e_INVALID_OBJECT; //either src!=plug or dst!=socket
  } else
    return e_NOT_READY; //either plug or socket is still connected

  if (e_RDY != (retval = CConnection::connect(pa_nDstId, pa_roSrcFB,
      pa_roDstFB, pa_sDstInput))) {
    poPlug->disconnect();
    poSocket->disconnect();
    retval = e_INVALID_OBJECT;
    disconnect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput); //reset the connection in a valid state if the connection could not be established.
  }
  return retval;
}

EMGMResponse CAdapterConnection::disconnect(TConnectionID pa_nDstId,
    CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB,
    const char *pa_sDstInput) {
  EMGMResponse retval = CConnection::disconnect(pa_nDstId, pa_roSrcFB,
      pa_roDstFB, pa_sDstInput);

  if (e_RDY == retval) { // the CConnection class didn't respond an error
    if (m_poPlug!=0) {
      m_poPlug->disconnect(this);
      m_poPlug=0;
    }
    if (m_poSocket!=0) {
      m_poSocket->disconnect(this);
      m_poSocket = 0;
    }

  }
  return retval;
}
