/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - adapter support - initial version
 *******************************************************************************/
#ifndef _ADAPTERCONN_H_
#define _ADAPTERCONN_H_

#include "./datatypes/any.h"
#include "conn.h"
//#include "adapter.h"

class CAdapter;

/*! \ingroup CORE\brief Class for handling a adapter connection.
 *  @author IH
 *  @version 20080115/IH - Initial version based on dataconnection
 */
class CAdapterConnection : public CConnection {
private:
  CAdapter *m_poPlug;
  CAdapter *m_poSocket;
public:
  CAdapterConnection(TConnectionID pa_nSourceId);
  virtual ~CAdapterConnection();
  /*! \brief Establishes the connection.
   * 
   *   With this command the connection is created and the FBs are set to appropriate states.
   *   \param pa_sDst     String of the destination of the conneciton format: "FBName.InputName".
   *   \param pa_roSrcFB  Reference to the source FB of this connection.
   *   \param pa_roDstFB  Reference to the destination FB of this connection.
   *   \param pa_sDstInput Name of the input of the destination FB.
   *   \return Can be the following response:
   *     - e_RDY ... creation went well.  
   *     - e_NO_SUCH_OBJECT ... The destination is not a valid input.
   *     - e_INVALID_STATE ... The specified connection already exists.
   */
  virtual EMGMResponse connect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB,
      CFunctionBlock &pa_roDstFB, const char *pa_sDstInput);
  /*! \brief Disconnects the connection.
   * 
   *   With this command the connection is removed and the FBs are set to appropriate states.
   *   \param pa_sDst     String of the destination of the conneciton format: "FBName.InputName".
   *   \param pa_roSrcFB  Reference to the source FB of this connection.
   *   \param pa_roDstFB  Reference to the destination FB of this connection.
   *   \param pa_sDstInput Name of the input of the destination FB.
   *   \return Can be the following response:
   *     - e_RDY ... Deletion went well.  
   *     - e_NO_SUCH_OBJECT ... The destination is not a valid input.
   */
  virtual EMGMResponse disconnect(TConnectionID pa_nDstId,
      CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB,
      const char *pa_sDstInput);
  /*! \brief Set class member variable m_poPlug.
   * 
   *   Set class member variable m_poPlug.
   *   \param pa_poPlug ... pointer to data variable
   */
  void setPlug(CAdapter **pa_poPlug) {
    m_poPlug = *pa_poPlug;
  }
  /*! \brief Get class member variable m_poPlug.
   * 
   *   Get class member variable m_poPlug.
   *   \return pointer to class member variable m_poPlug
   */
  CAdapter **getPlug() {
    return &m_poPlug;
  }
  /*! \brief Set class member variable m_poSocket.
   * 
   *   Set class member variable m_poSocket.
   *   \param pa_poSocket ... pointer to data variable
   */
  void setSocket(CAdapter **pa_poSocket) {
    m_poSocket = *pa_poSocket;
  }
  /*! \brief Get class member variable m_poSocket.
   * 
   *   Get class member variable m_poSocket.
   *   \return pointer to class member variable m_poSocket
   */
  CAdapter **getSocket() {
    return &m_poSocket;
  }

};

typedef CAdapterConnection *TAdapterConnectionPtr;

#endif /*_ADAPTERCONN_H_*/
