/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *   20080709: AZ, alil@ausers.sourceforge.net - Changed type of m_nECCState var to CIEC_UINT in order to allow
 *             he access for read and write commands.
 *******************************************************************************/
#ifndef _BASICFB_H_
#define _BASICFB_H_

#include "funcbloc.h"
#include "./datatypes/forte_uint.h"

/*!\ingroup CORE\brief Class for handling firmware basic function blocks.
 *
 *
 * @author GuGr
 * @version 20060207/AZ - Removed number of EOs in CFunctionBlock-Constructor
 * @version 20060206/AZ - Changed event connection handling
 * @version 20051019/GuGr - First version as basis for interface description
 */
class CBasicFB : public CFunctionBlock {
private:
protected:
  CIEC_UINT m_nECCState; //! the current state of the ecc. start value is 0 = initial state id
  TUINT8 m_nNumInternals;        //!< Number of internal variables (max 254)
  const TFBNameListEntry *m_acInternalNames; //!< List of the internal variable names
  TIEC_ANYPtr *m_apoInternals;                 //!< A list of pointers to the internal variables.


  virtual void executeEvent(TEventID pa_nEIID){
    bool bCleared = executeECCOnce( pa_nEIID);
    while(bCleared)
      bCleared = executeECCOnce( cg_nInvalidEventID);
  };

  /** Execute ECC with given event input id and check if an transition clears. If a
   * transition clears the new state should be entered and the ECC actions should be
   * executed.
   *
   * @param pa_nEIID the event that triggered the execution, if further recursive invocations are triggered the value is cg_nInvalidEventID
   * @param pa_poExecEnv the execution environment (event chain) which triggered the exectuion
   * @return true if an transition cleared, false if not
   */
  virtual bool executeECCOnce(TEventID pa_nEIID) = 0;
public:
/*!\brief The main constructur for a basic function block.
 */
  CBasicFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
           const SFBInterfaceSpec *pa_pstInterfaceSpec,
                 TEventConnectionPtr *pa_apoEOConns,
                 TDataConnectionPtr *pa_apoDIConns,
                 TIEC_ANYPtr *pa_apoDIs,
                 TDataConnectionPtr *pa_apoDOConns,
                 TIEC_ANYPtr *pa_apoDOs,
                 TUINT8 pa_nNumInternals,
          const TFBNameListEntry *pa_acInternalNames,
                 TIEC_ANYPtr *pa_apoInternals);
  virtual ~CBasicFB();

/*!\brief Get the pointer to a variable of the FB.
 *
 * \param pa_acVarName Reference to the varible name.
 * \return Pointer to the variable or 0. The variable may be input, output or in
 *         the case of a BFB an internal var.
 */
  virtual CIEC_ANY* getVar(char *pa_acVarName);

/*!\brief Get the pointer to a internal variable of the basic FB.
 *
 * \param pa_acVarName Reference to the internal variable name.
 * \return Pointer to the internal variable or 0.
 */
  virtual CIEC_ANY *getInternalVar(const char *pa_acInternalName);
};

#endif /*_BASICFB_H_*/
