/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support
 *******************************************************************************/
#include <string.h>
#include "cfb.h"
#include "adapter.h"

CCompositeFB::CCompositeFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
           const SFBInterfaceSpec *pa_pstInterfaceSpec,
                 TEventConnectionPtr *pa_apoEOConns,
                 TDataConnectionPtr *pa_apoDIConns,
                 TCInterface2InternalDataConnectionPtr *pa_apoIf2InDConns,
                 TIEC_ANYPtr *pa_apoDIs,
                 TDataConnectionPtr *pa_apoDOConns,
                 TCInternal2InterfaceDataConnectionPtr *pa_apoIn2IfDConns,
                 TIEC_ANYPtr *pa_apoDOs,
                 int pa_nNumInternalFBs,
                 TFunctionBlockPtr *pa_apoInternalFBs,
                 TUINT8 pa_nNumAdapters,
                 CAdapter **pa_apoAdapters):
    CFunctionBlock(pa_nAppl,
                   pa_poSrcRes,
                   pa_pstInterfaceSpec,
                   pa_apoEOConns,
                   pa_apoDIConns,
                   pa_apoDIs,
                   pa_apoDOConns,
                   pa_apoDOs,
                   pa_nNumAdapters,
                   pa_apoAdapters){
  m_nNumInternalFBs = pa_nNumInternalFBs;
  m_apoInternalFBs = pa_apoInternalFBs;
  m_apoIf2InDConns = pa_apoIf2InDConns;
  m_apoIn2IfDConns = pa_apoIn2IfDConns;
}

CCompositeFB::~CCompositeFB(){
  CTypeLib &roTypeLib(CTypeLib::getInstance());
  for(int i = 0; i < m_nNumInternalFBs; ++i){
    roTypeLib.deleteFB(m_apoInternalFBs[i]);
  }
}

//TODO: gugr: check this func!!!
bool CCompositeFB::connectDI(const unsigned int pa_nDIID, CDataConnection *pa_poDataCon){
  if(pa_nDIID < cm_pstInterfaceSpec->m_nNumDIs){
    m_apoDIConns[pa_nDIID] = pa_poDataCon;                                 //connect to data connection object
      if(m_apoDIs[pa_nDIID] == 0){ //check if m_apoDIs[] is an ANY
        if(pa_poDataCon->getValue() != 0){ //check if pa_poDataCon->getValue is an ANY
          m_apoDIs[pa_nDIID]= pa_poDataCon->getValue()->clone();
          if(0 != m_apoIf2InDConns[pa_nDIID]){                                   //check if internal connected
            m_apoIf2InDConns[pa_nDIID]->setValue(m_apoDIs[pa_nDIID]);
            m_apoIf2InDConns[pa_nDIID]->cloneInputInterfaceValue();
          }
        }
      }
      return true;
    }

  return false;
}

//TODO: gugr: check this func!!!
bool CCompositeFB::connectDO(const unsigned int pa_nDOID, CDataConnection *pa_poDataCon){
  if(pa_nDOID < cm_pstInterfaceSpec->m_nNumDOs){
    m_apoDOConns[pa_nDOID] = pa_poDataCon;
      if(m_apoDOs[pa_nDOID] == 0){ //check if m_apoDOs[] is an ANY
        if(pa_poDataCon->getValue() != 0){ //check if pa_poDataCon->getValue() is not an ANY
          m_apoDOs[pa_nDOID] = pa_poDataCon->getValue()->clone();
          if(0 != m_apoIn2IfDConns[pa_nDOID]){                                   //check if internal connected
            m_apoIn2IfDConns[pa_nDOID]->setValue(m_apoDIs[pa_nDOID]);
            m_apoIn2IfDConns[pa_nDOID]->cloneOutputInterfaceValue();
          }
        }
      }
      return true;
    }
  return false;
}

EMGMResponse CCompositeFB::startManagedObject(void){
  EMGMResponse retval = CFunctionBlock::startManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i = 0; i < m_nNumInternalFBs; ++i){
    	if (m_apoInternalFBs[i])
      	m_apoInternalFBs[i]->startManagedObject();
    }
  }
  return retval;
}

EMGMResponse CCompositeFB::stopManagedObject(void){
  EMGMResponse retval = CFunctionBlock::stopManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i = 0; i < m_nNumInternalFBs; ++i){
    	if (m_apoInternalFBs[i])
      	m_apoInternalFBs[i]->stopManagedObject();
    }
  }
  return retval;
}

EMGMResponse CCompositeFB::killManagedObject(void){
  EMGMResponse retval = CFunctionBlock::killManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i = 0; i < m_nNumInternalFBs; ++i){
    	if (m_apoInternalFBs[i])
      	m_apoInternalFBs[i]->killManagedObject();
    }
  }
  return retval;
}

EMGMResponse CCompositeFB::resetManagedObject(void){
  EMGMResponse retval = CFunctionBlock::resetManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i = 0; i < m_nNumInternalFBs; ++i){
    	if (m_apoInternalFBs[i])
      	m_apoInternalFBs[i]->resetManagedObject();
    }
  }
  return retval;
}

//TODO make this const char * and working with const char *
CIEC_ANY *CCompositeFB::getVar(const char *pa_acVarName){
  CIEC_ANY *poRetVal = CFunctionBlock::getVar(pa_acVarName);
  if (0 == poRetVal){ //The named element is no in- or output var
    char *acBuf = strchr(pa_acVarName, '.'); //find the first dot in the list
    if('\0' != acBuf){ //we found a dot
      TIdentifier acFBName;
      strncpy(acFBName, pa_acVarName, ((acBuf - pa_acVarName) > cg_nIdentifierLength) ? cg_nIdentifierLength : (acBuf - pa_acVarName));
      acFBName[cg_nIdentifierLength] = '\0';
      acBuf++;
      CStringDictionary::TStringId nFBId = CStringDictionary::getInstance().getId(acFBName);
      for(int i = 0; i < m_nNumInternalFBs; ++i){
        if(nFBId == m_apoInternalFBs[i]->getInstanceNameId()){
          poRetVal = m_apoInternalFBs[i]->getVar(acBuf);
          break;
        }
      }
    }
  }
  return poRetVal;
}

CFunctionBlock *CCompositeFB::createContainedFB(const char *pa_acInstanceName, const char *pa_acTypeName, CResource *pa_poRes){
  return CTypeLib::getInstance().createFB(CStringDictionary::getInstance().insert(pa_acInstanceName), CStringDictionary::getInstance().insert(pa_acTypeName), pa_poRes, CStringDictionary::scm_nInvalidStringId);
}
