/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20080424: ih, hegny@acin.tuwien.ac.at - added adapter support
 *     20071220: alil, alil@users.sourceforge.net - added support for reading/writing CFB internal elements
 *******************************************************************************/
#ifndef _CFB_H_
#define _CFB_H_

#include "funcbloc.h"
#include "in2ifeco.h"
#include "if2indco.h"
#include "in2ifdco.h"

/*!\ingroup CORE\brief Class for handling firmware composite function blocks.
 *
 * @author GuGr & tstras
 * @version 20080424/ih - added support for adapters
 * @version 20071220/alil - added support for reading/writing CFB internal elements
 * @version 20060721/tstras - methods for start, stop, kill & reset of internal FBs added
 * @version 20060718/gugr - interface changed to new interface to internal data connection
 * @version 20061707/GuGr & tstras - First version as basis for interface description
 */
class CCompositeFB : public CFunctionBlock {
private:
  TUINT16 m_nNumInternalFBs;
  TFunctionBlockPtr *m_apoInternalFBs;

protected:
  TCInterface2InternalDataConnectionPtr *m_apoIf2InDConns;
  TCInternal2InterfaceDataConnectionPtr *m_apoIn2IfDConns;

  CFunctionBlock *createContainedFB(const char *pa_acInstanceName, const char *pa_acTypeName, CResource *pa_poRes);

public:
/*!\brief The main constructor for a composite function block.
 *
 * The interface of the contructor has been extended from the basic FB's interface with the following values:
 * \param pa_nNumInternalFBs number of FBs contained in the composite FB
 * \param pa_apoInternalFBs list of pointers to the FBs contained in the FB
 * \param pa_acFBNames list of names of the contained FBs
 */
  CCompositeFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
           const SFBInterfaceSpec *pa_pstInterfaceSpec,
                 TEventConnectionPtr *pa_apoEOConns,
                 TDataConnectionPtr *pa_apoDIConns,
                 TCInterface2InternalDataConnectionPtr *pa_apoIf2InDConns,
                 TIEC_ANYPtr *pa_apoDIs,
                 TDataConnectionPtr *pa_apoDOConns,
                 TCInternal2InterfaceDataConnectionPtr *pa_apoIn2IfDConns,
                 TIEC_ANYPtr *pa_apoDOs,
                 int pa_nNumInternalFBs,
                 TFunctionBlockPtr *pa_apoInternalFBs,
                 TUINT8 pa_nNumAdapters = 0,
                 CAdapter **pa_apoAdapters = 0 );
  virtual ~CCompositeFB();

/*!\brief Connects specific data input of a CFB with a specific data connection.
 *
 * \param pa_acDIName Reference to the data input name.
 * \param pa_poDataCon Pointer to the data connection the CFB should be connected to.
 * \return TRUE on success, FALSE if data output not exists or output is already connected.
 */
  virtual bool connectDI(const unsigned int pa_nDIID, CDataConnection *pa_poDataCon);
/*!\brief Connects specific data output of a CFB with a specific data connection.
 *
 * \param pa_acDOName Reference to the data output name.
 * \param pa_poDataCon Pointer to the data connection the CFB should be connected to.
 * \return TRUE on success, FALSE if data output not exists or output is already connected.
 */
  virtual bool connectDO(const unsigned int pa_nDOID, CDataConnection *pa_poDataCon);
  //! implement improved version of get Var for CFBs, allowing to read and write to internal elements
  virtual CIEC_ANY* getVar(const char *pa_acVarName);

  virtual EMGMResponse startManagedObject(void);

  virtual EMGMResponse stopManagedObject(void);

  virtual EMGMResponse killManagedObject(void);

  virtual EMGMResponse resetManagedObject(void);
};

#endif /*_CFB_H_*/
