/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20071029: az, alil@users.sourceforge.net - destrucutor now deletes the m_poValue
 *******************************************************************************/
#include "dataconn.h"
#include "funcbloc.h"


EMGMResponse CDataConnection::connect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput) {
  EMGMResponse retval = e_NO_SUCH_OBJECT;

  unsigned int nInID = pa_roDstFB.getDIID(pa_sDstInput);

  if (nInID == cg_nInvalidDataInputID)  { //gugr: check if DstFB has data input, check if SrcFB has data output done during creation of connection
    return retval;
  }

  CIEC_ANY *poDstInput = pa_roDstFB.getDataInput(nInID);
  bool bCasted = false, bCanConnect = false;

  unsigned int nOutID = pa_roSrcFB.getDOID(getOutputName());

  CIEC_ANY::EDataTypeID srcFBDTID = CIEC_ANY::e_ANY;
  CIEC_ANY::EDataTypeID dstFBDTID = CIEC_ANY::e_ANY;
  if (poDstInput!= 0) {
	  dstFBDTID = poDstInput->getDataTypeID(); //gugr: get the datatype of local FB data input variable object
  }
  if (m_poValue == 0) {
    if (nOutID != cg_nInvalidDataOutputID) {
      CIEC_ANY *poSrcOutput = pa_roSrcFB.getDataOutput(nOutID);

      if (poSrcOutput != 0) {
    	  srcFBDTID = poSrcOutput->getDataTypeID(); //gugr: get the datatype of local FB data output variable object
      }

	  if (srcFBDTID == dstFBDTID) {
		  bCanConnect = true;
		}
	  else {
			if (((srcFBDTID == CIEC_ANY::e_ANY) && (dstFBDTID != CIEC_ANY::e_ANY)) || ((srcFBDTID != CIEC_ANY::e_ANY) && (dstFBDTID == CIEC_ANY::e_ANY))) {
				bCanConnect = true;
			}
			else {
				bool bUp = false, bDown = false, bCross = false;
				CIEC_ANY::isCastable(srcFBDTID, dstFBDTID, bUp, bDown, bCross);
				if (bUp || bDown || bCross) {
					bCanConnect = true;
					bCasted = true;
				}
			}
	  }

      if (! bCanConnect) {
    	  return e_INVALID_OBJECT;
      }

      if (srcFBDTID != CIEC_ANY::e_ANY) {
    	  m_poValue = poSrcOutput->clone();
      }
      else if (dstFBDTID != CIEC_ANY::e_ANY) {
        m_poValue = poDstInput->clone();
      }
      if (0 == m_poValue)
        return e_INVALID_OBJECT;
      pa_roSrcFB.connectDO(nOutID, this);
    }
	}

	srcFBDTID = m_poValue->getDataTypeID();
	if (srcFBDTID != dstFBDTID && srcFBDTID != CIEC_ANY::e_ANY && dstFBDTID != CIEC_ANY::e_ANY) {
		bool bUp = false, bDown = false, bCross = false;
		CIEC_ANY::isCastable(srcFBDTID, dstFBDTID, bUp, bDown, bCross);
		bCasted = (bUp || bDown || bCross);
	}

	m_bCasting |= bCasted;

  if (e_RDY == (retval
      = CConnection::connect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput))) { //gugr: check if destination already exists for this connection
    //now connect the target input
    if (srcFBDTID != dstFBDTID && ! bCasted) {
      if (dstFBDTID != CIEC_ANY::e_ANY) {
        retval = e_INVALID_OBJECT;
        disconnect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput); //az: leave the connection class in a valid sate if the connection could not be established.
      }
    }
    if (e_RDY == retval) {
      pa_roDstFB.connectDI(nInID, this);
    }
  } else { // Clean up
    if (m_lstDestinationIds.empty()) {
      pa_roSrcFB.connectDO(nOutID, 0); //disconnect output
      delete m_poValue;
      m_poValue = 0;
    }
  }

  return retval;
}

EMGMResponse CDataConnection::disconnect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput) {

	unsigned int nInID = pa_roDstFB.getDIID(pa_sDstInput);
	unsigned int nOutID = pa_roDstFB.getDOID(getOutputName());
	EMGMResponse retval = CConnection::disconnect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput);
  if (e_RDY == retval) { // the CConnection class didn't respond an error
    pa_roDstFB.connectDI(nInID, 0);
    if (m_lstDestinationIds.empty()) {
      if (!cg_nInvalidDataOutputID) {
        //we have a dot
        delete m_poValue;
        m_poValue = 0;
        pa_roSrcFB.connectDO(nOutID, 0); //disconnect output
      }
    }
  }
  return retval;
}
