/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20081128: mmm, melik@acin.tuwien.ac.at - initial version
 *******************************************************************************/
#include "any_ele.h"
#include <limits.h>

//9223372036854775807LL => 20 digits with sign

bool CIEC_ANY_ELEMENTARY::toString(string &pa_rsValue){
  EDataTypeID eID = getDataTypeID();
  char cBuffer[21]; //9223372036854775807 => 20 digits with sign + 1 for \0
  unsigned int i = 0;
  bool bSigned = true;
  bool bNegative = false;
  unsigned long long nDivisor = 0;
  long long nSDivisor = 0;
  unsigned long long nUBuffer = 0;
  long long nSBuffer = 0;
  switch (eID)  {
    case e_SINT: {
      nSBuffer = getTINT8();
      nSDivisor = 100;
      break;
   }
    case e_USINT:
    case e_BYTE:  {
      nUBuffer = getTUINT8();
      nDivisor = 100;
      bSigned = false;
      break;
   }
   case e_INT:  {
     nSBuffer = getTINT16();
     nSDivisor = 10000;
     break;
   }
   case e_UINT:
   case e_WORD: {
     nUBuffer = getTUINT16();
     nDivisor = 10000;
     bSigned = false;
     break;
   }
   case e_DINT: {
     nSBuffer = getTINT32();
     nSDivisor = 1000000000L;
     break;
   }
   case e_UDINT:
   case e_DWORD:  {
     nUBuffer = getTUINT32();
     nDivisor = 1000000000L;
     bSigned = false;
     break;
   }
   case e_LINT:  {
     nSBuffer = getTUINT64();
     nSDivisor = 1000000000000000000LL;
     break;
   }
   case e_ULINT:
   case e_LWORD:  {
     nUBuffer = getTUINT64();
     nDivisor = 10000000000000000000ULL;
     bSigned = false;
     break;
   }
    default:  {
     return false;
   }
 }
  if(true == bSigned) {
    if(nSBuffer < 0) {
      bNegative = true;
      cBuffer[i] = '-';
      i++;
    }
    bool bLeadingZeros = true;
    do {
      if(true == bLeadingZeros && 0 == nSBuffer/nSDivisor) {
        nSDivisor /= 10;
        continue;
      }
      bLeadingZeros = false;
      if(true == bNegative) {
        cBuffer[i] = '0' - char(nSBuffer/nSDivisor);
        nSBuffer = nSBuffer + (cBuffer[i] - '0') * nSDivisor;
      }
      else {
        cBuffer[i] = char(nSBuffer/nSDivisor) + '0';
        nSBuffer = nSBuffer - (cBuffer[i] - '0') * nSDivisor;
      }

      nSDivisor /= 10;
      i++;
    }while(nSDivisor > 1);

    if(true == bNegative) {
      cBuffer[i] = '0' - char(nSBuffer/nSDivisor);
    }
    else {
      cBuffer[i] = char(nSBuffer/nSDivisor) + '0';
    }
    cBuffer[i+1] = '\0';
  }
  else {
    bool bLeadingZeros = true;
    do {
      if(0 == nUBuffer/nDivisor && true == bLeadingZeros){
        nDivisor /= 10;
        continue;
      }
      bLeadingZeros = false;
      cBuffer[i] = char(nUBuffer/nDivisor) + '0';
      nUBuffer = nUBuffer - (cBuffer[i] - '0') * nDivisor;
      nDivisor /= 10;
      i++;
    }while(nDivisor > 1);
    cBuffer[i] = char(nUBuffer/nDivisor) + '0';
  }

  cBuffer[i+1] = '\0';
  pa_rsValue.assign(cBuffer);
  return true;
}


bool CIEC_ANY_ELEMENTARY::fromString(const string &pa_rsValue) {
  const char* cBuffer = pa_rsValue.c_str();
  unsigned long long nUUpperBound;
  long long nSUpperBound;
  long long nSLowerBound;
  bool bSigned=true;

  long long nMultiplier = 10;
  if('#' == cBuffer[1])	{
	  switch(cBuffer[0]) {
	    case '2':  {
	      nMultiplier = 2;
	      break;
	    }
	    case '8':  {
	      nMultiplier = 8;
	      break;
	    }
	  }
	  cBuffer = cBuffer + 2;
  }
  if('#' == cBuffer[2] && '1' == cBuffer[0] && '6' == cBuffer[1]) {
    nMultiplier = 16;
    cBuffer = cBuffer + 3;
  }

  EDataTypeID eID = getDataTypeID();
  switch (eID)  {
    case e_SINT:  {
      nSUpperBound = CHAR_MAX;
      nSLowerBound = CHAR_MIN;
      break;
    }
    case e_INT: {
      nSUpperBound = INT_MAX;
      nSLowerBound = INT_MIN;
      break;
    }
    case e_DINT:  {
      nSUpperBound = LONG_MAX;
      nSLowerBound = LONG_MIN;
      break;
    }
    case e_LINT:  {
      nSUpperBound = LONG_LONG_MAX;
      nSLowerBound = LONG_LONG_MIN;
      break;
    }

    case e_USINT:
    case e_BYTE:  {
      nUUpperBound = UCHAR_MAX;
      bSigned=false;
      break;
    }

    case e_UINT:
    case e_WORD:  {
      nUUpperBound = UINT_MAX;
      bSigned=false;
      break;
    }

    case e_UDINT:
    case e_DWORD:  {
      nUUpperBound = ULONG_MAX;
      bSigned=false;
      break;
    }

    case e_ULINT:
    case e_LWORD: {
      nUUpperBound = ULONG_LONG_MAX;
      bSigned=false;
      break;
    }
    default:  {
      return false;
    }
  }

  if(true == bSigned)  {
    bool bNegativeNumber = false;
    long long nValue=0;
    for(;*cBuffer;cBuffer++)  { //Do until '/0'
      if((*cBuffer) != '-') {
        if(*cBuffer >= '0' && *cBuffer <= '9') {
      nValue = nMultiplier*nValue + (*cBuffer - '0');
        }
        else  {
          nValue = nMultiplier*nValue + (toupper(*cBuffer) - 'A') + 10; //for hexadecimal numbers
        }
      }
      else
      {
        bNegativeNumber = true;
      }
    }
    if(nValue <= nSUpperBound && nValue >= nSLowerBound)  {
      if(bNegativeNumber == true)  {
        nValue = nValue*-1;
      }
    setTINT64(nValue);
    return true;
    }
    else  {
      return false;
    }
  }
  else  {
    unsigned long long nValue=0;
    for(;*cBuffer;cBuffer++)  { //Do until '/0'
      if(*cBuffer >= '0' && *cBuffer <= '9') {
        nValue = nMultiplier*nValue + (*cBuffer - '0');
      }
      else  {
        nValue = nMultiplier*nValue + (toupper(*cBuffer) - 'A') + 10; //for hexadecimal numbers
      }
    }
    if(nValue <= nUUpperBound)  {
      setTUINT64(nValue);
      return true;
    }
    else  {
      return false;
    }
  }
  return false;
}
