/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "forte_array.h"

//DEFINE_FIRMWARE_DATATYPE(ARRAY)

CIEC_ARRAY::CIEC_ARRAY(CIEC_ANY& pa_poRefVal) : CIEC_ANY_DERIVED(e_ARRAY) {

  m_nLength           = 0;
  m_poReferenceValue  = &pa_poRefVal;
  m_apoValue            = 0;
}

CIEC_ARRAY::CIEC_ARRAY(TUINT16 pa_nLength, CIEC_ANY *pa_poRefVal) : CIEC_ANY_DERIVED(e_ARRAY) {

  m_nLength           = pa_nLength;
  m_poReferenceValue  = pa_poRefVal;
  m_apoValue            = 0;

  if(m_nLength > 0){
    m_apoValue = new TIEC_ANYPtr[m_nLength];

    for(int i=0; i < m_nLength; ++i)
      m_apoValue[i] = m_poReferenceValue->clone();
  }
}

CIEC_ARRAY::CIEC_ARRAY(const CIEC_ARRAY& pa_roValue) :
            CIEC_ANY_DERIVED(e_ARRAY){

  m_apoValue            = new TIEC_ANYPtr[pa_roValue.m_nLength];
  m_nLength           = pa_roValue.m_nLength;
  if(0 != pa_roValue.m_poReferenceValue)  {
  m_poReferenceValue  = pa_roValue.m_poReferenceValue->clone();
  }

  for(int i=0; i < m_nLength; ++i)
    m_apoValue[i] =  pa_roValue.m_apoValue[i]->clone();
}


CIEC_ARRAY::~CIEC_ARRAY(){

  clear();

  if ( m_poReferenceValue )
	  delete m_poReferenceValue;
}

//TODO: Think on faster method
void CIEC_ARRAY::setValueComplex(const CIEC_ANY* pa_poValue) {

  if ( pa_poValue ) {

        if (pa_poValue->getDataTypeID() == e_ARRAY ) {

        	if ( ((CIEC_ARRAY*)pa_poValue)->m_poReferenceValue) {

            //this->clear();
            //this->resize(((CIEC_ARRAY*)pa_poValue)->m_nLength, ((CIEC_ARRAY*)pa_poValue)->m_poReferenceValue);

            for(int i=0; i < m_nLength; ++i) {

            if ( m_apoValue[i] && ((CIEC_ARRAY*)pa_poValue)->m_apoValue[i] ) {
            	m_apoValue[i] = ((CIEC_ARRAY*)pa_poValue)->m_apoValue[i];
            }
            }
        }
    }
  }
}

void CIEC_ARRAY::resize(TUINT16 size) {

  if ( m_nLength == size ) return;

  clear();

  m_nLength           = size;
  m_apoValue            = new TIEC_ANYPtr[m_nLength];

  for(int i=0; i < m_nLength; ++i)
    m_apoValue[i] = m_poReferenceValue->clone();
}

void CIEC_ARRAY::resize(TUINT16 size, CIEC_ANY *pa_poRefVal) {

  if ( this->m_nLength == size && this->m_poReferenceValue->getDataTypeID() == pa_poRefVal->getDataTypeID() ) return;

  clear();

  this->m_poReferenceValue  = pa_poRefVal;
  this->m_nLength           = size;
  this->m_apoValue            = new TIEC_ANYPtr[m_nLength];

  for(int i=0; i < m_nLength; ++i)
    m_apoValue[i] = m_poReferenceValue->clone();
}

void CIEC_ARRAY::clear() {

  for(int i=0; i < m_nLength; ++i)
    if ( m_apoValue[i] ) delete m_apoValue[i];

  if ( m_apoValue ) delete ( m_apoValue );

  m_nLength = 0;
  m_apoValue  = 0;
}

bool CIEC_ARRAY::fromString(const string& pa_rsValue) {

  if ( pa_rsValue[0] == '[' && pa_rsValue[pa_rsValue.length()-1] == ']' ) {

    unsigned int num_elements = 0;
    unsigned int n1 = 0, n2 = 0;
    unsigned int i;

    /*
     * count elements
     */
    n1 = 1;
		num_elements = (pa_rsValue.length() == 2) ? 0 : 1;
    while ( 1 ) {
      n2 = pa_rsValue.find ( ',', n1 );

      if ( n2 < pa_rsValue.length() ) {

        num_elements++;
        n1 = n2+1;
      }
      else break;
    }

    /*
     * allocate array elements
     */
    m_nLength = num_elements;
    m_apoValue = new TIEC_ANYPtr[m_nLength];

    for(i=0; i < m_nLength; ++i)
      m_apoValue[i] = m_poReferenceValue->clone();

    /*
     * set element values
     */
    n1 = 1;
    for(i=0; i < m_nLength; ++i)
    {
      n2 = pa_rsValue.find ( ',', n1 );

      if ( n2 < pa_rsValue.length() ) {

        string substring = pa_rsValue.substr ( n1, n2-n1 );
        m_apoValue[i]->fromString(substring);

        n1 = n2+1;
      }
      else {
        string substring = pa_rsValue.substr ( n1, pa_rsValue.length()-1-n1 );
        m_apoValue[i]->fromString(substring);
				i++;
				break;
			}
    }

		return (i == m_nLength);
  }
  return false;
}

bool CIEC_ARRAY::toString(string &pa_rsValue) {

  pa_rsValue = "[";

  for(unsigned int i=0; i < m_nLength; ++i) {

    string substr;

    m_apoValue[i]->toString( substr );

    if (i == 0) pa_rsValue.append ( substr );
    else pa_rsValue.append ( "," + substr );
  }

  pa_rsValue.append ( "]" );

  return true;
}
