/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_BOOL_H_
#define _FORTE_BOOL_H_

#include "any_bit.h"

/*!\ingroup COREDTS CIEC_BOOL represents the bool data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081121/mmm - changed constructors and operator = and operator TBOOL8
 *  @version 20070307/az - changed serialization code for supporting also the
 *                         Serialization of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051202/tstras - fromString() & toString() method implemented
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051017/tstras & imuell - Initial version
 */
class CIEC_BOOL : public CIEC_ANY_BIT {
  DECLARE_FIRMWARE_DATATYPE(BOOL)

public:
  CIEC_BOOL() : CIEC_ANY_BIT(e_BOOL){ setTBOOL8(false); }
  explicit CIEC_BOOL(const CIEC_BOOL& pa_rbValue) : CIEC_ANY_BIT(e_BOOL) {
    setValue(pa_rbValue);
  }
  explicit CIEC_BOOL(bool pa_bValue) : CIEC_ANY_BIT(e_BOOL) {
    setTBOOL8(pa_bValue);
  }
  virtual ~CIEC_BOOL() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_BOOL(*this); }

/*! \brief Operator: CIEC_BOOL data type = bool data type
 *
 *   This command implements the assignment operator for the IEC61131 datatype BOOL.
 *   The parameter value is a reference onto a given C++ variable (call by reference).
 *   \param pa_rbValue  Reference for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_BOOL& operator = (bool pa_bValue) {
    setTBOOL8(pa_bValue);
    return *this;
  }
/*! \brief Converts CIEC_BOOL to elementary bool
 *
 *   Conversion operator for converting CIEC_BOOL to elementary bool
 */
  operator bool () const { return getTBOOL8(); };

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering systems.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool toString(string& pa_rsValue);
};

#endif /*_FORTE_BOOL_H_*/
