/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_BYTE_H_
#define _FORTE_BYTE_H_

#include "any_bit.h"

/*!\ingroup COREDTS CIEC_BYTE represents the byte data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - changed constructors and operator = and operator TBYTE
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051114/tstras & imuell - Initial version
 */
class CIEC_BYTE : public CIEC_ANY_BIT {
    DECLARE_FIRMWARE_DATATYPE(BYTE)

public:
  CIEC_BYTE() : CIEC_ANY_BIT(e_BYTE) { };
  explicit CIEC_BYTE(const CIEC_BYTE& pa_roValue) : CIEC_ANY_BIT(e_BYTE) {
    setValue(pa_roValue);
  }
  explicit CIEC_BYTE(TBYTE pa_cValue) : CIEC_ANY_BIT(e_BYTE) {
    setTUINT8(pa_cValue);
  }
  virtual ~CIEC_BYTE() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_BYTE(*this); }

/*! \brief Operator: CIEC_BYTE data type = CIEC_BYTE data type
 *
 *   This command implements the assignment operator for the IEC61131 datatype BYTE.
 *   The parameter value is a reference onto a given C++ variable (call by reference).
 *   \param pa_rcValue  Reference for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_BYTE& operator = (TBYTE pa_rcValue) {
    setTUINT8(pa_rcValue);
    return *this;
  }
/*! \brief Converts CIEC_BYTE to elementary byte
 *
 *   Conversion operator for converting CIEC_BYTE to elementary byte
 */
  operator TBYTE () const { return getTUINT8(); };
};

#endif /*_FORTE_BYTE_H_*/
