/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_DATE_H_
#define _FORTE_DATE_H_

#include "any_dat.h"

/*!\ingroup COREDTS CIEC_DATE represents the time data types according to IEC 61131.
 * @author: unknown
 * @version 20081121/mmm - changed constructors and operator TUINT64, added setValueComplex, removed getValue and setValue
 *
 */

class CIEC_DATE : public CIEC_ANY_DATE {
  DECLARE_FIRMWARE_DATATYPE(DATE)
private:
protected:
public:
  CIEC_DATE() : CIEC_ANY_DATE(e_DATE) { }

  explicit CIEC_DATE(const CIEC_DATE& pa_roValue) : CIEC_ANY_DATE(e_DATE) {
    setValue(pa_roValue);
  }
  explicit CIEC_DATE(TUINT64 pa_nValue) : CIEC_ANY_DATE(e_DATE) {
    setTUINT64(pa_nValue);
  }
  virtual ~CIEC_DATE() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_DATE(*this); }

/*! \brief Operator: CIEC_DATE data type = long data type
 *
 *   This command implements the assignment operator for the C++ datatype long long.
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_nValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_DATE& operator = (TUINT64 pa_nValue) {
    setTUINT64(pa_nValue);
    return *this;
  }
/*! \brief Operator: CIEC_TIME data type operator++
 *
 *   This command implements the increment operator for the IEC61131 datatype TIME.
 *   The parameter value is represented by a C++ variable (call by value).
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_DATE& operator ++ (void) {
    setTUINT64(getTUINT64()+1);
    return *this;
  }
/*! \brief Converts CIEC_SINT to elementary 32 bit integer
 *
 *   Conversion operator for converting CIEC_SINT to elementary 32 bit integer
 */
  operator TUINT64 () const { return getTUINT64(); };

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool toString(string& pa_rsValue);
};

#endif /*_FORTE_DATE_H_*/
