/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_DWORD_H_
#define _FORTE_DWORD_H_

#include "any_bit.h"

/*!\ingroup COREDTS CIEC_DWORD represents the dword data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - changed constructors, operator = and operator TDWORD, removed getValue and setValue
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051116/tstras & imuell - Initial version
 */
class CIEC_DWORD : public CIEC_ANY_BIT {
  DECLARE_FIRMWARE_DATATYPE(DWORD)

public:
  CIEC_DWORD() : CIEC_ANY_BIT(e_DWORD) { }
  explicit CIEC_DWORD(const CIEC_DWORD& pa_roValue) : CIEC_ANY_BIT(e_DWORD){
    setValue(pa_roValue);
  }
  explicit CIEC_DWORD(TDWORD pa_cValue) : CIEC_ANY_BIT(e_DWORD) {
    setTUINT32(pa_cValue);
  }
  virtual ~CIEC_DWORD() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_DWORD(*this); }

/*! \brief Operator: CIEC_DWORD data type = long data type
 *
 *   This command implements the assignment operator for the C++ datatype UNSIGNED LONG
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_cValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_DWORD& operator = (TDWORD pa_cValue) {
    setTUINT32(pa_cValue);
    return *this;
  }
/*! \brief Converts CIEC_DWORD to elementary word
 *
 *   Conversion operator for converting CIEC_DWORD to elementary word
 */
  operator TDWORD () const { return getTUINT32(); };
};

#endif /*_FORTE_DWORD_H_*/
