/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_INT_H_
#define _FORTE_INT_H_

#include "any_int.h"

/*!\ingroup COREDTS CIEC_INT represents the int data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - changed constructors, operator = and operator TUINT16, removed getValue and setValue
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051017/tstras & imuell - Initial version
 */
class CIEC_INT : public CIEC_ANY_INT {
  DECLARE_FIRMWARE_DATATYPE(INT)

public:
  CIEC_INT() : CIEC_ANY_INT(e_INT) { }
  explicit CIEC_INT(const CIEC_INT& pa_roValue) : CIEC_ANY_INT(e_INT)  {
    setValue(pa_roValue);
  }
  explicit CIEC_INT(TINT16 pa_nValue) : CIEC_ANY_INT(e_INT) {
    setTINT16(pa_nValue);
  }
  virtual ~CIEC_INT() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_INT(*this); }

/*! \brief Operator: CIEC_INT data type = short data type
 *
 *   This command implements the assignment operator for the C++ datatype SHORT
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_nValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_INT& operator = (TINT16 pa_nValue) {
    setTINT16(pa_nValue);
    return *this;
  }
/*! \brief Converts CIEC_INT to elementary 16 bit integer
 *
 *   Conversion operator for converting CIEC_INT to elementary 16 bit integer
 */
  operator TINT16 () const { return getTINT16(); };
};

#endif /*_FORTE_INT_H_*/
