/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_REAL_H_
#define _FORTE_REAL_H_

#include "any_rea.h"

/*!\ingroup COREDTS CIEC_REAL represents the real data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081112/mmm - changed deviation from IIEC_ANY_REAL to CIEC_ANY_REAL due to changing class structure
 *  						changed order of declarations to public - protected - due to design rules changes
 *  						corrected spelling in comments; deleted m_fValue, getDataTypeID, getValue and setValue due to changing class structure
 *  @version 20070307/az - changed serialization code for supporting also the
 *                         Serialization of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060707/tstras - bugfix in fromString() method
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051111/tstras & imuell - Initial version
 */
class CIEC_REAL : public CIEC_ANY_REAL {
  DECLARE_FIRMWARE_DATATYPE(REAL)

  public:
  CIEC_REAL() : CIEC_ANY_REAL(e_REAL) { }
  explicit CIEC_REAL(const CIEC_REAL& pa_roValue) : CIEC_ANY_REAL(e_REAL) {
    setValue(pa_roValue);
  }
  explicit CIEC_REAL(TFLOAT pa_fValue) : CIEC_ANY_REAL(e_REAL) {
    setTFLOAT(pa_fValue);
  }
  virtual ~CIEC_REAL() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_REAL(*this); }

/*! \brief Operator: CIEC_REAL data type = float data type
 *
 *   This command implements the assignment operator for the C++ datatype FLOAT.
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_fValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_REAL& operator = (TFLOAT pa_fValue) {
    setTFLOAT(pa_fValue);
    return *this;
  };
/*! \brief Converts CIEC_REAL to elementary 16 bit float
 *
 *   Conversion operator for converting CIEC_REAL to elementary 16 bit float
 */
  operator TFLOAT () const { return getTFLOAT(); };

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successfull - something went wrong!
 *     -  true....conversion was successfull.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successfull - something went wrong!
 *     -  true....conversion was successfull.
 */
  virtual bool toString(string& pa_rsValue);

protected:
private:

};

#endif /*_FORTE_REAL_H_*/
