/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_SINT_H_
#define _FORTE_SINT_H_

#include "any_int.h"

/*!\ingroup COREDTS CIEC_SINT represents the sint data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - changed constructors, operator = and operator TUINT8, removed getValue and setValue,
 *    renamed class from IIEC_SINT to CIEC_SINT due to to changing class structure
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051114/tstras & imuell - Initial version
 */
class CIEC_SINT : public CIEC_ANY_INT {
  DECLARE_FIRMWARE_DATATYPE(SINT)

public:
  CIEC_SINT() : CIEC_ANY_INT(e_SINT) { }
  explicit CIEC_SINT(const CIEC_SINT& pa_roValue): CIEC_ANY_INT(e_SINT)  {
    setValue(pa_roValue);
  }
  explicit CIEC_SINT(TINT8 pa_nValue): CIEC_ANY_INT(e_SINT) {
    setTINT8(pa_nValue);
  }
  virtual ~CIEC_SINT() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_SINT(*this); }

/*! \brief Operator: CIEC_SINT data type = char data type
 *
 *   This command implements the assignment operator for the C++ datatype CHAR
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_nValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_SINT& operator = (TINT8 pa_nValue) {
    setTINT8(pa_nValue);
    return *this;
  }
/*! \brief Converts CIEC_SINT to elementary 8 bit integer
 *
 *   Conversion operator for converting CIEC_SINT to elementary 8 bit integer
 */
  operator TINT8 () const { return getTINT8(); };
};

#endif /*_FORTE_SINT_H_*/
