/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_TIME_H_
#define _FORTE_TIME_H_

#include "any_mag.h"

/*!\ingroup COREDTS CIEC_TIME represents the time data types according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081311/mmm - deleted getDataPtr(), getDataTypeID(); changed IIEC_ANY_STRING to CIEC_ANY_STRING due to class structure change
 *    removed getValue, setValue; added setValueComplex
 *  @version 20070307/az - changed serialization code for supporting also the
 *                         Serialization of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060718/az     - fixed serialize and deserialize to be endian independant
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051017/tstras & imuell - Initial version
 */

class CIEC_TIME : public CIEC_ANY_MAGNITUDE {
  DECLARE_FIRMWARE_DATATYPE(TIME)
private:

protected:
public:
  CIEC_TIME() : CIEC_ANY_MAGNITUDE(e_TIME) { }
  explicit CIEC_TIME(const CIEC_TIME& pa_roValue) : CIEC_ANY_MAGNITUDE(e_TIME) {
    setValue(pa_roValue);
  }
  explicit CIEC_TIME(TINT32 pa_nValue) : CIEC_ANY_MAGNITUDE(e_TIME) {
    setTUINT32(pa_nValue);
  }
  virtual ~CIEC_TIME() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_TIME(*this); }
/*! \brief Get data type id method
 *
 *   With this command the data type of the actual object can be identified.
 *   The data type identifier is unique for each data type within FORTE.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - The data type identification number of the object.
 */

  CIEC_TIME& operator = (TINT32 pa_nValue) {
    setTINT32(pa_nValue);
    return *this;
  }
/*! \brief Operator: CIEC_TIME data type operator++
 *
 *   This command implements the increment operator for the IEC61131 datatype TIME.
 *   The parameter value is represented by a C++ variable (call by value).
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_TIME& operator ++ (void) {
    setTINT32(getTINT32()+1);
    return *this;
  }
/*! \brief Converts CIEC_SINT to elementary 32 bit integer
 *
 *   Conversion operator for converting CIEC_SINT to elementary 32 bit integer
 */
  operator TINT32 () const { return getTINT32(); };

/*! \brief Converts string value to data type value
 *
 *   This command implements a conversion function from IEC61131
 *   data type (string format) to a C++ conform type.
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool fromString(const string& pa_rsValue);
/*! \brief Converts data type value to string
 *
 *   This command implements a conversion function from C++ data type
 *   to IEC61131 conform data type (string format).
 *   This function is necessary for communication with a proper engineering system.
 *   \param pa_rsValue  Reference to the given String
 *   \return Can be the following response:
 *     - false....conversion was not successful - something went wrong!
 *     -  true....conversion was successful.
 */
  virtual bool toString(string& pa_rsValue);
};

#endif /*_MTIME_H_*/
