/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_UDINT_H_
#define _FORTE_UDINT_H_

#include "any_int.h"

/*!\ingroup COREDTS CIEC_UDINT represents the udint data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - removed setValue, getValue, changed operator= and operator TUINT32
 *    changed constructors, renamed class due to class structure change
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051116/tstras & imuell - Initial version
 */
class CIEC_UDINT : public CIEC_ANY_INT {
  DECLARE_FIRMWARE_DATATYPE(UDINT)

public:
  CIEC_UDINT() : CIEC_ANY_INT(e_UDINT) { }
  CIEC_UDINT(const CIEC_UDINT& pa_roValue) : CIEC_ANY_INT(e_UDINT)  {
    setValue(pa_roValue);
  }
  explicit CIEC_UDINT(TUINT32 pa_nValue) : CIEC_ANY_INT(e_UDINT) {
    setTUINT32(pa_nValue);
  }
  virtual ~CIEC_UDINT() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_UDINT(*this); }

/*! \brief Operator: CIEC_UDINT data type = unsigned long data type
 *
 *   This command implements the assignment operator for the C++ datatype UNSIGNED LONG
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_nValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_UDINT& operator = (TUINT32 pa_nValue) {
    setTUINT32(pa_nValue);
    return *this;
  }
/*! \brief Converts CIEC_UDINT to elementary unsigned 32 bit integer
 *
 *   Conversion operator for converting CIEC_UDINT to elementary unsigned 32 bit integer
 */
  operator TUINT32 () const { return getTUINT32(); };
};

#endif /*_FORTE_UDINT_H_*/
