/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _FORTE_UINT_H_
#define _FORTE_UINT_H_

#include "any_int.h"

/*!\ingroup COREDTS CIEC_UINT represents the uint data type according to IEC 61131.
 *  @author tstras & imuell
 *  @version 20081201/mmm - removed toString, fromString
 *  @version 20081121/mmm - removed setValue, getValue, changed operator= and operator TUINT32
 *    changed constructors, renamed class due to class structure change
 *  @version 20070307/az - changed serialisation code for supporting also the
 *                         serialisation of arrays.
 *  @version 20061108/az - added in sertion ito data type library in the typelib
 *  @version 20061107/gebenh - getDataPtr() method added
 *  @version 20060719/tstras - deletion of mathematical operators,
 *                             conversion operator added
 *  @version 20060322/tstras - isFacadeDataType() method added, bugfix for
 *                             handling FACADE data type in setValue() &
 *                             getValue() methods
 *  @version 20060321/tstras - bug for FACADE data type in setValue() &
 *                             getValue() methods fixed
 *  @version 20051205/tstras - clone() method added
 *  @version 20051201/tstras - getDataTypeID() method, setValue(IIEC_ANY*) &
 *                             getValue(IIEC_ANY*) added
 *  @version 20051115/tstras & imuell - Initial version
 */
class CIEC_UINT : public CIEC_ANY_INT {
  DECLARE_FIRMWARE_DATATYPE(UINT)

public:
  CIEC_UINT() : CIEC_ANY_INT (e_UINT) { }
  explicit CIEC_UINT(const CIEC_UINT& pa_roValue) : CIEC_ANY_INT(e_UINT)  {
    setValue(pa_roValue);
  }
  explicit CIEC_UINT(TUINT16 pa_nValue) : CIEC_ANY_INT(e_UINT) {
    setTUINT16(pa_nValue);
  }
  virtual ~CIEC_UINT() {}
/*! \brief Makes a clone of the data type object
 *
 *   With this command a clone object of the actual data type object is created.
 *   This clone object is necessary for establishing data-connections.
 *   \param - No parameters necessary.
 *   \return Can be the following response:
 *     - A clone of the actual data type object.
 */
  virtual CIEC_ANY* clone(void) { return new CIEC_UINT(*this); }

/*! \brief Operator: CIEC_UINT data type = unsigned short data type
 *
 *   This command implements the assignment operator for the C++ datatype UNSIGNED SHORT
 *   The parameter value is represented by a C++ variable (call by value).
 *   \param pa_nValue  Value for assignment.
 *   \return Can be the following response:
 *     - Pointer to given object.
 */
  CIEC_UINT& operator= (TUINT16 pa_nValue) {
    setTUINT16(pa_nValue);
    return *this;
  }
/*! \brief Converts CIEC_UDINT to elementary unsigned 16 bit integer
 *
 *   Conversion operator for converting CIEC_UDINT to elementary unsigned 16 bit integer
 */
  operator TUINT16 () const { return getTUINT16(); };
};

#endif /*_FORTE_UINT_H_*/
