/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "ecet.h"
#include "esfb.h"

const unsigned int CEventChainExecutionThread::scm_nEventListSize = 256; //First try with a size of 20 maybe to small

CEventChainExecutionThread::CEventChainExecutionThread(): CThread(){
 m_pstPoolStart = m_pstEventListStart = getEventChainListPool( scm_nEventListSize, &m_pstEventListEnd);
 m_pstEventListEnd->m_pstNext = m_pstEventListStart; //make it to a ring buffer
 m_pstEventListEnd = m_pstEventListStart; //end equals start means that the buffer is empty
}

bool CEventChainExecutionThread::run(void){
  bool retval = true; //normal exit all went fine.
  m_oListSync.lock();
  while(m_bAlive){ //thread is allowed to execute
    if(m_pstEventListEnd == m_pstEventListStart){
      m_oListSync.unlock();
      selfSuspend();
      m_oListSync.lock();
    }
    else{
      m_oListSync.unlock();
      if(0 != m_pstEventListStart->m_pstEventEntry) //check if we have a valid pointer maybe not realy necessary
        m_pstEventListStart->m_pstEventEntry->m_poFB->receiveInputEvent(
              m_pstEventListStart->m_pstEventEntry->m_nEIID, *this);
      m_pstEventListStart->m_pstEventEntry = 0;   
      m_oListSync.lock();    
      m_pstEventListStart = m_pstEventListStart->m_pstNext;
    }
  }
  m_oListSync.unlock(); // FIX ht: required to be sure that the list is unlocked if the thread is no longer allowed to execute (m_bAlive == false)
  return retval;
}

bool CEventChainExecutionThread::startEventChain(CEventSourceFB *pa_poECStartEventSource){
  m_oListSync.lock();
  addEventEntry(pa_poECStartEventSource->getEventSourceEventEntry());
  m_oListSync.unlock();
  resumeSelfSuspend();
  return true;
}

bool CEventChainExecutionThread::addEventEntry(SEventEntry *pa_poEventToAdd){
  bool bRetVal = false;
  if(m_pstEventListEnd->m_pstNext != m_pstEventListStart){ 
    // there is space left in the list
    m_pstEventListEnd->m_pstEventEntry = pa_poEventToAdd;
    m_pstEventListEnd = m_pstEventListEnd->m_pstNext;
    bRetVal = true;
	}  
	 //TODO raise error message or allocate a bigger list because the list is full
	return bRetVal; 
}

SEventChainEntry *CEventChainExecutionThread::getEventChainListPool(unsigned int pa_nCount, SEventChainEntry **pa_ppstListEnd){
//TODO this is only a simple implemenation better would be some memory pool for this  
  SEventChainEntry *pstRetVal = new SEventChainEntry[pa_nCount];
  
  if(0 != pstRetVal){
    SEventChainEntry *pstRunner = pstRetVal;
    for(unsigned int i = 1; i < pa_nCount; ++i){
      pstRunner->m_pstNext = &(pstRetVal[i]);
      ++pstRunner;
    }
    *pa_ppstListEnd = &(pstRetVal[pa_nCount - 1]);
    (*pa_ppstListEnd)->m_pstNext = 0;
  }
  return pstRetVal;
}
  
void CEventChainExecutionThread::returnEventChainListPool(SEventChainEntry *pa_pstStart){
//TODO this is only a simple implemenation better would be some memory pool for this
  delete[] pa_pstStart;
}

