/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/ 
#ifndef _ECET_H_
#define _ECET_H_

#include <list>
#include "eventconn.h"
#include "event.h"
#include "../arch/active_arch/thread.h"
#include "../arch/active_arch/sync.h"

class CEventSourceFB;
struct SEventEntry; // forward declaration of evnetEntry struct

//!\ingroup CORE Structure for holding one entry in the event chain list (e.g for an event dispatcher)
struct SEventChainEntry{
  SEventEntry *m_pstEventEntry;
  SEventChainEntry *m_pstNext;
};

/*! \ingroup CORE\brief Class for executing one event chain.
 *  
 * 
 * @author AZ
 * @version 20050902/AZ - Initial version as basis for the interface documentation
 */
class CEventChainExecutionThread : public CThread{
private:
  static const unsigned int scm_nEventListSize; //!number of entries the event list will have 

/*! \brief List of input events to deliver.
 * 
 * This list stores the necessary information for all events to deliver that occured within this event chain.
 */ 
  SEventChainEntry *m_pstEventListStart;
  SEventChainEntry *m_pstEventListEnd;
  SEventChainEntry *m_pstPoolStart; //!< a pointer to the memory start got from the pool for returning it back.
 //! SyncObject for protecting the list in regard to several accesses 
  CSyncObject m_oListSync;
  
  /** \brief Get an event chain list pool of given size
  * 
  * if necessary the memory for this list is allocated
  * @param pa_nCount number of elements in the pool
  * @param pa_ppstListEnd a return pointer to the end of the list
  * @return pointer to the start of the list on error 0 is returned
  */
  static SEventChainEntry *getEventChainListPool(unsigned int pa_nCount, SEventChainEntry **pa_ppstListEnd);
  
/** \brief Give back an event chain list Pool previosly got by getEventChainListPool
  *
  * the end of the list is automatically determined when through the next 
  * element in the list enries. 
  * @param pa_pstStart pointer to the start of the list. Needs to be the same pointer as that the has been returned by getEventChainListPool!!!
  */
  static void returnEventChainListPool(SEventChainEntry *pa_pstStart);
  
protected:  

public:
	CEventChainExecutionThread();
	virtual ~CEventChainExecutionThread(){
    returnEventChainListPool( m_pstPoolStart); 
  };
  
/*! \brief Clear the event chain.
 * 
 */  
  virtual void clear(void){
    m_oListSync.lock();
    m_pstEventListEnd = m_pstEventListStart;
    m_oListSync.unlock();
  };

/*! \brief The thread run()-method where the events are sent to the FBs and the FBs are executed in.
 * 
 * If there is an entry in the Event List the event will be delivered and the FB executed.
 * If there is no entry in this list the CEventChainExecutionThread will suspend itself and remove itself from the 
 * active EventChainExecutionlist in CFBNExecution.
 * \return true if thread execution ended successfull 
 */  
  virtual bool run(void);

/*!\brief Start the a new event chain with the given ES as starting FB.
 * 
 * 
 * \param pa_poECStartFB event source of the EC to start
 */
  bool startEventChain(CEventSourceFB *pa_poECStartFB);
/*!\brief Add an new event entry to the event chain
 * 
 * \param pa_poEventToAdd new event entry
 */  
  bool addEventEntry(SEventEntry *pa_poEventToAdd); 
  
  CSyncObject &getListSyncObj(void) { return m_oListSync; };
};

#endif /*ECET_H_*/
