/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "eventconn.h"
#include "ecet.h"
#include "funcbloc.h"

using namespace std;

CEventConnection::~CEventConnection(){
  SECEventListEntry *pstRunner = m_pstDestinationsStart, *pstToDelete;
  while(0 != pstRunner){
    pstToDelete = pstRunner;
    pstRunner = pstRunner->m_pstNext;  
    delete pstToDelete;
  }
}

EMGMResponse CEventConnection::connect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput){
  EMGMResponse retval = e_NO_SUCH_OBJECT; 
  TEventID nEIID;
  if(cg_nInvalidEventID != (nEIID = pa_roDstFB.getEIID(pa_sDstInput))){
    if(e_RDY == (retval = CConnection::connect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput))){
      if(0 == m_pstDestinationsStart){ //this is the first entry in this connection therefore also connect to the source
        const char *acOutName = getOutputName();
        if(NULL != acOutName)
          pa_roSrcFB.connectEO(acOutName, this);    
      }
      SECEventListEntry *pstNewEntry = new SECEventListEntry;
      //TODO check if memory could be allocated
      
      pstNewEntry->m_stEventEntry.m_poFB = &pa_roDstFB;
      pstNewEntry->m_stEventEntry.m_nEIID = nEIID;
      pstNewEntry->m_pstNext = 0;
      if(0 == m_pstDestinationsStart)
        m_pstDestinationsStart = pstNewEntry;
      else{
        SECEventListEntry *pstRunner = m_pstDestinationsStart;
        while(0 != pstRunner->m_pstNext) //find the end of the list cost flash but makes the event connection smaller
          pstRunner = pstRunner->m_pstNext;
        pstRunner->m_pstNext = pstNewEntry;  
      }
    }
  }
  return retval;
}

EMGMResponse  CEventConnection::disconnect(TConnectionID pa_nDstId, CFunctionBlock &pa_roSrcFB, CFunctionBlock &pa_roDstFB, const char *pa_sDstInput){  
  EMGMResponse retval = CConnection::disconnect(pa_nDstId, pa_roSrcFB, pa_roDstFB, pa_sDstInput);
  if(e_RDY == retval){
    // the CConnection class didn't respond an error
    if(0 != m_pstDestinationsStart){
      TEventID nEIID = pa_roDstFB.getEIID(pa_sDstInput);
      SECEventListEntry *pstRunner = m_pstDestinationsStart;
      
      if((pstRunner->m_stEventEntry.m_nEIID == nEIID) && 
         (pstRunner->m_stEventEntry.m_poFB == &pa_roDstFB)){ // we are the first element in the list
        m_pstDestinationsStart = pstRunner->m_pstNext;
        delete pstRunner;  
      }
      else{
        while(0 != pstRunner->m_pstNext){
          if((pstRunner->m_pstNext->m_stEventEntry.m_nEIID == nEIID) &&
             (pstRunner->m_pstNext->m_stEventEntry.m_poFB == &pa_roDstFB)){
            SECEventListEntry  *pstToDelete = pstRunner->m_pstNext;
            pstRunner->m_pstNext = pstToDelete->m_pstNext;   //remove the element from the list
            delete pstToDelete;
            break;
					} else {
						pstRunner = pstRunner->m_pstNext;
					}
				}
      }      
    }
    
    if(0 == m_pstDestinationsStart){ //this is the was the last entry in this connection -> detach from the source
      const char *acOutName = getOutputName();
      if(NULL != acOutName)
        pa_roSrcFB.connectEO(acOutName, 0);    
    }
  }
  return retval;
}

void CEventConnection::triggerEvent(CEventChainExecutionThread &pa_poExecEnv){
  SECEventListEntry *pstRunner = m_pstDestinationsStart;
  pa_poExecEnv.getListSyncObj().lock();
  while(0 != pstRunner){
    pa_poExecEnv.addEventEntry(&(pstRunner->m_stEventEntry));
    pstRunner = pstRunner->m_pstNext;  
  }
  pa_poExecEnv.getListSyncObj().unlock();
}
