/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support
 *******************************************************************************/
#include <string.h>
#include "funcbloc.h"
#include "resource.h"
#include "adapter.h"

CFunctionBlock::CFunctionBlock(CStringDictionary::TStringId pa_nAppl,
    CResource *pa_poSrcRes, const SFBInterfaceSpec *pa_pstInterfaceSpec,
    TEventConnectionPtr *pa_apoEOConns, TDataConnectionPtr *pa_apoDIConns,
    TIEC_ANYPtr *pa_apoDIs, TDataConnectionPtr *pa_apoDOConns,
    TIEC_ANYPtr *pa_apoDOs,
    TUINT8 pa_nNumAdapters, CAdapter **pa_apoAdapters) {
  m_nApplId = pa_nAppl;
  m_poResource = pa_poSrcRes;

  cm_pstInterfaceSpec = pa_pstInterfaceSpec;
  m_apoEOConns = pa_apoEOConns;
  m_apoDIConns = pa_apoDIConns;
  m_apoDIs = pa_apoDIs;
  m_apoDOConns = pa_apoDOConns;
  m_apoDOs = pa_apoDOs;

  m_apoAdapters = pa_apoAdapters;
  m_nNumAdapters = pa_nNumAdapters;

  int i;
	if (m_apoEOConns) {
		for (i = 0; i< cm_pstInterfaceSpec->m_nNumEOs; ++i)
			m_apoEOConns[i] = 0;
	}
	if (m_apoDIConns) {
		for (i = 0; i< cm_pstInterfaceSpec->m_nNumDIs; ++i)
			m_apoDIConns[i] = 0;
	}
	if (m_apoDOConns) {
		for (i = 0; i< cm_pstInterfaceSpec->m_nNumDOs; ++i)
			m_apoDOConns[i] = 0;
	}
}

TEventID CFunctionBlock::getEIID(const char *pa_acEIName) {
  for (int i = 0; i < cm_pstInterfaceSpec->m_nNumEIs; ++i) {
    if (!strcmp(pa_acEIName, cm_pstInterfaceSpec->m_acEINames[i]))
      return i;
  }
  return cg_nInvalidEventID;
}

bool CFunctionBlock::hasEventOutput(const char *pa_acEOName) {
  for (int i = 0; i < cm_pstInterfaceSpec->m_nNumEOs; ++i)
    if (!strcmp(pa_acEOName, cm_pstInterfaceSpec->m_acEONames[i]))
      return true;
  return false;
}

bool CFunctionBlock::connectEO(const char *pa_acEOName,
    CEventConnection *pa_poEventCon) {
  for (int i = 0; i < cm_pstInterfaceSpec->m_nNumEOs; ++i)
    if (!strcmp(pa_acEOName, cm_pstInterfaceSpec->m_acEONames[i])) {
      m_apoEOConns[i] = pa_poEventCon;
      return true;
    }
  return false;
}

unsigned int CFunctionBlock::getDIID(const char *pa_acDIName) const {
	  for (unsigned int i = 0; i < cm_pstInterfaceSpec->m_nNumDIs; ++i)
	    if (!strcmp(pa_acDIName, cm_pstInterfaceSpec->m_acDINames[i])) {
	      return i;
	    }
	  return cg_nInvalidDataInputID;
}

unsigned int CFunctionBlock::getDOID(const char *pa_acDOName) const {
	  for (int i = 0; i < cm_pstInterfaceSpec->m_nNumDOs; ++i)
	    if (!strcmp(pa_acDOName, cm_pstInterfaceSpec->m_acDONames[i])) {
	      return i;
	    }
	  return cg_nInvalidDataOutputID;
}

bool CFunctionBlock::connectDI(unsigned int pa_nDIID, CDataConnection *pa_poDataCon) {
  	if (pa_nDIID!=cg_nInvalidDataInputID && pa_nDIID < cm_pstInterfaceSpec->m_nNumDIs) { //catch invalid ID
  		if (m_apoDIConns[pa_nDIID] != 0) {
  			DEVLOG_ERROR_3(cg_nID_CFunctionBlock, 0x01, "%s cannot connect input data %s to more sources\n", CStringDictionary::getInstance().get(getFBTypeId()), cm_pstInterfaceSpec->m_acDINames[pa_nDIID]);
			}
  		m_apoDIConns[pa_nDIID] = pa_poDataCon;
  		if (m_apoDIs[pa_nDIID] == 0){
  		  	m_apoDIs[pa_nDIID] = pa_poDataCon->getValue()->clone();
  		}
  		return true;
  	}

  return false;
}

bool CFunctionBlock::connectDO(unsigned int pa_nDOID, CDataConnection *pa_poDataCon) {
  	if (pa_nDOID!=cg_nInvalidDataOutputID && pa_nDOID < cm_pstInterfaceSpec->m_nNumDOs) { //catch invalid ID
  		if (m_apoDOConns[pa_nDOID] != 0) {
  			DEVLOG_ERROR_3(cg_nID_CFunctionBlock, 0x01, "%s cannot connect input data %s to more sources\n", CStringDictionary::getInstance().get(getFBTypeId()), cm_pstInterfaceSpec->m_acDONames[pa_nDOID]);
			}
  		m_apoDOConns[pa_nDOID] = pa_poDataCon;
  		if (m_apoDOs[pa_nDOID] == 0){
  			m_apoDOs[pa_nDOID] = pa_poDataCon->getValue()->clone();
  		}
  		return true;
  	}

  return false;
}

CIEC_ANY *CFunctionBlock::getDataOutput(unsigned int pa_nDOID) {
	if (pa_nDOID <= cm_pstInterfaceSpec->m_nNumDOs) {
		return m_apoDOs[pa_nDOID];
	}
	else {
		return 0;
	}
}

CIEC_ANY *CFunctionBlock::getDataInput(unsigned int pa_nDIID) {
      if (pa_nDIID <= cm_pstInterfaceSpec->m_nNumDIs) {
    	  return m_apoDIs[pa_nDIID];
      }
      else {
    	  return 0;
      }
}

CIEC_ANY *CFunctionBlock::getVar(const char *pa_acAdapterName) {
  unsigned int nDIID = getDIID(pa_acAdapterName);

  if (nDIID != cg_nInvalidDataInputID) {
	  return getDataInput(nDIID);
  }
  else {
	  return getDataOutput(getDOID(pa_acAdapterName));
  }
}

CAdapter *CFunctionBlock::getAdapter(const char *pa_acAdapterName) {
  CStringDictionary::TStringId nFBId = CStringDictionary::getInstance().getId(pa_acAdapterName);
  for (int i = 0; i < m_nNumAdapters; ++i){
    if( m_apoAdapters[i]->getInstanceNameId() ==  nFBId){
      return m_apoAdapters[i];
    }
  }
  return 0;
}

void CFunctionBlock::sendOutputEvent(TEventID pa_nEO) {
  //  printf("Function Block sending event: %d (maxid: %d)\n", pa_nEO, cm_pstInterfaceSpec->m_nNumEOs - 1);
  if (pa_nEO < cm_pstInterfaceSpec->m_nNumEOs) {
    //    printf(   "Event name: %s, with index: %d\n", cm_pstInterfaceSpec->m_acEONames[pa_nEO], cm_pstInterfaceSpec->m_anEOWithIndexes[pa_nEO]);
    if (-1 != cm_pstInterfaceSpec->m_anEOWithIndexes[pa_nEO]) {
      const TDataIOID
          *poEOWithStart = &(cm_pstInterfaceSpec->m_anEOWith[cm_pstInterfaceSpec->m_anEOWithIndexes[pa_nEO]]);
      //TODO think on this lock
      m_poResource->m_oResDataConSync.lock();
      for (int i = 0; poEOWithStart[i] != 255; ++i) {
        //printf("Output With. i: %d, DO: %d\n", i, poEOWithStart[i]);
        if (0 != m_apoDOConns[poEOWithStart[i]])
          m_apoDOConns[poEOWithStart[i]]->writeData(m_apoDOs[poEOWithStart[i]]);
      }
      //TODO think on this lock
      m_poResource->m_oResDataConSync.unlock();
    }

    if ( 0 != m_apoEOConns[pa_nEO]) {
      //      printf("triggering events\n");
      m_apoEOConns[pa_nEO]->triggerEvent(*m_poInvokingExecEnv);
    }
  }
  //  printf("finished sendoutputevent\n");
}

bool CFunctionBlock::configureFB(const char *) {
  return true;
}

void CFunctionBlock::receiveInputEvent(TEventID pa_nEIID, CEventChainExecutionThread &pa_poExecEnv) {
  //  printf("Function Block (%s) got event: %d (maxid: %d)\n", getFBType(), pa_nEIID, cm_pstInterfaceSpec->m_nNumEIs - 1);
  if (e_RUNNING == getState()) {
    int i;
    if (pa_nEIID < cm_pstInterfaceSpec->m_nNumEIs) {
      //        printf(   "Event name: %s\n", cm_pstInterfaceSpec->m_acEINames[pa_nEIID]);
      if (-1 != cm_pstInterfaceSpec->m_anEIWithIndexes[pa_nEIID]) {
        const TDataIOID *poEIWithStart = &(cm_pstInterfaceSpec->m_anEIWith[cm_pstInterfaceSpec->m_anEIWithIndexes[pa_nEIID]]);

        //TODO think on this lock
        m_poResource->m_oResDataConSync.lock();
        for (i = 0; poEIWithStart[i] != 255; ++i)
          if (0 != m_apoDIConns[poEIWithStart[i]])
            m_apoDIConns[poEIWithStart[i]]->readData(m_apoDIs[poEIWithStart[i]]);
        //TODO think on this lock
        m_poResource->m_oResDataConSync.unlock();
      }
    }
    m_poInvokingExecEnv = &pa_poExecEnv;
    executeEvent(pa_nEIID);
    //    printf("after execute event\n");
    if (-1 != cm_pstInterfaceSpec->m_anEOWithIndexes[cm_pstInterfaceSpec->m_nNumEOs]) {
      //printf("need to handle outputs without with\n");
      const TDataIOID *poEOWithStart = &(cm_pstInterfaceSpec->m_anEOWith[cm_pstInterfaceSpec->m_anEOWithIndexes[cm_pstInterfaceSpec->m_nNumEOs]]);
      //TODO think on this lock
      m_poResource->m_oResDataConSync.lock();
      for (i = 0; poEOWithStart[i] != 255; ++i)
        if (0 != m_apoDOConns[poEOWithStart[i]])
          m_apoDOConns[poEOWithStart[i]]->writeData(m_apoDOs[poEOWithStart[i]]);
      //TODO think on this lock
      m_poResource->m_oResDataConSync.unlock();
    }

  }
  //  else
  //   printf("  FB not in running state\n");
  //printf("finished receive input event\n");
}

EMGMResponse CFunctionBlock::startManagedObject(void){
  EMGMResponse retval = CManagedObject::startManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i= 0; i < m_nNumAdapters; ++i){
      if(m_apoAdapters[i]){
        m_apoAdapters[i]->startManagedObject();
      }
    }
  }
  return retval;
}

EMGMResponse CFunctionBlock::stopManagedObject(void){
  EMGMResponse retval = CManagedObject::stopManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i= 0; i < m_nNumAdapters; ++i){
      if(m_apoAdapters[i]){
        m_apoAdapters[i]->stopManagedObject();
      }
    }
  }
  return retval;
}

EMGMResponse CFunctionBlock::killManagedObject(void){
  EMGMResponse retval = CManagedObject::killManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i= 0; i < m_nNumAdapters; ++i){
      if(m_apoAdapters[i]){
        m_apoAdapters[i]->killManagedObject();
      }
    }
  }
  return retval;
}

EMGMResponse CFunctionBlock::resetManagedObject(void){
  EMGMResponse retval = CManagedObject::resetManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){
    for(int i= 0; i < m_nNumAdapters; ++i){
      if(m_apoAdapters[i]){
        m_apoAdapters[i]->resetManagedObject();
      }
    }
  }
  return retval;
}
