/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _MANAGEDOBJECT_H_
#define _MANAGEDOBJECT_H_

#include "mgmcmd.h"

/*!\defgroup CORE FORTE Core
 * \brief The FORTE Core contains the core elements of the run-time environment.
 */

/*!\ingroup CORE \brief Class handling the mechanisms for managed objects.
 * 
 * 
 * @author AZ
 * @version 20051108/AZ - inital version supporting all the states defined in IEC 61499 for managed elements
 */
class CManagedObject{
public:
/*!\brief Possible states of a runable object.
 * 
 */
  enum E_FBStates{e_IDLE, e_RUNNING, e_STOPPED, e_KILLED};
private:
/*!\brief Attribute defines if runable object can be deleted by a managemant commando.
 * 
 * Default value is set to true.
 * If the runable object is declared in a device or resource specification it must be set to false.
 */ 
  bool m_bDeletable;
/*!\brief Current state of the runable object.
 * 
 */
  E_FBStates m_enFBState;
protected:
public:
	CManagedObject(){
    m_enFBState = e_IDLE;
    m_bDeletable = true;          
  };
	virtual ~CManagedObject(){};
/*!\brief Get information if the runable object is deletable by a management commando.
 * 
 */    
  bool getDeletable(void) const {return m_bDeletable;};
/*!\brief Set attribute to enable/disable the runable object deletion by a management commando.
 * 
 */
  void setDeletable(const bool &pa_bDelAble){m_bDeletable = pa_bDelAble;};
/*!\brief Return if the runable object is allowed to be deleted now.
 * 
 * This is more complex then the simple deletebale flag as the current state has to be incorporated.
 * \return true if currently all conditions are met to be deletable
 */
  bool isCurrentlyDeleteable(void)const {return ((m_bDeletable)&&((m_enFBState == e_STOPPED)||(m_enFBState == e_KILLED))); };
/*! \brief Start the ManagedObject.
 * 
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */  
  virtual EMGMResponse startManagedObject(void);
/*! \brief Stop the ManagedObject.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */  
  virtual EMGMResponse stopManagedObject(void);
/*! \brief Kill the ManagedO.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */  
  virtual EMGMResponse killManagedObject(void);
/*!\brief Reset the ManagedObject.
 * 
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
  virtual EMGMResponse resetManagedObject(void);  
/*!\brief return the current execution state of the managed object
 */
  E_FBStates getState(void ) const { return m_enFBState; };
};

#endif /*_RUNABLE_H_*/
