/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20080421: ih, hegny@acin.tuwien.ac.at - implemented adapters; clean-up
 *     20071029: az, alil@users.sourceforge.net - enhanced clean-up of connection list and FB list in destructor
 *******************************************************************************/
#include <string.h>
#include "objhand.h"
#include "if2indco.h"
#include "typelib.h"
#include "datatypes/forte_string.h"

EMGMResponse C61499ObjectHandler::executeMGMCommand(SManagementCMD &pa_oCommand){
	EMGMResponse retval = e_INVALID_DST;

  if(CStringDictionary::scm_nInvalidStringId == pa_oCommand.m_nDestination){
    switch(pa_oCommand.m_nCMD){
      case cg_nMGM_CMD_Create_FBType:
        break;
      case cg_nMGM_CMD_Create_FBInstance:
        retval = createFB((pa_oCommand.m_nFirstParam & 0xFFFFFFFF), pa_oCommand.m_nSecondParam, pa_oCommand.m_sAdditionalParams.c_str());
        break;
      case cg_nMGM_CMD_Create_Connection:
        retval = createConnection(pa_oCommand.m_nFirstParam, pa_oCommand.m_nSecondParam);
        break;
      case cg_nMGM_CMD_Delete_FBInstance:
        retval = deleteFB((pa_oCommand.m_nFirstParam & 0xFFFFFFFF));
        break;
      case cg_nMGM_CMD_Delete_Connection:
        retval = deleteConnection(pa_oCommand.m_nFirstParam, pa_oCommand.m_nSecondParam);
        break;
      case cg_nMGM_CMD_Delete_AllFBInstances:
        retval = deleteAllFB();
        break;
      case cg_nMGM_CMD_Start:
        retval = executeStartReq((pa_oCommand.m_nFirstParam & 0xFFFFFFFF));
        break;
      case cg_nMGM_CMD_Stop:
        retval = executeStopReq((pa_oCommand.m_nFirstParam & 0xFFFFFFFF));
        break;
      case cg_nMGM_CMD_Read:
        retval = readValue(pa_oCommand.m_nFirstParam, pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_Write:
        retval = writeValue(pa_oCommand.m_nFirstParam, pa_oCommand.m_sAdditionalParams.c_str());
        break;
      case cg_nMGM_CMD_Kill:
        retval = executeKillReq((pa_oCommand.m_nFirstParam & 0xFFFFFFFF));
        break;
      case cg_nMGM_CMD_QUERY_FB:
        retval = executeQueryReq(pa_oCommand.m_nFirstParam, pa_oCommand.m_nSecondParam, pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_QUERY_Connection:
        retval = executeQueryCon(pa_oCommand.m_nFirstParam, pa_oCommand.m_nSecondParam, pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_QUERY_FBTypes:
        retval = queryFBTypes(pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_QUERY_DTTypes:
        retval = queryDTTypes(pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_QUERY_RESTypes:
        retval = queryRESTypes(pa_oCommand.m_sAdditionalParams);
        break;
      case cg_nMGM_CMD_Reset:
        retval = executeResetReq((pa_oCommand.m_nFirstParam & 0xFFFFFFFF));
        break;
      default:
        retval = e_UNSUPPORTED_CMD;
        break;
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::createFB(CStringDictionary::TStringId pa_nFBNameId, CStringDictionary::TStringId pa_nFBTypeId, const char *pa_acAppl){
  EMGMResponse retval = e_RDY;
  if(0 == getFB(pa_nFBNameId)){ // test if the resource does not contain any FB with the same name

    CFunctionBlock *newFB = createFBObject(pa_nFBNameId, pa_nFBTypeId, CStringDictionary::getInstance().insert(pa_acAppl));
    if(0 != newFB){ //we could create a FB now add it to the list of contained resouces
    	m_lstpoObjectInstanceList.insert(TObjInstanceList::value_type(pa_nFBNameId, newFB));
    }
    else
      retval = m_roTypeLib.getLastError();
  }
  else
    retval = e_INVALID_STATE;  // a FB with this name is already existing in this resource
  return retval;
}

EMGMResponse C61499ObjectHandler::createConnection(TConnectionID pa_nSourceId, TConnectionID pa_nDestId){
  EMGMResponse retval = e_NO_SUCH_OBJECT;

  CFunctionBlock *srcFB = this;
  CStringDictionary::TStringId nFBIdBuf = CConnection::extractFBNameId(pa_nSourceId);
  if(CStringDictionary::scm_nInvalidStringId != nFBIdBuf){
    srcFB = getFB(nFBIdBuf);
  }

  CFunctionBlock *dstFB = getFB(CConnection::extractFBNameId(pa_nDestId));

  CConnection *newConnection = 0;
  if((0 != srcFB) && (0 != dstFB)){ // check if the named fbs are existing in this resource
    newConnection = getConnection(pa_nSourceId); //try to find if there is already a connection with given source
    const char *acInName = CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_nDestId));

    if(0 == newConnection){
      // there is no connection with given source try to create a new one
      const char *acOutName = CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_nSourceId));
      unsigned int nDOID;
      if(srcFB == this){
        nDOID = getDIID(acOutName);
        if(nDOID != cg_nInvalidDataOutputID) {
          CIEC_ANY *poInParam = getDataInput(nDOID);
          newConnection = m_roTypeLib.createInterfaceDataConnection(pa_nSourceId, poInParam);
        }
    }
    else{
      if(srcFB->hasEventOutput(acOutName)){
        newConnection = m_roTypeLib.createEventConnection(pa_nSourceId); // it was an event connection to create
      }
      else{
        nDOID = srcFB->getDOID(acOutName);
        if(nDOID != cg_nInvalidDataOutputID){
          newConnection = m_roTypeLib.createDataConnection(pa_nSourceId);  // it was an data connection to create
        }
        else{
          if (0 != srcFB->getAdapter(acOutName)){
            newConnection = m_roTypeLib.createAdapterConnection(pa_nSourceId);
          }
        }
      }
    }
        //TODO check if the connection could be created and if not write appropriate response
      if(0 != newConnection){
        // we have a valid connection
        retval = newConnection->connect(pa_nDestId, *srcFB, *dstFB, acInName); //establish the connection
        if(e_RDY == retval){
          m_lstpoConnectionList.insert(TConnectionList::value_type(pa_nSourceId, newConnection));
        }
        else { // the connection could not be established
          if(newConnection->isEmpty()){ //Check if it was the only destination this connection had
            if(srcFB == this)
              ((CDataConnection *)newConnection)->setValue(0);
            m_roTypeLib.deleteConnection(newConnection); // if yes delete the connection as it point nowhere
          }
        }
      }
    }
    else { // Connection exists, Dest. added to the Dest.-List
      retval = newConnection->connect(pa_nDestId, *srcFB, *dstFB, acInName); //establish the connection
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::deleteFB(CStringDictionary::TStringId pa_nFBNameId){
  EMGMResponse retval = e_NO_SUCH_OBJECT;

  if(CStringDictionary::scm_nInvalidStringId != pa_nFBNameId){
    TObjInstanceList::iterator itToDel(m_lstpoObjectInstanceList.find(pa_nFBNameId));
    if(itToDel != m_lstpoObjectInstanceList.end()){
      if(itToDel->second->isCurrentlyDeleteable()){
        if(deleteFBObject(itToDel->second)){
          m_lstpoObjectInstanceList.erase(itToDel);
          retval = e_RDY;
        }
        else
          retval = e_INVALID_STATE;
      }
      else
        retval = e_INVALID_STATE;
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::deleteAllFB() {
  //we don't have to check for RMT_RES as the RMT_RES is not managed by an object handler. See RMT_DEV for details

  killManagedObject();

  for(TConnectionList::iterator itRunner(m_lstpoConnectionList.begin());
	    itRunner != m_lstpoConnectionList.end(); ++itRunner){
    m_roTypeLib.deleteConnection(itRunner->second);
  }

  for(TObjInstanceList::iterator itRunner(m_lstpoObjectInstanceList.begin());
      itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
    m_roTypeLib.deleteFB(itRunner->second);
  }
	return e_RDY;
}

EMGMResponse C61499ObjectHandler::deleteConnection(TConnectionID pa_nSourceId, TConnectionID pa_nDestId){
  EMGMResponse retval = e_NO_SUCH_OBJECT;

  CFunctionBlock *srcFB = this;
  CStringDictionary::TStringId nFBIdBuf = CConnection::extractFBNameId(pa_nSourceId);
  if(CStringDictionary::scm_nInvalidStringId != nFBIdBuf){
    srcFB = getFB(nFBIdBuf);
  }

  CFunctionBlock *dstFB = getFB(CConnection::extractFBNameId(pa_nDestId));

  if((0 != srcFB) && (0 != dstFB)){ // check if the named fbs are existing in this resource
    CConnection *poConToDel = getConnection(pa_nSourceId);
    if(0 != poConToDel){
      retval = poConToDel->disconnect(pa_nDestId, *srcFB, *dstFB,
                                      CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_nDestId)));
      if(poConToDel->isEmpty()){

        if(srcFB == this)
          ((CDataConnection *)poConToDel)->setValue(0);
        m_roTypeLib.deleteConnection(poConToDel); // if yes delete the connection as it point nowhere
        m_lstpoConnectionList.erase(pa_nSourceId);
      }
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::writeValue(TConnectionID pa_nDestId, const char *pa_acValue){
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;

  CFunctionBlock *poFB = this;
  CStringDictionary::TStringId nFBIdBuf = CConnection::extractFBNameId(pa_nDestId);
  if(CStringDictionary::scm_nInvalidStringId != nFBIdBuf){
    poFB = getFB(nFBIdBuf);
  }

  CIEC_ANY *var = 0;
  if (0 != poFB) {
    var = poFB->getVar(CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_nDestId)));
  }
  if (var != 0) {
    if (pa_acValue != 0) {
      if(var->fromString(pa_acValue)){
        eRetVal = e_RDY;
      }
      else{
        eRetVal = e_BAD_PARAMS;
      }
    }
    else{
      eRetVal = e_BAD_PARAMS; // 0 is not supported in the fromString method
    }
  }
  return eRetVal;
}

EMGMResponse C61499ObjectHandler::readValue(TConnectionID pa_nSourceId, string &pa_sValue){
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;

  CFunctionBlock *poFB = this;
  CStringDictionary::TStringId nFBIdBuf = CConnection::extractFBNameId(pa_nSourceId);
  if(CStringDictionary::scm_nInvalidStringId != nFBIdBuf){
    poFB = getFB(nFBIdBuf);
  }

  CIEC_ANY *var = 0;
  if (0 != poFB) {
    var = poFB->getVar(CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_nSourceId)));
  }

  if(0 != var) {
    if(var->toString(pa_sValue)) {
      eRetVal = e_RDY;
    }
    else {
      eRetVal = e_INVALID_OBJECT;
    }
  }
  return eRetVal;
}

EMGMResponse C61499ObjectHandler::executeQueryReq(TConnectionID pa_nSourceId, TConnectionID &pa_nDestId, string &pa_sResponse){	
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;
  TObjInstanceList::iterator itRunner;

  if(CStringDictionary::scm_nInvalidStringId == pa_nSourceId) {
    if(CStringDictionary::scm_nInvalidStringId == pa_nDestId){
      //fetch all fb instnacenames of all types and return them
      for(itRunner = m_lstpoObjectInstanceList.begin();
          itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
        if (itRunner->second->getApplId() != CStringDictionary::scm_nInvalidStringId) {
          pa_sResponse += CStringDictionary::getInstance().get(itRunner->second->getApplId());
          pa_sResponse += ".";
        }
        pa_sResponse += itRunner->second->getInstanceName();
        pa_sResponse += ",";
      }

      if(pa_sResponse[pa_sResponse.size()-1]==',') {
        pa_sResponse = pa_sResponse.substr(0, pa_sResponse.size()-1);
      }
      eRetVal = e_RDY;
    } else {
      //fetch all fb instnacenames of the defined type and return them
      for(itRunner = m_lstpoObjectInstanceList.begin();
          itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
        if(itRunner->second->getFBTypeId() == pa_nDestId){
          if (itRunner->second->getApplId() != CStringDictionary::scm_nInvalidStringId) {
            pa_sResponse += CStringDictionary::getInstance().get(itRunner->second->getApplId());
            pa_sResponse += ".";
          }
          pa_sResponse += itRunner->second->getInstanceName();
          pa_sResponse += ",";
        }
      }

      if(pa_sResponse[pa_sResponse.size()-1]== ',') {
        pa_sResponse = pa_sResponse.substr(0, pa_sResponse.size()-1);
      }
      eRetVal = e_RDY;
     }
  } else { // Query whether FB instancename exists + FB Type
    if(CStringDictionary::scm_nInvalidStringId == pa_nDestId){
      const char *acSourceName = CStringDictionary::getInstance().get(pa_nSourceId);
      for(itRunner = m_lstpoObjectInstanceList.begin();
                itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
        if (itRunner->second->getApplId() != CStringDictionary::scm_nInvalidStringId) {
          pa_sResponse = CStringDictionary::getInstance().get(itRunner->second->getApplId());
          pa_sResponse += ".";
        }
        pa_sResponse += itRunner->second->getInstanceName();
        if (!strcmp(pa_sResponse.c_str(), acSourceName)) {
          pa_nDestId = itRunner->second->getFBTypeId();
          eRetVal = e_RDY;
          break;
        }
        pa_sResponse.clear();
      }

      if(eRetVal != e_RDY) {
        pa_sResponse = "";
        eRetVal = e_NO_SUCH_OBJECT;
      }
    }
  }
  return eRetVal;
}
EMGMResponse C61499ObjectHandler::executeQueryCon(TConnectionID pa_nSourceId, TConnectionID pa_nDestId, string &pa_sResponse) {
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;

  if((CStringDictionary::scm_nInvalidStringId == pa_nSourceId) &&
      (CStringDictionary::scm_nInvalidStringId == pa_nDestId)){ // both parameters are *

    for(TConnectionList::iterator itRunner(m_lstpoConnectionList.begin());
        itRunner != m_lstpoConnectionList.end(); ++itRunner){

      pa_sResponse += CStringDictionary::getInstance().get(itRunner->second->getSourceId());
      std::set<TConnectionID>::iterator end(itRunner->second->getDestinationIds()->end());
      for(std::set<TConnectionID>::iterator itDestRunner(itRunner->second->getDestinationIds()->begin()); 
          itDestRunner != end; ++itDestRunner){
        pa_sResponse += "-";
        pa_sResponse += CStringDictionary::getInstance().get(*itDestRunner);
      }
      pa_sResponse += ",";
    }
    if(pa_sResponse[pa_sResponse.size()-1]== ',') {
      pa_sResponse = pa_sResponse.substr(0, pa_sResponse.size()-1);
    }
    eRetVal = e_RDY;
  }
  else{
    // TODO check for a better implementation
    if((CStringDictionary::scm_nInvalidStringId == pa_nSourceId) &&
       (CStringDictionary::scm_nInvalidStringId != pa_nDestId)){ // only first parameter is *

      string temp;
      for(TConnectionList::iterator itRunner(m_lstpoConnectionList.begin());
          itRunner != m_lstpoConnectionList.end(); ++itRunner){
        temp = CStringDictionary::getInstance().get(itRunner->second->getSourceId());

        std::set<TConnectionID>::iterator end(itRunner->second->getDestinationIds()->end());
        for(std::set<TConnectionID>::iterator itDestRunner(itRunner->second->getDestinationIds()->begin()); 
            itDestRunner != end; ++itDestRunner){
          if ((*itDestRunner == pa_nDestId) &&
              (0 == strstr(pa_sResponse.c_str(), temp.c_str()))) {
            pa_sResponse += temp;
            pa_sResponse += "-";
            pa_sResponse += CStringDictionary::getInstance().get(*itDestRunner);
            pa_sResponse += ",";
            break;
          }
        }
      }
      if(pa_sResponse[pa_sResponse.size() - 1]== ',') {
        pa_sResponse = pa_sResponse.substr(0, pa_sResponse.size() - 1);
      }
      eRetVal = e_RDY;
    }
    else{
      if((CStringDictionary::scm_nInvalidStringId != pa_nSourceId) &&
         (CStringDictionary::scm_nInvalidStringId == pa_nDestId)){ // only second parameter is *

        for(TConnectionList::iterator itRunner(m_lstpoConnectionList.begin());
            itRunner != m_lstpoConnectionList.end(); ++itRunner){
          if(itRunner->second->getSourceId() == pa_nSourceId) {
            pa_sResponse += CStringDictionary::getInstance().get(itRunner->second->getSourceId());

            std::set<TConnectionID>::iterator end(itRunner->second->getDestinationIds()->end());
            for(std::set<TConnectionID>::iterator itDestRunner(itRunner->second->getDestinationIds()->begin()); 
                itDestRunner != end; ++itDestRunner){
              pa_sResponse += "-";
              pa_sResponse += CStringDictionary::getInstance().get(*itDestRunner);
            }
            break; // all destinations of this connection are included
          }
        }
        if(pa_sResponse[pa_sResponse.size()-1]== ',') {
          pa_sResponse = pa_sResponse.substr(0, pa_sResponse.size()-1);
        }
        eRetVal = e_RDY;
      }
    }
  }
  return eRetVal;
}

EMGMResponse C61499ObjectHandler::executeStartReq(CStringDictionary::TStringId pa_nFBNameId){
  EMGMResponse retval;

  if(CStringDictionary::scm_nInvalidStringId == pa_nFBNameId){
    retval = startManagedObject();
  }
  else{
    CFunctionBlock *fb = getFB(pa_nFBNameId);
    if(0 != fb)
      retval = fb->startManagedObject();
    else
      retval = e_NO_SUCH_OBJECT;
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::executeStopReq(CStringDictionary::TStringId pa_nFBNameId){
  EMGMResponse retval;
  if(CStringDictionary::scm_nInvalidStringId == pa_nFBNameId){
    retval = stopManagedObject();
  }
  else{
    CFunctionBlock *fb = getFB(pa_nFBNameId);
    if(0 != fb)
      retval = fb->stopManagedObject();
    else
      retval = e_NO_SUCH_OBJECT;
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::executeKillReq(CStringDictionary::TStringId pa_nFBNameId){
  EMGMResponse retval;
  if(CStringDictionary::scm_nInvalidStringId == pa_nFBNameId){
    retval = killManagedObject();
  }
  else{
    CFunctionBlock *fb = getFB(pa_nFBNameId);
    if(0 != fb)
      retval = fb->killManagedObject();
    else
      retval = e_NO_SUCH_OBJECT;
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::executeResetReq(CStringDictionary::TStringId pa_nFBNameId){
  EMGMResponse retval;

  if(CStringDictionary::scm_nInvalidStringId == pa_nFBNameId){
    retval = resetManagedObject();
  }
  else{
    CFunctionBlock *fb = getFB(pa_nFBNameId);
    if(0 != fb)
      retval = fb->resetManagedObject();
    else
      retval = e_NO_SUCH_OBJECT;
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::startManagedObject(void){
  EMGMResponse retval = CManagedObject::startManagedObject();
  if((e_RDY == retval)||(e_RUNNING == getState())){ // second part handles the self transition in the run state
    // on start read the input connections and set the input vars
    for(int i = 0; i < cm_pstInterfaceSpec->m_nNumDIs; ++i)
      if(0 != m_apoDIConns[i])
        m_apoDIConns[i]->readData( m_apoDIs[i]);
    for(TObjInstanceList::iterator itRunner(m_lstpoObjectInstanceList.begin());
          itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
      if((e_RDY != (retval = itRunner->second->startManagedObject()))&&(e_RUNNING != itRunner->second->getState())){ // second part handles the self transition in the run state
        break;
      }
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::stopManagedObject(void){
  EMGMResponse retval = CManagedObject::stopManagedObject();
  if(e_RDY == retval){
    for(TObjInstanceList::iterator itRunner(m_lstpoObjectInstanceList.begin());
          itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
      if(e_RDY != (retval = itRunner->second->stopManagedObject())){
        break;
      }
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::killManagedObject(void){
  EMGMResponse retval = CManagedObject::killManagedObject();
  if(e_RDY == retval){
    for(TObjInstanceList::iterator itRunner(m_lstpoObjectInstanceList.begin());
              itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
      if(e_RDY != (retval = itRunner->second->killManagedObject())){
        break;
      }
    }
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::resetManagedObject(void){
  EMGMResponse retval = CManagedObject::resetManagedObject();
  if(e_RDY == retval){
    for(TObjInstanceList::iterator itRunner(m_lstpoObjectInstanceList.begin());
              itRunner != m_lstpoObjectInstanceList.end(); ++itRunner){
      if(e_RDY != (retval = itRunner->second->resetManagedObject())){
        break;
      }
    }
  }
  return retval;
}

CFunctionBlock *C61499ObjectHandler::getFB(CStringDictionary::TStringId pa_nFBInstanceNameId){
  CFunctionBlock *retval = 0;

  if(CStringDictionary::scm_nInvalidStringId != pa_nFBInstanceNameId){
    TObjInstanceList::iterator itFB(m_lstpoObjectInstanceList.find(pa_nFBInstanceNameId));
    if(itFB != m_lstpoObjectInstanceList.end()){
      retval = itFB->second;
    }
  }

  return retval;
}

CConnection *C61499ObjectHandler::getConnection(TConnectionID pa_nSourceId){
  CConnection *retval = 0;

  TConnectionList::iterator itConn(m_lstpoConnectionList.find(pa_nSourceId));
  if(itConn != m_lstpoConnectionList.end()){
    retval = itConn->second;
  }
  return retval;
}

EMGMResponse C61499ObjectHandler::queryFBTypes(string &pa_sValue){
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;
  CTypeLib &roTypeLib(CTypeLib::getInstance());
  CTypeLib::CTypeEntry *poToFirst = roTypeLib.getFBLibStart();
  if (poToFirst != 0) {
    pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    while(poToFirst->m_poNext != 0) {
      poToFirst = poToFirst->m_poNext;
      pa_sValue += ",";
      pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    }
    eRetVal = e_RDY;
  }
  return eRetVal;
}
EMGMResponse C61499ObjectHandler::queryRESTypes(string &pa_sValue){
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;
  CTypeLib &roTypeLib(CTypeLib::getInstance());
  CTypeLib::CTypeEntry *poToFirst = roTypeLib.getResLibStart();
  if (poToFirst != 0) {
    pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    while(poToFirst->m_poNext != 0) {
      poToFirst = poToFirst->m_poNext;
      pa_sValue += ",";
      pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    }
    eRetVal = e_RDY;
  }
  return eRetVal;
}
EMGMResponse C61499ObjectHandler::queryDTTypes(string &pa_sValue){
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;
  CTypeLib &roTypeLib(CTypeLib::getInstance());
  CTypeLib::CTypeEntry *poToFirst = roTypeLib.getDTLibStart();
  if (poToFirst != 0) {
    pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    while(poToFirst->m_poNext != 0) {
      poToFirst = poToFirst->m_poNext;
      pa_sValue += ",";
      pa_sValue += CStringDictionary::getInstance().get(((CTypeLib::CTypeEntry *)poToFirst)->getTypeNameId());
    }
    eRetVal = e_RDY;
  }
  return eRetVal;
}

EMGMResponse C61499ObjectHandler::queryTypeVersion(TUINT32 pa_nSource, string &pa_sValue) {
  EMGMResponse eRetVal = e_NO_SUCH_OBJECT;
  return eRetVal;
}
