/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _RESOURCE_H_
#define _RESOURCE_H_

#include <list>
#include "objhand.h"
#include "device.h"
#include "ecet.h"

/*! \ingroup CORE\brief Base class for all resources handling the reconfiguration management within this 
 * resource and the background execuiton of event chains.
 *  
 * @author AZ
 * @version 20070912/AZ - added code for sampling data input connections on startManagedObject()
 * @version 20070912/AZ - removed getResType() same functionality is provided by getFBType()
 * @version 20070309/HT - added the parameter "pa_acAppl" to createFBObject (it contains the Name of the Application, which the FB belongs to)  
 * @version 20060719/AZ - changed base class of CResource to C61499ObjectHandler to unify management commands
 * @version 20060719/AZ - changed base class of CResource to CFunctionBlock in order to support more easily parameter inputs and their connections to internal FBs.
 * @version 20060619/AZ - added sync object for sycnronizing all dataconnection within one resource
 * @version 20060329/smr - changed startManagedObject(); moved start of execution to end 
 * @version 20060124/AZ+smr - fixed bug in startManagedObject()
 * @version 20051108/AZ - changed baseclass to CManagedObject
 * @version 20051104/AZ - implemented create/ delete connection, write and read FB parameters
 * @version 20051022/AZ - added support for an own eventchainexecution thread per resource 
 * @version 20051016/AZ - implemented reconfiguration commands for create FB-Instance, start, stop
 * @version 20050902/AZ - Initial version as basis for the interface documentation
 */
class CResource : public C61499ObjectHandler{

private:
/*! \brief The device the resource is contained in.
 * 
 *  This refereence helps the resource to access means provided by the device.
 */  
  CDevice &m_roDevice;
/*!\brief The event chain exeuction of background (low priority) event chains started within this resource 
 */
  CEventChainExecutionThread m_oResourceEventExecution;

protected:  
  virtual void executeEvent(TEventID ) {};
  
  virtual CFunctionBlock *createFBObject(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nFBTypeId, CStringDictionary::TStringId pa_nApplId){
    return m_roTypeLib.createFB(pa_nInstanceNameId, pa_nFBTypeId, this, pa_nApplId);
  };
  
  virtual bool deleteFBObject(CFunctionBlock *pa_poFBObj){
    return m_roTypeLib.deleteFB(pa_poFBObj);
  };
public:
  CSyncObject m_oResDataConSync; 
/*! \brief The main contructor for a resource.
 * 
 * The resource can only be generated with a given device the resource is contained. A resource can not exist outside of an device.
 *  \param pa_roDevice the device the resource is contained in
 */  
	CResource(CDevice &pa_roDevice, const SFBInterfaceSpec *pa_pstInterfaceSpec,
                     TDataConnectionPtr *pa_apoDIConns, TIEC_ANYPtr *pa_apoDIs) : C61499ObjectHandler(0, 0, pa_pstInterfaceSpec, 0, pa_apoDIConns, pa_apoDIs, 0, 0),
    m_roDevice(pa_roDevice) {
  };
	virtual ~CResource() {};

   
/*! \brief Get the device the resource is contained in
 */  
  CDevice &getDevice(void) {return m_roDevice; };
 
  CEventChainExecutionThread *getResourceEventExecution(void) { return &m_oResourceEventExecution; };
  
  virtual EMGMResponse startManagedObject(void);
  virtual EMGMResponse stopManagedObject(void); 
  virtual EMGMResponse killManagedObject(void); 
};

#endif /*RESOURCE_H_*/
