/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20081216: mmm - initial version
 *******************************************************************************/

#include "CITASerDeser.h"

/*Initialization csm_aDataTags array
 *
 */
const TBYTE CITASerDeser::csm_aDataTags[][2] = {
    //!< {Tag, Size of data + tag size}; Size == 255 means unknown
    {e_APPLICATION + e_PRIMITIVE + e_ANY_TAG, 255},
    {e_APPLICATION + e_PRIMITIVE + e_BOOL_TAG, 1},
    {e_APPLICATION + e_PRIMITIVE + e_SINT_TAG, 2},
    {e_APPLICATION + e_PRIMITIVE + e_INT_TAG, 3},
    {e_APPLICATION + e_PRIMITIVE + e_DINT_TAG, 5},
    {e_APPLICATION + e_PRIMITIVE + e_LINT_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_USINT_TAG, 2},
    {e_APPLICATION + e_PRIMITIVE + e_UINT_TAG, 3},
    {e_APPLICATION + e_PRIMITIVE + e_UDINT_TAG, 5},
    {e_APPLICATION + e_PRIMITIVE + e_ULINT_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_BYTE_TAG, 2},
    {e_APPLICATION + e_PRIMITIVE + e_WORD_TAG, 3},
    {e_APPLICATION + e_PRIMITIVE + e_DWORD_TAG, 5},
    {e_APPLICATION + e_PRIMITIVE + e_LWORD_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_REAL_TAG, 5},
    {e_APPLICATION + e_PRIMITIVE + e_LREAL_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_DATE_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_TIME_OF_DAY_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_DATE_AND_TIME_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_TIME_TAG, 9},
    {e_APPLICATION + e_PRIMITIVE + e_STRING_TAG, 255},
    {e_APPLICATION + e_PRIMITIVE + e_WSTRING_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_DerivedData_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_DirectlyDerivedData_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_EnumeratedData_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_SubrangeData_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_ARRAY_TAG, 255},
    {e_APPLICATION + e_CONSTRUCTED + e_STRUCT_TAG, 255}
};

CITASerDeser::CITASerDeser() {
  // TODO Auto-generated constructor stub
}

CITASerDeser::~CITASerDeser() {
  // TODO Auto-generated destructor stub
}

int CITASerDeser::serialize(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData)  {
  int nRetVal = -1;
  if(0 < pa_nStreamSize){
    nRetVal = serializeTag(pa_pcBytes, pa_pCIECData);
    if(nRetVal > 0){
      if(nRetVal > pa_nStreamSize)
        nRetVal = 0;
      else
        nRetVal = serializeValue(pa_pcBytes + 1, pa_nStreamSize, pa_pCIECData, nRetVal);
    }
  }
  return nRetVal;
}

int CITASerDeser::deserialize(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData)  {
  int nRetVal = -1;
  if(0 < pa_nStreamSize){
    if(e_APPLICATION == (pa_pcBytes[0] & e_APPLICATION)) {
      if(deserializeTag(pa_pcBytes, pa_pCIECData) > 0){
        if((nRetVal = deserializeValue(pa_pcBytes + 1, pa_nStreamSize -1, pa_pCIECData))>= 0)
          nRetVal += 1;
      }
      else{
        //TODO:mmm - Ask for this!
        //DEVLOG_ERROR(cg_nID_CIEC_ANY, 0x01, "Datatype error\n");
      }
    }
  }
  return nRetVal;
}

int CITASerDeser::serializeValue(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData, unsigned int nSize) {
  CIEC_ANY::EDataTypeID eDataType = pa_pCIECData->getDataTypeID();

  if(CIEC_ANY::e_BOOL < eDataType && eDataType <= CIEC_ANY::e_DATE_AND_TIME)  {
      //Simple Datatypes except bool can be handled the same way
    TBYTE* DataPtr = pa_pCIECData->getDataPtr();
    int nValueSize =  nSize -1; //Length of the tag

#ifdef FORTE_LITTLE_ENDIAN
    for (unsigned int i = 0; i < nValueSize;i++){
        pa_pcBytes[(nValueSize-1)-i] = DataPtr[i];
    }
#endif

#ifdef FORTE_BIG_ENDIAN
    if(CIEC_ANY::e_REAL != eDataType) {
      for (unsigned int i=0; i<nValueSize;i++){
#ifdef FORTE_USE_64BIT_DATATYPES
        pa_pcBytes[(nValueSize-1)-i] = DataPtr[(sizeof(TUINT64)-1)-i];
#else
        pa_pcBytes[(nValueSize-1)-i] = DataPtr[(sizeof(TUINT32)-1)-i];
#endif
        }
    }
    else  {
      for (unsigned int i = 0; i < nValueSize; i++){
      pa_pcBytes[(nValueSize - 1) - i] = DataPtr[(sizeof(TFLOAT) - 1) - i];
      }
    }

#endif
  return nSize;
  }
  else  {
    switch(eDataType){
      case CIEC_ANY::e_BOOL:  {
        if(!((CIEC_BOOL*)pa_pCIECData)->operator bool())  {
            //data of bool is encoded in the tag; if CIEC_BOOL == false => Tag must be e_APPLICATION + e_PRIMITIVE
          pa_pcBytes[-1] -= (TBYTE)(e_BOOL_TAG);
        }
        return nSize;
        break;
      }
      case CIEC_ANY::e_TIME:  {
        long long timeInMicroSeconds = (*(TUINT64*) (pa_pCIECData->getDataPtr())) * 1000000LL / CTimerHandler::sm_poFORTETimer->getTicksPerSecond();
        pa_pcBytes[0] = (TBYTE)(timeInMicroSeconds >> 56) & 0xFF;
        pa_pcBytes[1] = (TBYTE)(timeInMicroSeconds >> 48) & 0xFF;
        pa_pcBytes[2] = (TBYTE)(timeInMicroSeconds >> 40) & 0xFF;
        pa_pcBytes[3] = (TBYTE)(timeInMicroSeconds >> 32) & 0xFF;
        pa_pcBytes[4] = (TBYTE)(timeInMicroSeconds >> 24) & 0xFF;
        pa_pcBytes[5] = (TBYTE)(timeInMicroSeconds >> 16) & 0xFF;
        pa_pcBytes[6] = (TBYTE)(timeInMicroSeconds >> 8) & 0xFF;
        pa_pcBytes[7] = (TBYTE)(timeInMicroSeconds) & 0xFF;
        return nSize;
        break;
      }
      case CIEC_ANY::e_WSTRING: {
        string sBuffer = ((CIEC_STRING*)pa_pCIECData)->getValue();
        pa_pcBytes[0] = (TBYTE)((nSize >> 8) & 0x00FF);
        pa_pcBytes[1] = (TBYTE)(nSize & 0x00FF);
        for (TUINT16 i = 2; i<(nSize*2 + 2); i++){
          if(0 != i%2)  {
          pa_pcBytes[i] = sBuffer[(i/2)-1];
          }
          else  {
            pa_pcBytes[i] = 0;
          }
        }
        return nSize*2+3;
        break;
      }
      case CIEC_ANY::e_STRING:  {
        string sBuffer = ((CIEC_STRING*)pa_pCIECData)->getValue();
        pa_pcBytes[0] = (TBYTE)((nSize >> 8) & 0x00FF);
        pa_pcBytes[1] = (TBYTE)(nSize & 0x00FF);

        for (TUINT16 i = 0; i <= nSize; i++){
          pa_pcBytes[i+2] = sBuffer[i];
        }

        return nSize+3;
        break;
      }
      case CIEC_ANY::e_ARRAY: {
        pa_pcBytes[0] = (TBYTE)((nSize >> 8) & 0x00FF);
        pa_pcBytes[1] = (TBYTE)(nSize & 0x00FF);  //Number of array elements

        pa_pcBytes += 2;
        int nLen = serializeTag(pa_pcBytes,((CIEC_ARRAY*)pa_pCIECData)->getReferencePointer()); //length of an element

        ++pa_pcBytes;
        for (int i = 0; i < nSize; i++){  //serialize elements
          if(serializeValue(pa_pcBytes, pa_nStreamSize, (*((CIEC_ARRAY*)pa_pCIECData))[i], nLen) == -1) {
            return -1;
          }
          pa_pcBytes += nLen-1; //shift buffer array
        }
        return 1 + 2 + 1 + (nLen - 1) * nSize; // array tag + array len + contained data tag + Len needed by conatined data;
      }
      default: {
        return -1;
      }
    }
  }
  return 1;
}

int CITASerDeser::deserializeValue(TBYTE* pa_pcBytes, int pa_nStreamSize, TIEC_ANYPtr pa_pCIECData) {

  CIEC_ANY::EDataTypeID eDataType = pa_pCIECData->getDataTypeID();
  int nSize = csm_aDataTags[eDataType][1];
  //length of simple datatypes is given by the tag array

  if(CIEC_ANY::e_BOOL < eDataType && eDataType <= CIEC_ANY::e_DATE_AND_TIME) {//e_ANY to e_DATE_AND_TIME can be handled the same way
    if(pa_nStreamSize >= nSize-1)  {
      TBYTE* DataPtr = pa_pCIECData->getDataPtr();
      int nValueSize =  nSize -1; //Length of the tag

#ifdef FORTE_USE_64BIT_DATATYPES
      //setting m_uAnyData to 0
      *((long long*)DataPtr) = 0;
#else
      *((long*)DataPtr) = 0;
#endif
#ifdef FORTE_LITTLE_ENDIAN
      for (int i = 0; i < nValueSize; i++){
        DataPtr[i] = pa_pcBytes[(nValueSize - 1) - i];
      }
#endif
#ifdef FORTE_BIG_ENDIAN
      if(CIEC_ANY:: e_REAL != eDataType)
        for (unsigned int i=0; i < nValueSize;i++){
#ifdef FORTE_USE_64BIT_DATATYPES
          DataPtr[(sizeof(TUINT64)-1)-i] = pa_pcBytes[(nValueSize - 1) - i];
#else
          DataPtr[(sizeof(TUINT32)-1)-i] = pa_pcBytes[(nValueSize - 1)-i];
#endif
        }
      else  { //FLOAT must be processed this way, because it is always saved in the first 4 bytes
        for (unsigned int i = 0; i < nValueSize; i++){
          DataPtr[(sizeof(TFLOAT) - 1) - i] = pa_pcBytes[(nValueSize - 1) - i];
        }
      }
#endif
      return nSize;
    }
  }
  else  {
    switch(eDataType) {
      case CIEC_ANY::e_BOOL:  {
        *((CIEC_BOOL*)pa_pCIECData) = (csm_aDataTags[CIEC_ANY::e_BOOL][0] == pa_pcBytes[-1]);
        //bool data is encoded in bool tag
      }
      case CIEC_ANY::e_TIME:  {
        if(pa_nStreamSize >= 8){
                long long timeInMicroSeconds = ((((long long)pa_pcBytes[0]) << 56) & 0xFF00000000000000LL)
                + ((((long long)pa_pcBytes[1]) << 48) & 0x00FF000000000000LL)
                + ((((long long)pa_pcBytes[2]) << 40) & 0x0000FF0000000000LL)
                + ((((long long)pa_pcBytes[3]) << 32) & 0x000000FF00000000LL)
                + ((((long long)pa_pcBytes[4]) << 24) & 0x00000000FF000000)
                + ((((long long)pa_pcBytes[5]) << 16) & 0x0000000000FF0000)
                + ((((long long)pa_pcBytes[6]) << 8)  & 0x000000000000FF00)
                + (((long long)pa_pcBytes[7])         & 0x00000000000000FF);
                (*(CIEC_TIME*)pa_pCIECData) = ((TINT64)(timeInMicroSeconds * CTimerHandler::sm_poFORTETimer->getTicksPerSecond() / 1000000LL));
        }
        return nSize;
        break;
      }
      case CIEC_ANY::e_WSTRING: {
        if(pa_nStreamSize >=2)  {
        nSize = (((((TUINT16)pa_pcBytes[0]) << 8) & 0xFF00) +
            ((TUINT16)pa_pcBytes[1] & 0x00FF)) * 2;
        //size of WSTRING must be read from the incoming message
        }
        if(pa_nStreamSize >= nSize) {
          string sBuffer;
          for(TUINT16 i = 1+2; i <= nSize + 2; i+=2)  {
            sBuffer += pa_pcBytes[i];
          }
          ((CIEC_WSTRING*)pa_pCIECData)->setValue(sBuffer);
        }
        return nSize*2+2;
        break;
      }
      case CIEC_ANY::e_STRING: {
        if(pa_nStreamSize >=2)  {
            nSize = (((((TUINT16)pa_pcBytes[0]) << 8) & 0xFF00) +
                ((TUINT16)pa_pcBytes[1] & 0x00FF));
            //size of STRING must be read from the incoming message
        }
        if(pa_nStreamSize >= nSize) {
          string sBuffer;
          for(TUINT16 i = 2; i < nSize + 2; ++i)  {
            sBuffer += pa_pcBytes[i];
          }
          ((CIEC_STRING*)pa_pCIECData)->setValue(sBuffer);
        }
        return nSize+2;
        break;
      }
      case CIEC_ANY::e_ARRAY: {
        if(pa_nStreamSize >= 2){
          nSize = (((((TUINT16)pa_pcBytes[0]) << 8) & 0xFF00) +
                  ((TUINT16)pa_pcBytes[1] & 0x00FF));
          //number of elements in ARRAY must be read from the incoming message
          int nRetVal = 2;
             if(nSize > 0){
              if(pa_nStreamSize < 3)  {
                return 0;
              }
              if(0 > deserializeTag(&pa_pcBytes[2],((CIEC_ARRAY*)pa_pCIECData)->getReferencePointer()))  {
                return -1;
              }
              pa_pcBytes += 3;
              int nValueLen;
              pa_nStreamSize -= 3;
              nRetVal = 3;
              for(int i = 0; i < nSize; ++i){
                nValueLen = deserializeValue(pa_pcBytes, pa_nStreamSize, (*(CIEC_ARRAY*)pa_pCIECData)[i])-1;
                //size of the elements is given by the array datatype
                if(nValueLen <= 0)
                  return nValueLen;
                pa_nStreamSize -= nValueLen;
                pa_pcBytes += nValueLen;
                nRetVal += nValueLen;
              }
            }
            nSize = nRetVal;  //return number of bytes the array used up in pa_pcBytes
          }
        return nSize;
        break;
      }
      default:  {
        return -1;
      }
    }
  }
  return -1;
}
