/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20081215: mmm - initial version
 *******************************************************************************/

#ifndef ISERDESER_H_
#define ISERDESER_H_

#include "singlet.h"
#include "../datatypes/any.h"
#include "../datatypes/forte_array.h"
#include "../datatypes/forte_bool.h"
#include "../datatypes/forte_sint.h"
#include "../datatypes/forte_int.h"
#include "../datatypes/forte_dint.h"
#include "../datatypes/forte_usint.h"
#include "../datatypes/forte_uint.h"
#include "../datatypes/forte_udint.h"
#include "../datatypes/forte_byte.h"
#include "../datatypes/forte_word.h"
#include "../datatypes/forte_dword.h"
#include "../datatypes/forte_real.h"
#include "../datatypes/forte_lreal.h"
#include "../datatypes/forte_wstring.h"
#include "../datatypes/forte_string.h"
#include "../../arch/devlog.h"

using namespace std;

class ISerDeser {

public:
  virtual ~ISerDeser()  {};

  /*! \brief Interface for serializing data
   *
   *   \param pa_pcBytes  Array to write the serialized data to
   *   \param pa_nStreamSize  Remaining magnitude of the actual transmission frame
   *   \return Can be the following response:
   *     - retval = 1....serialization went well
   *     - retval = 0....less remaining magnitude of transmission frame
   *     - retval =-1....error
   *
   */
  virtual int serialize(TBYTE* , int , TIEC_ANYPtr) = 0;

  /*! \brief Interface for serializing data
   *
   *   \param pa_pcBytes  Array holding the data to deserialize from
   *   \param pa_nStreamSize  Remaining magnitude of the actual transmission frame
   *   \return Can be the following response:
   *     - retval > 0....deserialization went well, number of bytes used
   *     - retval = 0....less remaining magnitude of transmission frame
   *     - retval =-1....error
   */
  virtual int deserialize(TBYTE* , int , TIEC_ANYPtr) = 0;

  virtual void serializeNull(TBYTE*) = 0;

  virtual bool isNull(TBYTE* pa_pcBytes) = 0;
};

#endif /* ISERDESER_H_ */
