/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support
 *******************************************************************************/
#include <string.h>
#include "./datatypes/any.h"
#include "typelib.h"
#include "eventconn.h"
#include "dataconn.h"
#include "adapterconn.h"
#include "resource.h"
#include "if2indco.h"

CTypeLib::CTypeLib() {
  m_eLastErrorMSG = e_RDY;
  m_poFBLibStart = m_poFBLibEnd = 0;
  m_poRESLibStart = m_poRESLibEnd = 0;
  m_poDTLibStart = m_poDTLibEnd = 0;
}

CTypeLib::~CTypeLib() {
  //TODO free memory of fblibrary (maybe not necessary as typelib is deleted at the end of the program)
  //TODO free memory of RESlibrary (maybe not necessary as typelib is deleted at the end of the program)
}

CTypeLib::CTypeEntry *CTypeLib::findType(CStringDictionary::TStringId pa_nTypeId, CTypeLib::CTypeEntry *pa_poListStart) {
  CTypeEntry *retval = 0;
  for (CTypeEntry *poRunner = pa_poListStart; poRunner != 0; poRunner
      = poRunner->m_poNext)
    if (pa_nTypeId == poRunner->m_nTypeNameId) {
      retval = poRunner;
      break;
    }
  return retval;
}

CFunctionBlock *CTypeLib::createFB(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nFBTypeId, CResource *pa_poRes, CStringDictionary::TStringId pa_nApplId) {
  CFunctionBlock *newFB = 0;
  CTypeEntry *poToCreate = findType(pa_nFBTypeId, m_poFBLibStart);
  //TODO: Avoid that the user can create generic blocks.
  if (0 != poToCreate) {
    newFB
        = ((CFBTypeEntry *)poToCreate)->createFBInstance( pa_nApplId, pa_poRes);
    if (0 == newFB) // we could not create the requested object
      m_eLastErrorMSG = e_OVERFLOW;
  } else { //check for parametrizeable FBs (e.g. SERVER)
    TIdentifier acGenFBName = { "GEN_" };
    const char *acTypeBuf = CStringDictionary::getInstance().get(pa_nFBTypeId);
    const char *pcUnderScore = strchr(acTypeBuf, '_');
    if (0 != pcUnderScore) { // We found no underscore in the type name therefore it can not be a generic type
      int nCopyLen = pcUnderScore - acTypeBuf;
      if (nCopyLen > cg_nIdentifierLength - 4)
        nCopyLen = cg_nIdentifierLength - 4;
      strncpy(&(acGenFBName[4]), acTypeBuf, nCopyLen);
      acGenFBName[cg_nIdentifierLength] = '\0';
      poToCreate = findType(CStringDictionary::getInstance().getId(acGenFBName), m_poFBLibStart);
      if (0 != poToCreate) {
        newFB = ((CFBTypeEntry *)poToCreate)->createFBInstance(pa_nApplId, pa_poRes);
        if (0 == newFB) // we could not create the requested object
          m_eLastErrorMSG = e_OVERFLOW;
        else { // we got a configurable block
          if (!newFB->configureFB(acTypeBuf)) {
            //TODO: Handle an error, if the FB was not configured correctly!!!!
          }
        }
      } else
        m_eLastErrorMSG = e_UNSUPPORTED_TYPE;
    } else
      m_eLastErrorMSG = e_UNSUPPORTED_TYPE;
  }
  if(0 != newFB){
  	newFB->setInstanceNameId(pa_nInstanceNameId);
  }
  return newFB;
}

bool CTypeLib::deleteFB(CFunctionBlock *pa_poFBToDelete) {
	if (! pa_poFBToDelete)
		return true;

  delete pa_poFBToDelete;
  return true;
}

CResource *CTypeLib::createResource(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nResTypeId, CDevice &pa_poDevice) {
  CResource *newRes = 0;
  CTypeEntry *poToCreate = findType(pa_nResTypeId, m_poRESLibStart);

  if (0 != poToCreate) {
    newRes = ((CResTypeEntry *)poToCreate)->createResInstance(pa_poDevice);
    if (0 == newRes) // we could not create the requested object
      m_eLastErrorMSG = e_OVERFLOW;
    else{
    	newRes->setInstanceNameId(pa_nInstanceNameId);
    }
  }
  else
    m_eLastErrorMSG = e_UNSUPPORTED_TYPE;
  return newRes;
}

bool CTypeLib::deleteResource(CResource *pa_poRESToDelete) {
  delete pa_poRESToDelete;
  return true;
}

CIEC_ANY *CTypeLib::createDataTypeInstance(CStringDictionary::TStringId pa_nDTNameId) {
  CIEC_ANY *poNewDT = 0;
  CTypeEntry *poToCreate = findType(pa_nDTNameId, m_poDTLibStart);
  if (0 != poToCreate) {
    poNewDT = ((CDataTypeEntry *)poToCreate)->createDataTypeInstance();
    if (0 == poNewDT) // we could not create the requested object
      m_eLastErrorMSG = e_OVERFLOW;
  } else
    m_eLastErrorMSG = e_UNSUPPORTED_TYPE;

  return poNewDT;
}

bool CTypeLib::deleteDataTypeInstance(CIEC_ANY * pa_poDTInst) {
  delete pa_poDTInst;
  return true;
}

CEventConnection *CTypeLib::createEventConnection(TConnectionID pa_nSourceId) {
  return new CEventConnection(pa_nSourceId);
}

CDataConnection *CTypeLib::createDataConnection(TConnectionID pa_nSourceId) {
  return new CDataConnection(pa_nSourceId);
}

CAdapterConnection *CTypeLib::createAdapterConnection(TConnectionID pa_nSourceId) {
  return new CAdapterConnection(pa_nSourceId);
}

CInterface2InternalDataConnection *CTypeLib::createInterfaceDataConnection(TConnectionID pa_nSourceId, CIEC_ANY *pa_poValue) {
  return new CInterface2InternalDataConnection(pa_nSourceId, pa_poValue);
}

void CTypeLib::deleteConnection(CConnection *pa_poToDelete) {
  delete pa_poToDelete;
}

void CTypeLib::addFirmwareFBType(CFirmwareFBEntry *pa_poFirmewareFBEntry) {
  if (0 == findType(pa_poFirmewareFBEntry->m_nTypeNameId, m_poFBLibStart)) {
    if (m_poFBLibStart == 0)
      m_poFBLibStart = pa_poFirmewareFBEntry;
    else
      m_poFBLibEnd->m_poNext = pa_poFirmewareFBEntry;
    m_poFBLibEnd = pa_poFirmewareFBEntry;
  }
}

void CTypeLib::addFirmwareResourceType(CFirmwareResourceEntry *pa_poResourceTypeEntry) {
  if (0 == findType(pa_poResourceTypeEntry->m_nTypeNameId, m_poRESLibStart)) { //only add if it is not allready in
    if (m_poRESLibStart == 0)
      m_poRESLibStart = pa_poResourceTypeEntry;
    else
      m_poRESLibEnd->m_poNext = pa_poResourceTypeEntry;
    m_poRESLibEnd = pa_poResourceTypeEntry;
  }
}

void CTypeLib::addFirmwareDataType(CFirmwareDataTypeEntry *pa_poDTEntry) {
  if (0 == findType(pa_poDTEntry->m_nTypeNameId, m_poDTLibStart)) {
    if (m_poDTLibStart == 0)
      m_poDTLibStart = pa_poDTEntry;
    else
      m_poDTLibEnd->m_poNext = pa_poDTEntry;
    m_poDTLibEnd = pa_poDTEntry;
  }
}
