/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 * 			2008-04-24: ih, hegny@acin.tuwien.ac.at - added adapter support
 *******************************************************************************/
#ifndef _TYPELIB_H_
#define _TYPELIB_H_

#include "singlet.h"
#include "mgmcmd.h"


//forward declaration of a few classes to reduce includefile dependencies
class CDevice;
class CFunctionBlock;
class CResource;
class CConnection;
class CEventConnection;
class CDataConnection;
class CAdapterConnection;
class CInterface2InternalDataConnection;
class CIEC_ANY;


//!\ingroup CORE Type for a function pointer which allows to create a functionblock instance
  typedef CFunctionBlock *(*TFunctionBlockCreateFunc)(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes);
//!\ingroup CORE Type for a function pointer which allows to create a resource instance
  typedef CResource *(*TResourceCreateFunc)(CDevice &pa_roDevice);

//!\ingroup CORE Type for a function pointer which allows to create a data type instance
  typedef CIEC_ANY *(*TDataTypeCreateFunc)(void);

//!\ingroup CORE This define is used to create the definition necessary for FirmwareFunction blocks in order to get them automatically added to the FirmwareType list.
#define DECLARE_FIRMWARE_FB(fbclass) \
  private: \
    const static CTypeLib::CFirmwareFBEntry csm_oFirmwareFBEntry_##fbclass; \
  public:  \
    static CFunctionBlock *createFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes){ \
      return new fbclass(pa_nAppl, pa_poSrcRes);\
    }; \
    virtual CStringDictionary::TStringId getFBTypeId(void) const {return (csm_oFirmwareFBEntry_##fbclass.getTypeNameId()); };\
  private:

//!\ingroup CORE This define is used to create the implementation for the above definition.
#define DEFINE_FIRMWARE_FB(fbclass)\
  const CTypeLib::CFirmwareFBEntry fbclass::csm_oFirmwareFBEntry_##fbclass(CStringDictionary::getInstance().insert(#fbclass), fbclass::createFB);

//!\ingroup CORE This define is used to create the definition necessary for Adapter types.
#define DECLARE_ADAPTER_TYPE(adapterclass) \
  private: \
    const static CTypeLib::CAdapterTypeEntry csm_oAdapterTypeEntry_##adapterclass; \
  public:  \
    virtual CStringDictionary::TStringId getFBTypeId(void) const {return (csm_oAdapterTypeEntry_##adapterclass.getTypeNameId()); };\
  private:

//!\ingroup CORE This define is used to create the implementation for the above definition.
#define DEFINE_ADAPTER_TYPE(adapterclass)\
  const CTypeLib::CAdapterTypeEntry adapterclass::csm_oAdapterTypeEntry_##adapterclass(CStringDictionary::getInstance().insert(#adapterclass));



//!\ingroup CORE This define is used to create the definition necessary for Firmware resources in order to get them automatically added to the FirmwareType list.
#define DECLARE_FIRMWARE_RESOURCE(resourceclass) \
   public:  \
    static CResource *createResource(CDevice &pa_roDevice){ \
      return new resourceclass(pa_roDevice);\
    }; \
    const static CTypeLib::CFirmwareResourceEntry csm_oFirmwareResourceEntry_##resourceclass; \
    virtual CStringDictionary::TStringId getFBTypeId(void) const {return (csm_oFirmwareResourceEntry_##resourceclass.getTypeNameId()); };\
   private:

//!\ingroup CORE This define is used to create the implementation for the above definition.
#define DEFINE_FIRMWARE_RESOURCE(resourceclass)\
  const CTypeLib::CFirmwareResourceEntry resourceclass::csm_oFirmwareResourceEntry_##resourceclass(CStringDictionary::getInstance().insert(#resourceclass), resourceclass::createResource);


//!\ingroup CORE This define is used to create the definition necessary for Firmware datatype in order to get them automatically added to the FirmwareType list.
#define DECLARE_FIRMWARE_DATATYPE(datatypename) \
   public:  \
    static CIEC_ANY *createDataType(void){ \
      return new CIEC_##datatypename;\
    }; \
    const static CTypeLib::CFirmwareDataTypeEntry csm_oFirmwareDataTypeEntry_##datatypename; \
   private:

//!\ingroup CORE This define is used to create the implementation for the above definition.
#define DEFINE_FIRMWARE_DATATYPE(datatypename)\
  const CTypeLib::CFirmwareDataTypeEntry CIEC_##datatypename::csm_oFirmwareDataTypeEntry_##datatypename(CStringDictionary::getInstance().insert(#datatypename), CIEC_##datatypename::createDataType);



//!\ingroup CORE This define is used to create the definition necessary for GenericFunction blocks in order to get them automatically added to the FirmwareType list.
#define DECLARE_GENERIC_FB(fbclass) \
  private: \
    const static CTypeLib::CFirmwareFBEntry csm_oFirmwareFBEntry_##fbclass; \
  public:  \
    static CFunctionBlock *createFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes){ \
      return new fbclass(pa_nAppl, pa_poSrcRes);\
    }; \
  private:


//!\ingroup CORE This define is used to create the implementation for the above definition.
#define DEFINE_GENERIC_FB(fbclass)\
  const CTypeLib::CFirmwareFBEntry fbclass::csm_oFirmwareFBEntry_##fbclass(CStringDictionary::getInstance().insert(#fbclass), fbclass::createFB);




/*!\ingroup CORE \brief Class for storing the functionblock libraries.
 *
 *
 *
 * @author AZ
 * @version 20061108/SMR - Added defines for generic FBs
 * @version 20061108/AZ - Added an additional data type library
 * @version 20061107/AZ -  Changed type entrie handling as base work for supporting user defined FBs and also add a factory for datatypes
 * @version 20060117/SMR - Added support for generic FBs
 * @version 20050919/AZ - Initial version as basis for the interface documentation
 */

class CTypeLib{
  SINGLETON(CTypeLib)
public:
//! The base class for all type entries in the type lib.
  class CTypeEntry{
    private:
      CStringDictionary::TStringId m_nTypeNameId;
    public:
      CTypeEntry *m_poNext; //!< a pointer to the next element in the list. Will be used to build single linked list of type entries.

      CTypeEntry(CStringDictionary::TStringId pa_nTypeNameId) {
        m_nTypeNameId = pa_nTypeNameId;
        m_poNext = 0;
      };
      virtual ~CTypeEntry(void) {};

      CStringDictionary::TStringId getTypeNameId(void) const { return m_nTypeNameId; };

      friend class CTypeLib; //The type lib is a friend in order to speed up things and remove the need for calling gettype
  };

//! The base class for all function block types entries in the type lib.
  class CFBTypeEntry : public CTypeEntry{
    public:
      CFBTypeEntry(CStringDictionary::TStringId pa_nTypeNameId):CTypeEntry(pa_nTypeNameId) {};
      virtual ~CFBTypeEntry(void) {};
       virtual CFunctionBlock *createFBInstance(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) = 0;
  };

  /*!\brief Class for creating the entry data in the typelib for a specific Firmware FB-type
 */
  class CFirmwareFBEntry : public CFBTypeEntry{
    private:
      TFunctionBlockCreateFunc m_pfuncFBCreationFunc;
    public:
      CFirmwareFBEntry(CStringDictionary::TStringId pa_nTypeNameId, TFunctionBlockCreateFunc pa_pfuncCreateFB):
          CFBTypeEntry(pa_nTypeNameId){
        CTypeLib::getInstance().addFirmwareFBType(this);
        m_pfuncFBCreationFunc = pa_pfuncCreateFB;
      };
      virtual ~CFirmwareFBEntry(void){};
      virtual CFunctionBlock *createFBInstance(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes){
        return m_pfuncFBCreationFunc(pa_nAppl, pa_poSrcRes);
      }
  };

/*!\brief Class for adapter type entries in the type lib.
 */
      class CAdapterTypeEntry : public CTypeEntry{
        public:
          CAdapterTypeEntry(CStringDictionary::TStringId pa_nTypeNameId):CTypeEntry(pa_nTypeNameId) {};
          virtual ~CAdapterTypeEntry(void) {};
      };

//! The base class for all function block types entries in the type lib.
  class CResTypeEntry : public CTypeEntry{
    public:
      CResTypeEntry(CStringDictionary::TStringId pa_nTypeNameId):CTypeEntry(pa_nTypeNameId) {};
      virtual ~CResTypeEntry(void) {};
      virtual CResource *createResInstance(CDevice &pa_roDevice) = 0;
  };

/*!\brief Class for creating the entry data in the typelib for a specific Firmware Resource-type
 */
  class CFirmwareResourceEntry : public CResTypeEntry{
    private:
      TResourceCreateFunc m_pfuncResCreationFunc;
    public:
      CFirmwareResourceEntry(CStringDictionary::TStringId pa_nTypeNameId, TResourceCreateFunc pa_pfCreateResource):
          CResTypeEntry(pa_nTypeNameId){
        CTypeLib::getInstance().addFirmwareResourceType(this);
        m_pfuncResCreationFunc = pa_pfCreateResource;
      };
      virtual ~CFirmwareResourceEntry(void){};
      virtual CResource *createResInstance(CDevice &pa_roDevice){
        return m_pfuncResCreationFunc(pa_roDevice);
      }
  };

//! The base class for all data type entries in the type lib.
  class CDataTypeEntry : public CTypeEntry{
    public:
      CDataTypeEntry(CStringDictionary::TStringId pa_nTypeNameId) : CTypeEntry(pa_nTypeNameId) {};
      virtual ~CDataTypeEntry(void) {};
      virtual  CIEC_ANY *createDataTypeInstance(void) = 0;
  };

//! The class for all firmware data type entries in the type lib.
  class CFirmwareDataTypeEntry : public CDataTypeEntry{
    private:
    protected:
      TDataTypeCreateFunc m_pfuncDTCreateFunc;
    public:
      CFirmwareDataTypeEntry(CStringDictionary::TStringId pa_nTypeNameId, TDataTypeCreateFunc pa_pfuncDTCreateFunc) : CDataTypeEntry(pa_nTypeNameId){
        m_pfuncDTCreateFunc = pa_pfuncDTCreateFunc;
        CTypeLib::getInstance().addFirmwareDataType(this);
      };
      virtual ~CFirmwareDataTypeEntry(void) {};
      virtual  CIEC_ANY *createDataTypeInstance(void){
        return m_pfuncDTCreateFunc();
      }
  };

private:
/*!\brief Buffer for the last error that occured.
 */
  EMGMResponse m_eLastErrorMSG;

  CFBTypeEntry *m_poFBLibStart, //!< pointer to the beginn of the firmware fb library list
               *m_poFBLibEnd; //!<pointer to the end of the firmware fb library list

  CResTypeEntry *m_poRESLibStart, //!< pointer to the beginn of the resource library list
                  *m_poRESLibEnd; //!<pointer to the end of the resource ligrary list

  CDataTypeEntry *m_poDTLibStart, //!< pointer to the beginn of the data type library
                 *m_poDTLibEnd; //!< poiner to the end of the data type library

  static CTypeEntry *findType(CStringDictionary::TStringId pa_nTypeId, CTypeEntry *pa_poListStart);

protected:
public:
	~CTypeLib();
/*!\brief Create a new FB instance of given type and given instance name.
 *
 * Comment az: I've removed the param pa_sFBName as this information can be stored within the resource
 * \param pa_sFBType Type of the FB to create.
 * \param pa_poRes   Resource the FB is contained in.
 * \param pa_acAppl the name of the application to which the FB belongs to
 * \return On success a pointer to the new FB is returned, else the return value is 0 use getLastError for getting more information on the problem.
 *   posible errorcodes are:
 *    - e_UNSUPPORTED_TYPE   The requested FB type is not known to the typelib
 *    - e_INVALID_OPERATION The requested FB can not be created (e.g. out of memory)
 */
  CFunctionBlock *createFB(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nFBTypeId, CResource *pa_poRes, CStringDictionary::TStringId pa_nApplId);
/*\brief Delete the given FB
 */
  bool deleteFB(CFunctionBlock *pa_poFBToDelete);
/*!\brief Creates a new resource instance of given type and given instance name.
 *
 * Comment az: I've removed the param pa_sResName as this information can be stored within the CDeviceAdministrator
 * \param pa_sResType Type of the resource to create.
 * \param pa_poDevice Pointer to the device the resource is contained in.
 * \return On success a pointer to the new resource is returned else the return value is 0 use getLastError for getting more information on the problem.
 *   posible errorcodes are:
 *    - e_UNSUPPORTED_TYPE   The requested RES type is not known to the typelib
 *    - e_INVALID_OPERATION The requested RES can not be created (e.g. out of memory)
 */
  CResource *createResource(CStringDictionary::TStringId pa_nInstanceNameId, CStringDictionary::TStringId pa_nResTypeId, CDevice &pa_poDevice);
/*!\brief Delete the given Resource
 */
  bool deleteResource(CResource *pa_poRESToDelete);
//!Create an instance of an data type.
  CIEC_ANY *createDataTypeInstance(CStringDictionary::TStringId pa_nDTNameId);
//!Delete an instance of an data type.
  bool deleteDataTypeInstance(CIEC_ANY * pa_poDTInst);
/*!\brief Creates a new event connection with a given source identifier.
 *
 * \param pa_sSrc Source identifier of the connection to create.
 * \return On success a pointer to the new CEventConnection is returned else the returnvalue is 0 use getLastError for getting more information on the problem.
 */
  CEventConnection *createEventConnection(TConnectionID pa_nSourceId);
/*!\brief Creates a new data connection with a given source identifier.
 *
 * \param pa_sSrc Source identifier of the connection to create.
 * \return On success a pointer to the new CDataConnection is returned else the returnvalue is 0 use getLastError for getting more information on the problem.
 */
  CDataConnection *createDataConnection(TConnectionID pa_nSourceId);

  /*!\brief Creates a new data connection with a given source identifier.
   *
   * \param pa_sSrc Source identifier of the connection to create.
   * \return On success a pointer to the new CDataConnection is returned else the returnvalue is 0 use getLastError for getting more information on the problem.
   */
  CAdapterConnection *createAdapterConnection(TConnectionID pa_nSourceId);


  /*!\brief Creates a new interface data connection with a given source identifier.
 *
 * \param pa_sSrc Source identifier of the connection to create.
 * \param pa_poValue pointer to the data value the connection should carry
 * \return On success a pointer to the new CInterface2InternalDataConnection is returned else the returnvalue is 0 use getLastError for getting more information on the problem.
 */  
  CInterface2InternalDataConnection *createInterfaceDataConnection(TConnectionID pa_nSourceId, CIEC_ANY *pa_poValue);

/*!\brief Delete the connection
 */
  void deleteConnection(CConnection *pa_poToDelete);
/*!\brief Return a description of the last error that occured within the CTypeLib.
 *
 * e.g. Out of memory
 * \return Reference to the error string.
 */
  EMGMResponse getLastError(void) const { return m_eLastErrorMSG; };

/*!\brief add a Firmware FB type to the type lib (is mainly used by the coresponding entry class).
 */
  void addFirmwareFBType(CFirmwareFBEntry *pa_poFirmewareFBEntry);

/*!\brief add a Firmware Resource type to the type lib (is mainly used by the coresponding entry class).
 */
  void addFirmwareResourceType(CFirmwareResourceEntry *pa_poResourceTypeEntry);
/*!\brief add a Firmware data type to the type lib (is mainly used by the coresponding entry class).
 */
  void addFirmwareDataType(CFirmwareDataTypeEntry *pa_poDTEntry);
/*!\brief Get a pointer to the begin of the FB library list
 */
  CTypeEntry *getFBLibStart() { return m_poFBLibStart; }
/*!\brief Get a pointer to the begin of the resource library list
 */
  CTypeEntry *getResLibStart() { return m_poRESLibStart; }
/*!\brief Get a pointer to the begin of the datatype library list
 */
  CTypeEntry *getDTLibStart() { return m_poDTLibStart; }
};

#endif /*TYPELIB_H_*/
