/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "./commfb.h"

//TODO: Add right prefix to numbers
const unsigned int scm_QI_ID = 0;
const unsigned int scm_ID_ID = 1;
const unsigned int scm_QO_ID = 0;
const unsigned int scm_STATUS_ID = 1;

const TFBNameListEntry CCommFB::scm_acDataInputNames[3] = {"QI", "ID"};
const TFBNameListEntry CCommFB::scm_acDataOutputNames[3] = {"QO", "STATUS"};

const char* CCommFB::scm_sResponseTexts[7] = {"OK", "INVALID_ID", "TERMINATED", \
          "INVALID_OBJECT", "DATA_TYPE_ERROR", "INHIBITED", "NO_SOCKET"};

CCommFB::CCommFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,
          const  SFBInterfaceSpec *pa_stFBInterfaceSpec,
          TEventConnectionPtr *pa_apoEOConns) :
          CEventSourceFB(pa_nAppl, pa_poSrcRes, pa_stFBInterfaceSpec, pa_apoEOConns, 0,0,0,0) {
  m_poQI = 0;
  m_poID = 0;
  m_nConfiguredFBTypeNameId = CStringDictionary::scm_nInvalidStringId;
  for (int i=0;i<32;i++) m_poSD[i]=0;
  m_poQO = 0;
  m_poSTATUS=0;
  SD = 0;
  RD = 0;
  for (int i=0;i<32;i++) m_poRD[i]=0;

}

unsigned int CCommFB::getDIID(const char *pa_acDIName) const {
	char tempstring[7] = "SD_"; // Place to hold a String like "SD_XXX\0"
	  if(!strcmp(pa_acDIName, scm_acDataInputNames[0])) {
	    return scm_QI_ID;
	  }
	  if(!strcmp(pa_acDIName, scm_acDataInputNames[1])) {
	    return scm_ID_ID;
	  }
	  for(unsigned int i = 0; i < m_nNumDataInput; i++) {
	    snprintf(&(tempstring[3]), 7, "%i",i+1);
	    if(!strcmp(pa_acDIName,tempstring)) {
	      return i+2;
	    }
	  }
	  return cg_nInvalidDataInputID;
}

unsigned int CCommFB::getDOID(const char *pa_acDOName) const {
	char tempstring[7] = "RD_"; // Place to hold a String like "RD_XXX\0"
	if(!strcmp(pa_acDOName, scm_acDataOutputNames[0])) {
	    return scm_QO_ID;
	  }
	if(!strcmp(pa_acDOName, scm_acDataOutputNames[1])) {
	    return scm_STATUS_ID;
	  }
	for(unsigned int i = 0; i < m_nNumDataOutput; i++) {
	    snprintf(&(tempstring[3]), 7, "%i",i+1);
	    if(!strcmp(pa_acDOName,tempstring)) {
	      return i+2;
	    }
	  }
	return cg_nInvalidDataOutputID;
}

bool CCommFB::connectDO(const unsigned int pa_nDOID, CDataConnection* pa_poDataCon) {
	switch(pa_nDOID) {
		case scm_QI_ID: {
			m_poQO = pa_poDataCon;
			return true;
		}
		case scm_STATUS_ID: {
			m_poSTATUS = pa_poDataCon;
			return true;
		}
		default: {
			if(pa_nDOID-2 < m_nNumDataOutput) {
				m_poRD[pa_nDOID-2] = pa_poDataCon;
				RD[pa_nDOID-2] = pa_poDataCon->getValue()->clone();

				return true;
			}
			else return false;
		}
	}
}

bool CCommFB::connectDI(const unsigned int pa_nDIID, CDataConnection* pa_poDataCon) {
	switch(pa_nDIID) {
		case scm_QI_ID: {
			m_poQI = pa_poDataCon;
			return true;
		}
		case scm_ID_ID: {
			m_poID = pa_poDataCon;
			return true;
		}
		default: {
			if(pa_nDIID-2 < m_nNumDataInput) {
				m_poSD[pa_nDIID-2] = pa_poDataCon;
				SD[pa_nDIID-2] = pa_poDataCon->getValue()->clone();
				return true;
			}
			else return false;
		}
	}
  }

CIEC_ANY *CCommFB::getDataOutput (const unsigned int pa_nDOID){
  switch(pa_nDOID) {
	  case scm_QO_ID:
	    return &QO;

	  case scm_STATUS_ID:
	    return &STATUS;

	  default: {
		  if(pa_nDOID-2 < m_nNumDataOutput) {
			  return RD[pa_nDOID-2];
		  }
		  else return 0;
	  }
  }
}

CIEC_ANY *CCommFB::getDataInput (const unsigned int pa_nDIID){
	switch(pa_nDIID) {
		case scm_QI_ID:
		  return &QI;
		case scm_ID_ID:
       return &ID;
		default: {
			if (pa_nDIID-2 < m_nNumDataOutput) {
				return SD[pa_nDIID-2];
			}
			else return 0;
		}
	}
}

bool CCommFB::configureFB(const char *pa_acConfigString){
  TIdentifier tempstring;
  char *sParamA = 0;
  char *sParamB = 0;
  int inlength;
  int i;

  m_nConfiguredFBTypeNameId = CStringDictionary::getInstance().insert(pa_acConfigString);

  strncpy(tempstring, pa_acConfigString, cg_nIdentifierLength);
  tempstring[cg_nIdentifierLength] = '\0';
  inlength = strlen(tempstring);
  for (i=0; i < inlength-1; i++) {	// search first underscore
    if (tempstring[i] == '_') {
      sParamA = sParamB = &(tempstring[i+1]);
      break;
    }
  }
  if (sParamA != 0)	// search for 2nd underscore
    for (i=i+1; i < inlength-1; i++) {
      if (tempstring[i] == '_') {
        tempstring[i] = '\0';
        sParamB = &(tempstring[i+1]);
        break;
      }
    }

  if (sParamB == 0) // no underscore found
  	return false;

  m_nNumDataInput = atoi(sParamA);
  m_nNumDataOutput = atoi(sParamB);

  if (m_nNumDataInput > 0) {
     SD = new TIEC_ANYPtr[m_nNumDataInput];

     if (SD==0) {
       return false;
     }

     for (unsigned int i=0;i<m_nNumDataInput;i++) {
            SD[i]=0;
     }
   }
   if (m_nNumDataOutput > 0) {
     RD = new TIEC_ANYPtr[m_nNumDataOutput];
     for (unsigned int i=0;i<m_nNumDataOutput;i++) {
       RD[i]=0;
     }
     if (RD==0) {
       if (m_nNumDataInput > 0)
  	   	delete [] SD;
       return false;
     }
   }
   return true;
}


