/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _COMMFB_H_
#define _COMMFB_H_

#include "../core/esfb.h"
#include "../core/datatypes/forte_bool.h"
#include "../core/datatypes/forte_wstring.h"

/*! \brief Implementation of the Commfb Class.
 *
 * @author SMR
 * @version 20081120/mmm - added static consts QI_ID, QO_ID, ID_ID, STATUS_ID
 *  added getDIID, getDOID, modified getDataInput, getDataOutput, connectDI, connectDO, configureFB
 * @version 20070313/SMR - added getFBType() for correct handling of Querry-FB commandos
 * @version 20070921/SMR - removed all std::strings
 * @version 20070718/SMR - revised & commented version
 * @version 20060619/SMR - Introduced new FB interface
 * @version 20060427/SMR - Bugfix in recivedata() (caused Coredump)
 * @version 20060322/SMR - Various Changes in combination with server,client blocks
 * @version 20060316/SMR - First usable version
 */


class CCommFB : public CEventSourceFB {

private:

protected:
  static const unsigned int scm_QI_ID=0; //Input ID 0 for QI Input
  static const unsigned int scm_QO_ID=0; //Output ID 0 for QO Output
  static const unsigned int scm_ID_ID=1; //Input ID 1 for ID Input
  static const unsigned int scm_STATUS_ID=1; //Output ID 1 for STATUS Output
  static const char *scm_sResponseTexts[];
  static const TFBNameListEntry scm_acDataInputNames[];
  static const TFBNameListEntry scm_acDataOutputNames[];
  unsigned int m_nNumDataOutput;
  unsigned int m_nNumDataInput;

  CStringDictionary::TStringId m_nConfiguredFBTypeNameId;


  //Data Inputs
  CDataConnection *m_poQI;
  CDataConnection *m_poID;
  CDataConnection *m_poSD[32]; //TODO allocate this dynamic !!!
  //Data Outputs
  CDataConnection *m_poQO;
  CDataConnection *m_poSTATUS;
  CDataConnection *m_poRD[32]; //TODO allocate this dynamic !!!
  //Internal Datatypes
  CIEC_BOOL QI;
  CIEC_BOOL QO;
  CIEC_WSTRING ID;
  CIEC_WSTRING STATUS;

  //Pointers to the input/output datatypes
  TIEC_ANYPtr *SD;
  TIEC_ANYPtr *RD;

/*!\brief implementation of the getDataOutput method defined in the CFunctionBlock class
 * \param pa_nDOID Dataoutput ID
 * \return CIEC_ANY* to the corresponding data
 */
  CIEC_ANY* getDataOutput (const unsigned int pa_nDOID);
/*!\brief implementation of the getDataInput method defined in the CFunctionBlock class
 */
  CIEC_ANY* getDataInput (const unsigned int pa_nDIID);
/*!\brief implementation of the connectDO method defined in the CFunctionBlock class
 */
  bool connectDO(const unsigned int pa_nDOID, CDataConnection* pa_poDataCon);
/*!\brief implementation of the connectDI method defined in the CFunctionBlock class
 */
  bool connectDI(const unsigned int pa_nDIID, CDataConnection* pa_poDataCon);
/*!\brief The configureFB method is used by the typelib to parametrize the number of inputs/outputs
 * \param pa_acConfigString : the string containing the requested typename (eg. "CLIENT_3_2"
 */
  bool configureFB(const char *pa_acConfigString);

 /*!\brief Gets the index of the data input array for a specific data input
  * \param const pa_acDIName Pointer to the name of the data input
  * \return Returns the index of the Array or the Invalid ID
  */
  virtual unsigned int getDIID(const char *pa_acDIName) const;

 /*!\brief Gets the index of the data output array for a specific data output
  * \param const pa_acDOName Pointer to the name of the data output
  * \return Returns the index of the Array or the Invalid ID
  */
  virtual unsigned int getDOID(const char *pa_acDOName) const;

public:
  CCommFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes, \
    const  SFBInterfaceSpec *pa_stFBInterfaceSpec, \
    TEventConnectionPtr *pa_apoEOConns
  );

  virtual ~CCommFB(){
    delete [] SD;
    delete [] RD;
  };

/*!\brief The getFBType method is used by the Querry command to get the instances correct type name (eg. "CLIENT_3_2")
 * \return pointer to typename string
 */
  CStringDictionary::TStringId getFBTypeId(void) const{
    return m_nConfiguredFBTypeNameId;
  };
};

#endif //_COMMFB_H_



