/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: TIME2TIME
 *** Description: Basic Function Block Type
 *** Version: 
 ***     1.0: 2006-11-14/TS - PROFACTOR GmbH - 
 *************************************************************************/

#include "TIME2TIME.h"

DEFINE_FIRMWARE_FB(TIME2TIME)

const TFBNameListEntry TIME2TIME::scm_acDataInputNames[] = {"IN"};

const TFBNameListEntry TIME2TIME::scm_acDataOutputNames[] = {"OUT"};

const TEventID TIME2TIME::scm_nEventREQID = 0;
const TINT16 TIME2TIME::scm_anEIWithIndexes[] = {0};
const TDataIOID TIME2TIME::scm_anEIWith[] = {0, 255};
const TFBNameListEntry TIME2TIME::scm_acEventInputNames[] = {"REQ"};

const TEventID TIME2TIME::scm_nEventCNFID = 0;
const TDataIOID TIME2TIME::scm_anEOWith[] = {0, 255};
const TINT16 TIME2TIME::scm_anEOWithIndexes[] = {0, -1};
const TFBNameListEntry TIME2TIME::scm_acEventOutputNames[] = {"CNF"};

const SFBInterfaceSpec TIME2TIME::scm_stFBInterfaceSpec = {
  1,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  1,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

void TIME2TIME::alg_REQ(void){
OUT = IN;
}

const TINT16 TIME2TIME::scm_nStateSTART = 0;
const TINT16 TIME2TIME::scm_nStateREQ = 1;

void TIME2TIME::enterStateSTART(void){
  m_nECCState = scm_nStateSTART;
}

void TIME2TIME::enterStateREQ(void){
  m_nECCState = scm_nStateREQ;
  alg_REQ();
  sendOutputEvent( scm_nEventCNFID);
}

bool TIME2TIME::executeECCOnce(TEventID pa_nEIID){
  bool bRetVal = true;
  switch(m_nECCState){
    case scm_nStateSTART:
      if(scm_nEventREQID == pa_nEIID)
      	enterStateREQ();
      else
        bRetVal = false; //no transition cleared
      break;
    case scm_nStateREQ:
      if(1)
      	enterStateSTART();
      else
        bRetVal = false; //no transition cleared
      break;
    default:
      DEVLOG_ERROR(cg_nID_CFunctionBlock, 0x01 , "The ECC state is not in the valid range!");
      m_nECCState = 0; //0 is allways the initial state
      break;
  }
  return bRetVal;
}

TIME2TIME::TIME2TIME(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
       CBasicFB(pa_nAppl,
           pa_poSrcRes,
           &scm_stFBInterfaceSpec,
           m_apoEOConns,
           m_apoDIConns,
           m_apoDIs,
           m_apoDOConns,
           m_apoDOs,
            0, 0, 0) {
  m_apoDIs[0] = &IN;
  m_apoDOs[0] = &OUT;
}
 
TIME2TIME::~TIME2TIME(){
}



