/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "E_RESTART.h"
#include "../../core/resource.h"

DEFINE_FIRMWARE_FB(E_RESTART)

const TFBNameListEntry E_RESTART::scm_acEONames[] = {"COLD", "WARM", "STOP"};
const TINT16 E_RESTART::scm_anEOWithIndexes[] = {-1, -1, -1, -1};

const SFBInterfaceSpec E_RESTART::scm_stFBInterfaceSpec = {
  0, 
  0,
  0,
  0,
  3,
  scm_acEONames,
  0,
  scm_anEOWithIndexes,
  0,
  0,
  0,
  0,
};

const TEventID E_RESTART::csm_nCOLDID = 0, 
               E_RESTART::csm_nWARMID = 1, 
               E_RESTART::csm_nSTOPID = 3;

E_RESTART::E_RESTART(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes):
           CEventSourceFB(pa_nAppl, pa_poSrcRes,
          &scm_stFBInterfaceSpec, m_apoEOConns, 0, 0, 0, 0){
  setEventChainExecutor(pa_poSrcRes->getResourceEventExecution());
  m_bStartedOnce =  false;
  m_stEventSourceEventEntry.m_poFB = this;
}

void E_RESTART::executeEvent(TEventID pa_nEIID){
  if(cg_nExternalEventID == pa_nEIID){
    switch(getState()){
      case e_RUNNING:
        sendOutputEvent((m_bStartedOnce) ? csm_nWARMID : csm_nCOLDID);
        m_bStartedOnce =  true;  
        break;
      case e_STOPPED:
        sendOutputEvent(csm_nSTOPID);
        break;
      default:
        break;
    }
  }
}

EMGMResponse E_RESTART::startManagedObject(void){
  EMGMResponse retVal = CFunctionBlock::startManagedObject();
  if(e_RDY == retVal){
    getResource().getDevice().getDeviceExecution().startNewEventChain(this);
  }
  return retVal;
}

EMGMResponse E_RESTART::stopManagedObject(void){
  EMGMResponse retVal = CFunctionBlock::stopManagedObject();
  if(e_RDY == retVal)
    getResource().getDevice().getDeviceExecution().startNewEventChain(this);
  
  return retVal;
}

EMGMResponse E_RESTART::resetManagedObject(void){
  EMGMResponse retVal = CFunctionBlock::stopManagedObject();
  if(e_RDY == retVal) //we are rested reset also the flag in order to get an cold event on the next start
    m_bStartedOnce =  false;
  
  return retVal;
}

