/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include <string.h>
#include "DEV_MGR.h"
#include "../../core/device.h"
#include "../../core/resource.h"
#include <stdio.h>

DEFINE_FIRMWARE_FB(DEV_MGR)


const TFBNameListEntry DEV_MGR::scm_acEINames[2] = {"INIT", "REQ"};
const TDataIOID DEV_MGR::scm_anEIWith[6] = {0, 255, 0, 1, 2, 255};
const TINT16 DEV_MGR::scm_anEIWithIndexes[2] = {0, 2};
const TFBNameListEntry DEV_MGR::scm_acEONames[2] = {"INITO", "CNF"};
const TDataIOID DEV_MGR::scm_anEOWith[4] = {0, 255, 1, 255};
const TINT16 DEV_MGR::scm_anEOWithIndexes[3] = {0, 2, -1};
const TFBNameListEntry DEV_MGR::scm_acDINames[3] = {"QI", "DST", "RQST"};
const TFBNameListEntry DEV_MGR::scm_acDONames[2] = {"QO", "RESP"};

const SFBInterfaceSpec DEV_MGR::scm_stFBInterfaceSpec = {
  2,
  scm_acEINames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  2,
  scm_acEONames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  3,
  scm_acDINames,
  2,
  scm_acDONames
};

const TEventID DEV_MGR::csm_nEventINITOID = 0;
const TEventID DEV_MGR::csm_nEventCNFID = 1;

const TEventID DEV_MGR::csm_nEventINITID = 0;
const TEventID DEV_MGR::csm_nEventREQID = 1;

const char *DEV_MGR::scm_sMGMResponseTexts[13] = {"RDY",
                                                "BAD_PARAMS",
                                                "LOCAL_TERMINATION",
                                                "SYSTEM_TERMINATION",
                                                "NOT_READY",
                                                "UNSUPPORTED_CMD",
                                                "UNSUPPORTED_TYPE",
                                                "NO_SUCH_OBJECT",
                                                "INVALID_OBJECT",
                                                "INVALID_OPERATION",
                                                "INVALID_STATE",
                                                "OVERFLOW",
                                                "INVALID_DST"};

void  DEV_MGR::executeEvent (TEventID pa_nEIID){
  if(csm_nEventINITID == pa_nEIID){
    QO = QI;
    //up to now I dont know what to do on init
    sendOutputEvent(csm_nEventINITOID);
  }
  else
    if(csm_nEventREQID == pa_nEIID){
       if(true == QI)
        executeRQST();
      //TODO: if QI not true send suitable response
       DEVLOG_DEBUG(0x04,0x01,"sending cnf\n");
      sendOutputEvent(csm_nEventCNFID);
      DEVLOG_DEBUG(0x04,0x01,"sent cnf\n");
    }
}

void DEV_MGR::executeRQST(void){
  char acID[7]; //seven characters should be sufficien for holding the id

  SManagementCMD theCommand;
  EMGMResponse resp = e_INVALID_OBJECT;
  string::size_type pos = RQST.getValue().find_first_of('>');

  if(string::npos != pos){
    theCommand.m_nDestination = (DST.getValue().length() != 0) ?
                                CStringDictionary::getInstance().insert(DST.getValue().c_str()):
                                CStringDictionary::scm_nInvalidStringId;

    theCommand.m_nFirstParam = CStringDictionary::scm_nInvalidStringId;
    theCommand.m_nSecondParam = CStringDictionary::scm_nInvalidStringId;
    theCommand.m_sAdditionalParams.clear();

    if(parseRequest( RQST.getValue().c_str(), acID, theCommand.m_nCMD)){
      const char *acRequestPartLeft = strchr(&(RQST.getValue().c_str()[2]), '<');
      if(0 != acRequestPartLeft)
       acRequestPartLeft++; //point to the next charcter after the <
      // we got the command for execution
      // now check the rest of the data
      switch(theCommand.m_nCMD){
        case 0x00: // create something
          theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
          if(0 != acRequestPartLeft)
            switch(acRequestPartLeft[0]){
              case 'F': // we have an FB to Create
                if(parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                  theCommand.m_nCMD = cg_nMGM_CMD_Create_FBInstance;
                else
                  theCommand.m_nCMD = cg_nMGM_CMD_Create_FBType;
                break;
              case 'C': // we have an Connection to Create
                if(parseConnectionData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam))
                  theCommand.m_nCMD = cg_nMGM_CMD_Create_Connection;
                break;
              default: break;
            }
          break;
        case 0x01: //delete something
          theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
          if(0 != acRequestPartLeft)
            switch(acRequestPartLeft[0]){
              case 'F': // we have an FB to delete
                if(parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                  theCommand.m_nCMD = cg_nMGM_CMD_Delete_FBInstance;
                break;
              case 'C': // we have an Connection to delete
                if(parseConnectionData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam))
                  theCommand.m_nCMD = cg_nMGM_CMD_Delete_Connection;
                break;
              default: break;
            }
          break;
        case cg_nMGM_CMD_Start:
          if(0 != acRequestPartLeft)
            if('/' != acRequestPartLeft[0]){
              //if we have an additional xml token parse if it is an FB definition
              if(!parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
            }
          break;
        case cg_nMGM_CMD_Stop:
          if(0 != acRequestPartLeft)
            if('/' != acRequestPartLeft[0]){
              //if we have an additional xml token parse if it is an FB definition
              if(!parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
            }
          break;
        case cg_nMGM_CMD_Kill:
          if(0 != acRequestPartLeft)
            if('/' != acRequestPartLeft[0]){
              //if we have an additional xml token parse if it is an FB definition
              if(!parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
            }
          break;
        case cg_nMGM_CMD_Reset:
          if(0 != acRequestPartLeft)
            if('/' != acRequestPartLeft[0]){
              //if we have an additional xml token parse if it is an FB definition
              if(!parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams))
                theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
            }
          break;
        case cg_nMGM_CMD_Read:
          theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
          if(0 != acRequestPartLeft)
            if(parseConnectionData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam))
              theCommand.m_nCMD = cg_nMGM_CMD_Read;
          break;
        case cg_nMGM_CMD_Write:
          //We need an additional xml connection token parse if it is an connection definition
          theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
          if(0 != acRequestPartLeft)
            if(parseWriteData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_sAdditionalParams))
              theCommand.m_nCMD = cg_nMGM_CMD_Write;
          break;
        case 0x07: // query something
          theCommand.m_nCMD = cg_nMGM_CMD_INVALID;
          if(0 != acRequestPartLeft)
            switch(acRequestPartLeft[0]){
              case 'F': // query fb or fb type list
                if(!strncmp(acRequestPartLeft, "FBL", 3))
                  theCommand.m_nCMD = cg_nMGM_CMD_QUERY_FBTypes;
                if(parseFBData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam, theCommand.m_sAdditionalParams)) {
                  theCommand.m_nCMD = cg_nMGM_CMD_QUERY_FB;
                }
                break;
              case 'C': // query connection list
                if(parseConnectionData(acRequestPartLeft, theCommand.m_nFirstParam, theCommand.m_nSecondParam))
                  theCommand.m_nCMD = cg_nMGM_CMD_QUERY_Connection;
                break;
              case 'R': // query resource type list
                if(!strncmp(acRequestPartLeft, "RES", 3))
                  theCommand.m_nCMD = cg_nMGM_CMD_QUERY_RESTypes;
                break;
              case 'D': // query datatype list
                if(!strncmp(acRequestPartLeft, "DTL", 3))
                  theCommand.m_nCMD = cg_nMGM_CMD_QUERY_DTTypes;
                break;
              default: break;
            }
          break;
        default:
          break;
      }
      if (cg_nMGM_CMD_INVALID != theCommand.m_nCMD) {
        resp = m_poDevice.executeMGMCommand(theCommand);
      }
    }
  }
  if(0 < strlen(theCommand.m_sAdditionalParams.c_str())) {
    generateResponse(acID, resp, theCommand);
  }
  else {
    generateResponse(acID, resp);
  }
}

bool DEV_MGR::parseRequest(const char *pa_acRequestString, char *pa_sID, TMGMCommandType &pa_nRequestedCommand){
//first check if it is an management request

  if(!strncmp("<Request ID=\"", pa_acRequestString, 13)){
    int i = 13;
    int j;
    for(j = 0; pa_acRequestString[i] != '\"'; ++i, ++j){
      if(j >= 7)
        return false;
      pa_sID[j] = pa_acRequestString[i];
    }
    pa_sID[j] = '\0'; //close stack
    ++i;
    const char *acCommandStart = strchr((&pa_acRequestString[i]), '\"');
    if(acCommandStart != NULL){
      acCommandStart++; //this is the real start of the command
      if(!strncmp("CREATE", acCommandStart, 6))
        pa_nRequestedCommand = 0x00;
      else
        if(!strncmp("DELETE", acCommandStart, 6))
          pa_nRequestedCommand = 0x01;
        else
          if(!strncmp("START", acCommandStart, 5))
            pa_nRequestedCommand = cg_nMGM_CMD_Start;
          else
            if(!strncmp("STOP", acCommandStart, 4))
              pa_nRequestedCommand = cg_nMGM_CMD_Stop;
            else
              if(!strncmp("KILL", acCommandStart, 4))
                pa_nRequestedCommand = cg_nMGM_CMD_Kill;
              else
                if(!strncmp("RESET", acCommandStart, 5))
                  pa_nRequestedCommand = cg_nMGM_CMD_Reset;
                else
                  if(!strncmp("READ", acCommandStart, 4))
                    pa_nRequestedCommand = cg_nMGM_CMD_Read;
                  else
                    if(!strncmp("WRITE", acCommandStart, 5))
                      pa_nRequestedCommand = cg_nMGM_CMD_Write;
                    else
	                    if(!strncmp("QUERY", acCommandStart, 5))
	                      pa_nRequestedCommand = 0x07;
	                    else
                      return false;
      }
    }
    else
      return false;
    return true;
}

bool DEV_MGR::parseFBData(const char *pa_acRequestPartLeft, TConnectionID &pa_nFirstParam, TConnectionID &pa_nSecondParam, string &sAppName){
  bool bRetVal = false;

  if(!strncmp("FB Name=\"", pa_acRequestPartLeft, 9)) {
    if(strstr("EventIn=\"", pa_acRequestPartLeft) != 0) {
      // TODO parse all parameters
    } else {
      const char *acBuf = &(pa_acRequestPartLeft[9]);
      int i = 0;
      if(acBuf[0] != '*'){
        pa_nFirstParam = getIdentifierId(acBuf, '\"', i);
        acBuf = strchr(&(acBuf[i + 1]), '\"');
      }
      else{
        acBuf = strchr(&(acBuf[i + 2]), '\"');
      }

      if(acBuf != NULL){
        if(acBuf[1] != '*'){
          pa_nSecondParam = getIdentifierId(&(acBuf[1]), '\"', i);
          acBuf = strchr(&(acBuf[i + 1]), '\"');
          if(acBuf != NULL){
            // We have an application name given
            acBuf = &(acBuf[1]);
            i = 0;
            while((acBuf[i] != '\0') && (acBuf[i] != '\"')){
              sAppName += acBuf[i];
              i++;
            }
          }
        }
        bRetVal = true;
      }
    }
  }
  return bRetVal;
}

CStringDictionary::TStringId DEV_MGR::getIdentifierId(const char *pa_acIdentifierStart, char pa_cEndChar, int &pa_nCharsUsed){
  TIdentifier acBuf;

  for(pa_nCharsUsed = 0; pa_nCharsUsed < cg_nIdentifierLength; ++pa_nCharsUsed){
    if(pa_acIdentifierStart[pa_nCharsUsed] == pa_cEndChar){
      break;
    }
    acBuf[pa_nCharsUsed] = pa_acIdentifierStart[pa_nCharsUsed];
  }
  acBuf[pa_nCharsUsed] = '\0';
  return (0 != pa_nCharsUsed) ? CStringDictionary::getInstance().insert(acBuf) :
                                CStringDictionary::scm_nInvalidStringId;
}

TConnectionID DEV_MGR::getConnectionEndPointIdentifier(const char *pa_acIdentifierStart, int &pa_nCharsUsed){
  TConnectionID nRetVal = CStringDictionary::scm_nInvalidStringId;
  int i;

  if(pa_acIdentifierStart[0] != '*'){
    nRetVal = ((((TConnectionID)getIdentifierId(pa_acIdentifierStart, '.', i)) << 32) & 0xFFFFFFFF00000000LLU);
    pa_nCharsUsed = i;
    nRetVal += (((TConnectionID)getIdentifierId(&(pa_acIdentifierStart[i + 1]), '\"', i)) & 0xFFFFFFFF);
    pa_nCharsUsed += i;
  }
  return nRetVal;
}

bool DEV_MGR::parseConnectionData(const char *pa_acRequestPartLeft, TConnectionID &pa_nFirstParam, TConnectionID &pa_nSecondParam){
  bool bRetVal = false;
  if(!strncmp("Connection Source=\"", pa_acRequestPartLeft, 19)) {
    int i;
    pa_nFirstParam = getConnectionEndPointIdentifier(&(pa_acRequestPartLeft[19]), i);
    const char *acBuf = strchr(&(pa_acRequestPartLeft[i + 21]), '\"');
    if(acBuf != NULL){
      pa_nSecondParam = getConnectionEndPointIdentifier(&(acBuf[1]), i);
      bRetVal = true;
    }
  }
  return bRetVal;
}

bool DEV_MGR::parseWriteData(const char *pa_acRequestPartLeft, TConnectionID &pa_nFirstParam, string &pa_sValue){
  bool bRetVal = false;
  if(!strncmp("Connection Source=\"", pa_acRequestPartLeft, 19)) {
    int i;
    pa_acRequestPartLeft = &(pa_acRequestPartLeft[19]);
    //first retreive the data for the param
    if('&' == *pa_acRequestPartLeft){
      do{
        pa_acRequestPartLeft++;
      }while(';' != *pa_acRequestPartLeft);
      pa_acRequestPartLeft++;
    }
    if('\'' == *pa_acRequestPartLeft)
      pa_acRequestPartLeft++;
    // TODO: check if "-character should also be trimmed from pa_acRequestPartLeft
    for(; *pa_acRequestPartLeft != '\"'; ++pa_acRequestPartLeft){
      if('\0' == *pa_acRequestPartLeft){
        return false;
      }
      if('&' == *pa_acRequestPartLeft){
        do{
          pa_acRequestPartLeft++;
        }while(';' != *pa_acRequestPartLeft);
        pa_acRequestPartLeft += 2; //go beyound the ; and the following "
        break;
      }
      if('\'' == *pa_acRequestPartLeft){
        pa_acRequestPartLeft++;
        break;
      }
      pa_sValue += *pa_acRequestPartLeft;
    }

    pa_acRequestPartLeft = strchr(&(pa_acRequestPartLeft[1]), '\"');
    if (pa_acRequestPartLeft != 0){
      pa_nFirstParam = getConnectionEndPointIdentifier(&(pa_acRequestPartLeft[1]), i);
      bRetVal = true;
    }
  }
  return bRetVal;
}

void DEV_MGR::generateResponse(const char *pa_acID, EMGMResponse pa_eResp){
  char acRSPText[60] = {"<Response ID=\""};
  strncat(acRSPText, pa_acID, 59 - strlen(acRSPText));  //strlcat uses the real size of the string and is therefore saver that strncat. If strlcat is not available replace with strncat(acRSPText, pa_acID, 60 - strlen(acRSPText) - 1)
  strncat(acRSPText, "\"", 59 - strlen(acRSPText));
  if(e_RDY != pa_eResp){
    //sRspText += (" Reason=\"" + std::string(scg_sMGMResponseTexts[pa_eResp])+ "\"");
    strncat(acRSPText, " Reason=\"", 59 - strlen(acRSPText));
    strncat(acRSPText, scm_sMGMResponseTexts[pa_eResp], 59 - strlen(acRSPText));
    strncat(acRSPText, "\"", 59 - strlen(acRSPText));
  }
//  std::string sRspText("<Response ID=\"");
//  sRspText += pa_sID + "\"";
//  if(e_RDY != pa_eResp){
//    sRspText += (" Reason=\"" + std::string(scg_sMGMResponseTexts[pa_eResp])+ "\"");
//  }
  strncat(acRSPText, " />", 59 - strlen(acRSPText));
  RESP = acRSPText;
}

void DEV_MGR::generateResponse(const char *pa_acID, EMGMResponse pa_eResp, SManagementCMD &pa_stCMD){
  string sResponse = "<Response ID=\"";
  sResponse += pa_acID;
  sResponse += "\"";
  if(e_RDY != pa_eResp){
    sResponse +=  " Reason=\"";
    sResponse +=  scm_sMGMResponseTexts[pa_eResp];
    sResponse +=  "\"";
    sResponse +=  ">\n  ";
  } else {
    sResponse +=  ">\n  ";
    if(pa_stCMD.m_nCMD == cg_nMGM_CMD_QUERY_Connection) {
      sResponse += "<EndpointList>\n    ";
      sResponse += pa_stCMD.m_sAdditionalParams.c_str();
      sResponse += "\n  </EndpointList>";
    }
    else if (pa_stCMD.m_nCMD == cg_nMGM_CMD_QUERY_FB) {
      if((CStringDictionary::scm_nInvalidStringId != pa_stCMD.m_nSecondParam)  &&
         (CStringDictionary::scm_nInvalidStringId != pa_stCMD.m_nFirstParam)){
        sResponse += "<FB Name=\"";
        sResponse += pa_stCMD.m_sAdditionalParams.c_str();
        sResponse += "\" Type=\"";
        sResponse += CStringDictionary::getInstance().get(pa_stCMD.m_nSecondParam);
        sResponse += "\" />";
      }
      else {
        sResponse += "<FBList>\n    ";
        sResponse += pa_stCMD.m_sAdditionalParams.c_str();
        sResponse += "\n  </FBList>";
      }
    }
    else if (pa_stCMD.m_nCMD == cg_nMGM_CMD_Read) {
      sResponse += "<Connection Source=\"";
      if(CStringDictionary::scm_nInvalidStringId != pa_stCMD.m_nFirstParam){
        sResponse += CStringDictionary::getInstance().get(CConnection::extractFBNameId(pa_stCMD.m_nFirstParam));
        sResponse += ".";
        sResponse += CStringDictionary::getInstance().get(CConnection::extractPortNameId(pa_stCMD.m_nFirstParam));
      }
      sResponse += "\" Destination=\"";
      sResponse += pa_stCMD.m_sAdditionalParams.c_str();
      sResponse += "\" />";
    }
    else if (pa_stCMD.m_nCMD == cg_nMGM_CMD_QUERY_FBTypes) {
      sResponse += "<FBList>\n    ";
      sResponse += pa_stCMD.m_sAdditionalParams.c_str();
      sResponse += "\n  </FBList>";
    }
    else if (pa_stCMD.m_nCMD == cg_nMGM_CMD_QUERY_DTTypes) {
      sResponse += "<DTList>\n    ";
      sResponse += pa_stCMD.m_sAdditionalParams.c_str();
      sResponse += "\n  </DTList>";
    }
    else if (pa_stCMD.m_nCMD == cg_nMGM_CMD_QUERY_RESTypes) {
      sResponse += "<RESList>\n    ";
      sResponse += pa_stCMD.m_sAdditionalParams.c_str();
      sResponse += "\n  </RESList>";
    }
  }
  sResponse += "\n</Response>";
  RESP = sResponse.c_str();
}

DEV_MGR::DEV_MGR(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes):
         CServiceInterfaceFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec, m_apoEOConns, m_apoDIConns,
            m_apoDIs, m_apoDOConns, m_apoDOs),
         m_poDevice(pa_poSrcRes->getDevice()){
  m_apoDIs[0] = &QI;
  m_apoDIs[1] = &DST;
  m_apoDIs[2] = &RQST;
  m_apoDOs[0] = &QO;
  m_apoDOs[1] = &RESP;
}

DEV_MGR::~DEV_MGR(){
}
