/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _DEV_MGR_H_
#define _DEV_MGR_H_

#include <sifb.h>
#include <forte_bool.h>
#include <forte_string.h>

/*! \brief Implementation of the DEV_MGR FB.
 *
 * @author AZ
 * @version
 * @version 20060207/AZ - Removed number of EOs in CFunctionBlock-Constructor
 * @version 20060206/AZ - Changed event connection handling
 * @version 20051029/AZ - Initial version as basis for the interface documentation
 */
class DEV_MGR : public CServiceInterfaceFB{
  DECLARE_FIRMWARE_FB(DEV_MGR)

private:

  static const SFBInterfaceSpec scm_stFBInterfaceSpec;
  static const TFBNameListEntry scm_acEINames[2];
  static const TDataIOID scm_anEIWith[6];
  static const TINT16 scm_anEIWithIndexes[2];
  static const TFBNameListEntry scm_acEONames[2];
  static const TDataIOID scm_anEOWith[4];
  static const TINT16 scm_anEOWithIndexes[3];
  static const TFBNameListEntry scm_acDONames[2];
  static const TFBNameListEntry scm_acDINames[3];

  TEventConnectionPtr m_apoEOConns[2];
  TDataConnectionPtr m_apoDOConns[2];
  TIEC_ANYPtr m_apoDOs[2];
  TDataConnectionPtr m_apoDIConns[3];
  TIEC_ANYPtr m_apoDIs[3];


  static const TEventID csm_nEventINITOID;
  static const TEventID csm_nEventCNFID;

  static const TEventID csm_nEventINITID;
  static const TEventID csm_nEventREQID;

  CIEC_BOOL QI, QO;
  CIEC_STRING DST, RQST, RESP;
//! The device the block is contained in
  CDevice &m_poDevice;

/*!Extract a first and a second parameter from the xml string. This will be used to get the fbname and type or source and dest of  an connection xml string.
 * The identification is as follows. The first parameter will be taken till the first " is found in the string the second paramter will be taken from the second "
 * till the third " if the parameters in the requestpartleft string are longer then the lenght that can be stored in a TConnectionEndPointIdentifier
 * the paramter will be truncated.
 * Is a Application Name in the first parameter included (separated with '.'), the Applikation Name will be parsed too. The Application Name can include
 * come SubApplication Names. But its only necessary to find the last '.' and then seperate Instance Name and Application Name (with all its SubApplication Names)
 */
//   bool getFirstAndSecond(const char *pa_acRequestPartLeft, TConnectionEndPointIdentifier pa_acFirstParam,  TConnectionEndPointIdentifier pa_acSecondParam);
//   void getParameters(const char *pa_acRequestPartLeft, TConnectionEndPointIdentifier* pa_acParameters, int nSize, int &nReturnedPrams);
//   void getParametersNew(const char *pa_acRequestPartLeft, TConnectionEndPointIdentifier* pa_acParameters, int nSize, int &nReturnedPrams);
//   void getParametersWithAppName(const char *pa_acRequestPartLeft, TConnectionEndPointIdentifier* pa_acParameters, int nSize, int &nReturnedPrams);


  void executeRQST(void);
/*! \brief Parse the given request header to determine the ID and the requested command
 *
 * \param pa_acRequestString data of the request
 * \param pa_acID reference to a string where the found id is stored in
 * \param pa_nRequestedCommand reference to store the parsed command type in
 * \return true if the given command header could be parsed
 */
  bool parseRequest(const char *pa_acRequestString, char *pa_acID, TMGMCommandType &pa_nRequestedCommand);
/*! \brief Parse the given request that is left after parsing the header to parse FB data
 *
 * \param pa_acRequestPartLeft data of the request that has been left after parsing the header
 * \param pa_acFirstParam reference to a string where the FB-Name is stored in
 * \param pa_acSecondParam reference to a string where the FB-Type is stored in
 * \return true if the FB data could be parsed
 */
  bool parseFBData(const char *pa_acRequestPartLeft,
      TConnectionID &pa_nFirstParam,
      TConnectionID &pa_nSecondParam,
                   string &AppName);
/*! \brief Parse the given request that is left after parsing the header to parse connection data
 *
 * \param pa_acRequestPartLeft data of the request that has been left after parsing the header
 * \param pa_acFirstParam reference to a string where the connection source is stored in
 * \param pa_acSecondParam reference to a string where the connection destionation is stored in
 * \return true if the connection data could be parsed
 */  
  bool parseConnectionData(const char *pa_acRequestPartLeft, TConnectionID &pa_nFirstParam, TConnectionID &pa_nSecondParam);  
  bool parseWriteData(const char *pa_acRequestPartLeft, TConnectionID &pa_nFirstParam, string &pa_sValue);                         
  CStringDictionary::TStringId getIdentifierId(const char *pa_acIdentifierStart, char pa_cEndChar, int &pa_nCharsUsed);
  TConnectionID getConnectionEndPointIdentifier(const char *pa_acIdentifierStart, int &pa_nCharsUsed);
protected:
  virtual void  executeEvent (TEventID pa_nEIID);

//   bool parseConnectionData(const char *pa_acRequestPartLeft,
//                            TConnectionEndPointIdentifier* pa_acParameters, int &nReturnedParams);
//   bool parseReconnectData(const char *pa_acRequestPartLeft,
//                           TConnectionEndPointIdentifier pa_acFirstParam,
//                           TConnectionEndPointIdentifier pa_acSecondParam, string &pa_sValue);
/*! \brief set the RESP output of the DEV_MGR according to the given response data
 *
 * \param pa_acID id of the response
 * \param pa_eResp qualifier of the response
 */
  void generateResponse(const char *pa_acID, EMGMResponse pa_eResp);
/*! \brief set the RESP output of the DEV_MGR according to the given response data
 *
 * \param pa_acID id of the response
 * \param pa_eResp qualifier of the response
 * \param pa_stCMD the command type
 */
  void generateResponse(const char *pa_acID, EMGMResponse pa_eResp, SManagementCMD &pa_stCMD);


public:
  /*!\brief Type for the response of MGM command messages
 *
 * TODO fully define all responses as defined in IEC 61499 inc. numbers.
 */
  static const char *scm_sMGMResponseTexts[13];
	DEV_MGR(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes);
	virtual ~DEV_MGR();
};

#endif /*DEV_MGR_H_*/
