/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "DM_KRNL.h"
#include <typelib.h>

DEFINE_FIRMWARE_FB(DM_KRNL)

const TFBNameListEntry DM_KRNL::scm_acDataInputNames[] = {"QI", "ID"};

const TFBNameListEntry DM_KRNL::scm_acDataOutputNames[] = {"QO", "STATUS"};

const TEventID DM_KRNL::scm_nEventINITID = 0;
const TINT16 DM_KRNL::scm_anEIWithIndexes[] = {0};
const TDataIOID DM_KRNL::scm_anEIWith[] = {0, 1, 255};
const TFBNameListEntry DM_KRNL::scm_acEventInputNames[] = {"INIT"};

const TEventID DM_KRNL::scm_nEventINITOID = 0;
const TDataIOID DM_KRNL::scm_anEOWith[] = {0, 1, 255};
const TINT16 DM_KRNL::scm_anEOWithIndexes[] = {0, -1};
const TFBNameListEntry DM_KRNL::scm_acEventOutputNames[] = {"INITO"};



const SFBInterfaceSpec DM_KRNL::scm_stFBInterfaceSpec = {
  1,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  2,
  scm_acDataInputNames,
  2,
  scm_acDataOutputNames
};

const TUINT16 DM_KRNL::m_nNumInternalFBs = 2;

void DM_KRNL::executeEvent(TEventID pa_nEIID){
  switch(pa_nEIID){
    case scm_nEventINITID:
      m_oEventConnection_INIT.triggerEvent(*m_poInvokingExecEnv);
      break;
  }
}

DM_KRNL::DM_KRNL(CStringDictionary::TStringId pa_nAppl, CResource *pa_roSrcRes) : 
       CCompositeFB(pa_nAppl,
           pa_roSrcRes,
           &scm_stFBInterfaceSpec,
           m_apoEOConns,
           m_apoDIConns,
           m_apoIf2InDConns,
           m_apoDIs,
           m_apoDOConns,
           m_apoIn2IfDConns,
           m_apoDOs,
           m_nNumInternalFBs,
           m_apoInternalFBs),
       m_oEventConnection_SVR_INITO("SVR", "INITO"),
       m_oEventConnection_SVR_IND("SVR","IND"),
       m_oEventConnection_MGR_CNF("MGR","CNF"),
       m_oEventConnection_INIT(0,"INIT"),
       m_oEventConnection_MGR_INITO("MGR","INITO"),
       m_oDataConnection_QI("QI",&QI),
       m_oDataConnection_ID("ID",&ID),
       m_oDataConnection_SVR_STATUS("SVR","STATUS", &STATUS),
       m_oDataConnection_SVR_QO("SVR", "QO", &QO),
       m_oDataConnection_SVR_RD_2("SVR","RD_2"),
       m_oDataConnection_MGR_QO("MGR","QO"),
       m_oDataConnection_MGR_RESP("MGR","RESP"),
       m_oDataConnection_SVR_RD_1("SVR","RD_1"){
  
  m_apoInternalFBs[0] = createContainedFB("SVR", "SERVER_1_2", pa_roSrcRes);
  m_apoInternalFBs[1] = createContainedFB("MGR", "DEV_MGR",pa_roSrcRes); 

  m_apoIf2InDConns[0] = &m_oDataConnection_QI;
  m_apoIf2InDConns[1] = &m_oDataConnection_ID;
  m_apoIn2IfDConns[0] = &m_oDataConnection_SVR_QO;
  m_apoIn2IfDConns[1] = &m_oDataConnection_SVR_STATUS;

  m_oEventConnection_INIT.connect("MGR",*this,*m_apoInternalFBs[1],"INIT");
  m_oEventConnection_SVR_INITO.connectInterfaceEvent(this,scm_nEventINITOID);
  m_apoInternalFBs[0]->connectEO("INITO",&m_oEventConnection_SVR_INITO);
  m_oEventConnection_MGR_INITO.connect("SVR",*m_apoInternalFBs[1],*m_apoInternalFBs[0],"INIT");
  m_oEventConnection_SVR_IND.connect("MGR",*m_apoInternalFBs[0],*m_apoInternalFBs[1],"REQ");
  m_oEventConnection_MGR_CNF.connect("SVR",*m_apoInternalFBs[1],*m_apoInternalFBs[0],"RSP");

  m_oDataConnection_QI.connect("MGR",*this,*m_apoInternalFBs[1],"QI");
  m_oDataConnection_ID.connect("SVR",*this,*m_apoInternalFBs[0],"ID");
  m_oDataConnection_SVR_QO.connectInterfaceData(*m_apoInternalFBs[0],*this,"QO");
  m_oDataConnection_SVR_STATUS.connectInterfaceData(*m_apoInternalFBs[0],*this,"STATUS");
  m_oDataConnection_MGR_QO.connect("SVR",*m_apoInternalFBs[1],*m_apoInternalFBs[0],"QI");
  m_oDataConnection_SVR_RD_1.connect("MGR",*m_apoInternalFBs[0],*m_apoInternalFBs[1],"DST");
  m_oDataConnection_SVR_RD_2.connect("MGR",*m_apoInternalFBs[0],*m_apoInternalFBs[1],"RQST");
  m_oDataConnection_MGR_RESP.connect("SVR",*m_apoInternalFBs[1],*m_apoInternalFBs[0],"SD_1");

  m_apoDIs[0] = &QI;
  m_apoDIs[1] = &ID;
  m_apoDOs[0] = &QO;
  m_apoDOs[1] = &STATUS;
  
  
}
 
DM_KRNL::~DM_KRNL(){
}



