/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20090125: az, zoitl@acin.tuwien.ac.at - added check in function output in order to support new any data port design
 *     20070920: az, zoitl@acin.tuwien.ac.at - Reworked interface of functions returning just 0 to void
 *******************************************************************************/
 #include "./loccommfb.h"

CSyncObject CLocalCommFB::sm_oSync;
map<string, CLocalCommFB::TlocalCommGroup*, less<string> > CLocalCommFB::sm_MlocalCommGroup;


CLocalCommFB::TlocalCommGroup* CLocalCommFB::getlocalCommGroup(const string &pa_sID) {
  map<string, TlocalCommGroup*, less<string> >::iterator mapiter(sm_MlocalCommGroup.find(pa_sID));
  TlocalCommGroup* pCommGroup;
  //search map for existing ID
  if (sm_MlocalCommGroup.end() != mapiter) {  // found
  	// get existing CommGroup struct;
	  pCommGroup = (*mapiter).second;
  } else { // not found
  	  // create new CommGroup struct and sync object
  	  pCommGroup = new TlocalCommGroup;
  	  pCommGroup->m_oGroupSync = new CSyncObject;
  	  // insert ID into map
  	  sm_MlocalCommGroup[pa_sID] = pCommGroup;
  }
  return pCommGroup;
}

void CLocalCommFB::removelocalCommGroup(TlocalCommGroup* pa_pstCommGroup, const string &pa_sID) {

  // remove Commgroup if there are no more blocks in it
  if ((pa_pstCommGroup->m_lPublFBList.empty()) && pa_pstCommGroup->m_lPublFBList.empty()) {
  	delete pa_pstCommGroup->m_oGroupSync;
  	sm_MlocalCommGroup.erase(pa_sID);
  	delete pa_pstCommGroup;
  }
}

CLocalCommFB::TlocalCommGroup* CLocalCommFB::registerPublFB(const string &pa_sID) {
  TlocalCommGroup* pCommGroup;

  sm_oSync.lock();
  pCommGroup = getlocalCommGroup(pa_sID);
  pCommGroup->m_lPublFBList.push_back(this);
  sm_oSync.unlock();
  return pCommGroup;
}

void CLocalCommFB::registerSublFB(const string &pa_sID) {
  TlocalCommGroup* pCommGroup;

  sm_oSync.lock();
  pCommGroup = getlocalCommGroup(pa_sID);
  pCommGroup->m_lSublFBList.push_back(this);
  sm_oSync.unlock();
}

void CLocalCommFB::unregisterPublFB(const string &pa_sID) {
  TlocalCommGroup* pCommGroup;

  sm_oSync.lock();
  pCommGroup = getlocalCommGroup(pa_sID);
  pCommGroup->m_lPublFBList.remove(this);
  removelocalCommGroup(pCommGroup, pa_sID);
  sm_oSync.unlock();
}


void CLocalCommFB::unregisterSublFB(const string &pa_sID) {
  TlocalCommGroup* pCommGroup;

  sm_oSync.lock();
  pCommGroup = getlocalCommGroup(pa_sID);
  pCommGroup->m_lSublFBList.remove(this);
  removelocalCommGroup(pCommGroup, pa_sID);
  sm_oSync.unlock();
}

void CLocalCommFB::notify(TlocalCommGroup *pa_polocalCommGroup) {
  pa_polocalCommGroup->m_oGroupSync->lock();

  // go through GroupList and trigger all Subscribers
  for(list<CLocalCommFB*>::iterator listiter(pa_polocalCommGroup->m_lSublFBList.begin()); listiter != pa_polocalCommGroup->m_lSublFBList.end(); listiter++) {
   (*listiter)->output(SD);
  }
  pa_polocalCommGroup->m_oGroupSync->unlock();
}

void CLocalCommFB::output(TIEC_ANYPtr *data) {
  for (unsigned int i=0;i<m_nNumDataOutput;i++) {
    if((0 != RD[i]) && (0 != data[i])){
      if(RD[i]->getDataTypeID() == data[i]->getDataTypeID()){
        (RD[i])->setValue(*data[i]);
      }
    }
  }
  sm_poDeviceExecution->startNewEventChain(this);
}

