/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *     20070920: az, zoitl@acin.tuwien.ac.at - Reworked interface of functions returning just 0 to void
 *******************************************************************************/
#ifndef _LOCCOMMFB_H_
#define _LOCCOMMFB_H_

#include <map>
#include <list>
#include "./commfb.h"
#include "../arch/active_arch/sync.h"
#include "../core/extevhan.h"
#include "../core/devexec.h"

/*! Base class for all local communication function blocks (e.g. PUBL, SUBL).
 *
 * @author SMR
 * @version 20081121/mmm - minor changes due to elimination of ANY_FACADE
 * @version 20070920/AZ - Reworked interface of functions returning just 0 to void
 */
class CLocalCommFB : public CCommFB, public CExternalEventHandler {
  protected:
    //! Structure for storing the necessary data for all local Communication groups
    typedef struct SlocalCommGroup {
        list<CLocalCommFB*> m_lPublFBList;
        list<CLocalCommFB*> m_lSublFBList;
        CSyncObject *m_oGroupSync;
    } TlocalCommGroup;

  private:
    //! Map holding the datastructure of all local Communication Groups
    static map<string, TlocalCommGroup*, less<string> > sm_MlocalCommGroup;
    /*!\brief this function returns an pointer to a Commgroup datastructure corresponding to the ID parameter
     */
    static TlocalCommGroup* getlocalCommGroup(const string &pa_sID);
    /*!\brief this function cleans up the datastructures if a FB is removed
     */
    static void removelocalCommGroup(TlocalCommGroup* pa_pstCommGroup, const string &pa_sID);

  protected:
    /*!\brief variable holding the string with the current ID registered in the Communication Group
     */
    string m_sID;

    /*!\brief The Sync object used locking the access to the internal used datastructures
     */
    static CSyncObject sm_oSync;

    /*!\brief registers a Publ FB to a local Communicaton Group with the given ID
     */
    TlocalCommGroup* registerPublFB(const string &pa_sID);
    /*!\brief unregisters a Publ FB to a local Communicaton Group with the given ID
     */
    void unregisterPublFB(const string &pa_sID);
    /*!\brief registers a Subl FB to a local Communicaton Group with the given ID
     */
    void registerSublFB(const string &pa_sID);
    /*!\brief unregisters a Subl FB to a local Communicaton Group with the given ID
     */
    void unregisterSublFB(const string &pa_sID);
    /*!\brief used by publ-FBs to transfer data to the Subl-FBs
     */
    void notify(TlocalCommGroup *pa_polocalCommGroup);
    /*!\brief function called by notify() in each subl FB of a local Communication Group
     */
    void output(TIEC_ANYPtr *data);

  public:
    CLocalCommFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes, const SFBInterfaceSpec *pa_stFBInterfaceSpec, TEventConnectionPtr *pa_apoEOConns): CCommFB(pa_nAppl, pa_poSrcRes, pa_stFBInterfaceSpec, pa_apoEOConns){
    };
    virtual ~CLocalCommFB(){
      //TODO if connected close connection
    };

    void enableHandler(void) {
    };

    void disableHandler(void) {
    };

    void setPriority(int ) {
    };

    int getPriority(void) {
      return 0;
    };
};

#endif //_LOCCOMMFB_H_
