/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: FB_DIV_INT
 *** Description: Divides two INIT values
 *** Version: 
 ***     1.0: 2007-06-26/TS - PROFACTOR GmbH - 
 *************************************************************************/

#include "FB_DIV_INT.h"

DEFINE_FIRMWARE_FB(FB_DIV_INT)

const TFBNameListEntry FB_DIV_INT::scm_acDataInputNames[] = {"IN1", "IN2"};

const TFBNameListEntry FB_DIV_INT::scm_acDataOutputNames[] = {"OUT"};

const TEventID FB_DIV_INT::scm_nEventREQID = 0;
const TINT16 FB_DIV_INT::scm_anEIWithIndexes[] = {0};
const TDataIOID FB_DIV_INT::scm_anEIWith[] = {0, 1, 255};
const TFBNameListEntry FB_DIV_INT::scm_acEventInputNames[] = {"REQ"};

const TEventID FB_DIV_INT::scm_nEventCNFID = 0;
const TDataIOID FB_DIV_INT::scm_anEOWith[] = {0, 255};
const TINT16 FB_DIV_INT::scm_anEOWithIndexes[] = {0, -1};
const TFBNameListEntry FB_DIV_INT::scm_acEventOutputNames[] = {"CNF"};

const SFBInterfaceSpec FB_DIV_INT::scm_stFBInterfaceSpec = {
  1,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  2,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

void FB_DIV_INT::alg_REQ(void){
OUT = IN1/IN2;
}

const TINT16 FB_DIV_INT::scm_nStateSTART = 0;
const TINT16 FB_DIV_INT::scm_nStateREQ = 1;

void FB_DIV_INT::enterStateSTART(void){
  m_nECCState = scm_nStateSTART;
}

void FB_DIV_INT::enterStateREQ(void){
  m_nECCState = scm_nStateREQ;
  alg_REQ();
  sendOutputEvent( scm_nEventCNFID);
}

bool FB_DIV_INT::executeECCOnce(TEventID pa_nEIID){
  bool bRetVal = true;
  switch(m_nECCState){
    case scm_nStateSTART:
      if(scm_nEventREQID == pa_nEIID)
      	enterStateREQ();
      else
        bRetVal = false; //no transition cleared
      break;
    case scm_nStateREQ:
      if(1)
      	enterStateSTART();
      else
        bRetVal = false; //no transition cleared
      break;
    default:
      DEVLOG_ERROR(cg_nID_CFunctionBlock, 0x01 , "The ECC state is not in the valid range!");
      m_nECCState = 0; //0 is allways the initial state
      break;
  }
  return bRetVal;
}

FB_DIV_INT::FB_DIV_INT(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
       CBasicFB(pa_nAppl,
           pa_poSrcRes,
           &scm_stFBInterfaceSpec,
           m_apoEOConns,
           m_apoDIConns,
           m_apoDIs,
           m_apoDOConns,
           m_apoDOs,
            0, 0, 0) {
  m_apoDIs[0] = &IN1;
  m_apoDIs[1] = &IN2;
  m_apoDOs[0] = &OUT;
}
 
FB_DIV_INT::~FB_DIV_INT(){
}



