/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20080425: ih, hegny@acin.tuwien.ac.at    - adopted behaviour of Client-FBs to meet the new Version of compliance profile for feasibility studies
 *    20071029: az, alil@users.sourceforge.net - fixed bug in delte of data in the destructor
  *******************************************************************************/
#include <string.h>
#include "GEN_CLIENT.h"
#include "../../core/resource.h"


DEFINE_GENERIC_FB(GEN_CLIENT)

const TFBNameListEntry GEN_CLIENT::scm_acEventInputNames[2] = {"INIT", "REQ"};
const TFBNameListEntry GEN_CLIENT::scm_acEventOutputNames[2] = {"INITO", "CNF"};
const TINT16 GEN_CLIENT::scm_anEIWidthIndexes[2] = {-1, -1};
const TINT16 GEN_CLIENT::scm_anEOWidthIndexes[3] = {-1, -1, -1};



const SFBInterfaceSpec GEN_CLIENT::scm_stFBInterfaceSpec = { \
  2, \
  scm_acEventInputNames, \
  0, \
  scm_anEIWidthIndexes, \
  2, \
  scm_acEventOutputNames, \
  0, \
  scm_anEOWidthIndexes, \
  0, \
  0, \
  0, \
  0
};

const TEventID GEN_CLIENT::scm_nEventINITID = 0;
const TEventID GEN_CLIENT::scm_nEventREQID = 1;
const TEventID GEN_CLIENT::scm_nEventINITOID = 0;
const TEventID GEN_CLIENT::scm_nEventCNFID = 1;

const int GEN_CLIENT::scm_nSendBufferSize = 1024; //TODO: Check this value!
const int GEN_CLIENT::scm_nRecvBufferSize = 1024; //TODO: Check this value!


void  GEN_CLIENT::executeEvent(TEventID pa_nEIID){
int result=0;
//int i=0;

  switch(pa_nEIID) {
    case scm_nEventINITID:
      if (0 != m_poQI)
        m_poQI->readData(&QI);
      if (0 != m_poID)
        m_poID->readData(&ID);
      QO = QI;
      if (QI) { // INIT+
        strncpy(stConnectionInformation.m_pcConnectionID, ID, 31);
        stConnectionInformation.m_pcConnectionID[31] = '\0';
        if (m_roSocketHandler.registerSocketFB(stConnectionInformation) != 0)
          STATUS=scm_sResponseTexts[0]; //OK
        else {
          QO = 0;
          STATUS=scm_sResponseTexts[2]; // TERMINATED
        }
      }
      else {  //INIT-
        STATUS = ((m_roSocketHandler.unregisterSocketFB(this) == 0) ? scm_sResponseTexts[0] : scm_sResponseTexts[2]);
      }
      if (0 != m_poQO)
        m_poQO->writeData(&QO);
      if (0 != m_poSTATUS)
        m_poSTATUS->writeData(&STATUS);
      sendOutputEvent(scm_nEventINITOID);
      break;
    case scm_nEventREQID:   // REQ
      if (0 != m_poQI)
        m_poQI->readData(&QI);
      for (unsigned int i=0;i<m_nNumDataInput;i++) {
        if (0 != m_poSD[i] && SD[i] != 0) {
          (m_poSD[i])->readData(SD[i]);
        }
      }
      if(QI == 1){
        if (sendData()!= 0) {
        // TODO: do some error handling
        }
      }
      else{
        QO = false;
        STATUS = scm_sResponseTexts[5]; //INHIBITED
        if (0 != m_poQO)
          m_poQO->writeData(&QO);
        if (0 != m_poSTATUS)
            m_poSTATUS->writeData(&STATUS);
        // Sending CNF output event
        sendOutputEvent(scm_nEventCNFID);
      }
      break;
    case cg_nExternalEventID:
      result = receiveData();
      if (result == 0) {
        QO = true;
        STATUS = scm_sResponseTexts[0]; //OK
        if (0 != m_poQO)
          m_poQO->writeData(&QO);
        if (0 != m_poSTATUS)
          m_poSTATUS->writeData(&STATUS);
        for (unsigned int i=0;i<m_nNumDataOutput;i++)
          if (0 != m_poRD[i] && RD[i] != 0)
            (m_poRD[i])->writeData(RD[i]);
        sendOutputEvent(scm_nEventCNFID);
      }
      else{
        // something bad happend (e.g. server disconnected or recieving of data was not possible)
        STATUS = ((result == 1) ? scm_sResponseTexts[2] : scm_sResponseTexts[4]);
        QO = false;
        if (0 != m_poQO)
          m_poQO->writeData(&QO);
        if (0 != m_poSTATUS)
          m_poSTATUS->writeData(&STATUS);
        if (result==1) {
          m_roSocketHandler.unregisterSocketFB(this);
          sendOutputEvent(scm_nEventINITOID); // if an error happend we need to send INITO-
        }
        else {
          sendOutputEvent(scm_nEventCNFID);
        }
      }
      break;
    default:
      break;
  }
}

GEN_CLIENT::GEN_CLIENT(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes):
          CNetCommFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec,m_apoEOConns)
{
  setEventChainExecutor(pa_poSrcRes->getResourceEventExecution());
  m_stEventSourceEventEntry.m_poFB = this;

  stSendBuffer.m_pcData = new TBYTE[scm_nSendBufferSize];
  if (stSendBuffer.m_pcData == 0)
    DEVLOG_DEBUG(cg_nID_Client, 0x01, "GEN_CLIENT: unable to allocate memory for sendbuffer\n");
  stSendBuffer.m_nMaxSize = 0;
  stSendBuffer.m_nFilledSize = 0;
  stSendBuffer.m_nMaxSize = scm_nSendBufferSize;
  stSendBuffer.m_enConnectionState = CSocketHandler::e_Unconnected;
  stRecvBuffer.m_pcData = new TBYTE[scm_nRecvBufferSize];
  if (stRecvBuffer.m_pcData == 0)
    DEVLOG_DEBUG(cg_nID_Client, 0x01, "GEN_CLIENT: unable to allocate memory for recvbuffer\n");
  stRecvBuffer.m_nMaxSize = 0;
  stRecvBuffer.m_nFilledSize = 0;
  stRecvBuffer.m_nMaxSize = scm_nRecvBufferSize;
  stRecvBuffer.m_enConnectionState = CSocketHandler::e_Unconnected;

  stConnectionInformation.m_poSocketFB = this;
  stConnectionInformation.m_pstSendBuffer = &stSendBuffer;
  stConnectionInformation.m_pstRecvBuffer = &stRecvBuffer;
  stConnectionInformation.m_enServiceType = CSocketHandler::e_Client;

  strncpy(stConnectionInformation.m_pcConnectionID, "10.0.0.0:61499", 31);
  stConnectionInformation.m_pcConnectionID[31] = '\0';
}

GEN_CLIENT::~GEN_CLIENT(){
  m_roSocketHandler.unregisterSocketFB(this);
  delete[] stSendBuffer.m_pcData;
  delete[] stRecvBuffer.m_pcData;
}
