/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20071029: az, alil@users.sourceforge.net - fixed bug in delte of data in the destructor
 *******************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "GEN_SERCOMM.h"
#include "../../core/resource.h"
#include "../../arch/devlog.h"

DEFINE_FIRMWARE_FB(GEN_SERCOMM)

const TFBNameListEntry GEN_SERCOMM::scm_acEventInputNames[2] = {"INIT", "REQ"};
const TFBNameListEntry GEN_SERCOMM::scm_acEventOutputNames[2] = {"INITO", "IND"};
const TINT16 GEN_SERCOMM::scm_anEIWidthIndexes[2] = {-1, -1};
const TINT16 GEN_SERCOMM::scm_anEOWidthIndexes[3] = {-1, -1, -1};



const SFBInterfaceSpec GEN_SERCOMM::scm_stFBInterfaceSpec = { \
  2, \
  scm_acEventInputNames, \
  0, \
  scm_anEIWidthIndexes, \
  2, \
  scm_acEventOutputNames, \
  0, \
  scm_anEOWidthIndexes, \
  0, \
  0, \
  0, \
  0
};

const TEventID GEN_SERCOMM::scm_nEventINITID = 0;
const TEventID GEN_SERCOMM::scm_nEventREQID = 1;
const TEventID GEN_SERCOMM::scm_nEventINITOID = 0;
const TEventID GEN_SERCOMM::scm_nEventINDID = 1;

const int GEN_SERCOMM::scm_nSendBufferSize = 1024;
const int GEN_SERCOMM::scm_nRecvBufferSize = 1024;

void  GEN_SERCOMM::executeEvent(TEventID pa_nEIID){
 // TODO: Check return values of the called functions and set correct Status Message
  switch(pa_nEIID) {
    case scm_nEventINITID:
      initEvent();
      sendOutputEvent(scm_nEventINITOID);
      break;
    case scm_nEventREQID:
      requestEvent();
      break;
    case cg_nExternalEventID:
      externalEvent();
      sendOutputEvent(scm_nEventINDID);
      break;
    default:
      DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"GEN_SERCOMM:: got unknown EIID\n");
      // doing something here ??
  }
}

void GEN_SERCOMM::initEvent(void){
  std::string strBaud;

  if(0 != m_poQI)
    m_poQI->readData(&QI);
  if(0 != m_poID)
    m_poID->readData(&ID);
  if(0 != m_poBAUD)
    m_poBAUD->readData(&BAUD);
  BAUD.toString(strBaud);

  if(QI == true){ //INIT+
    strncpy(stConnectionInformation.m_pcConnectionID, ID, 7);
    stConnectionInformation.m_pcConnectionID[7] = '\0';
    stConnectionInformation.m_nBaudRate = atol(strBaud.c_str());
    //FIXME: Address of sendbuffer and receivebuffer already set in constructor,
    //but magically it changed at this point. Therefore it is set again.
    stConnectionInformation.m_pstSendBuffer = &stSendBuffer;
    stConnectionInformation.m_pstRecvBuffer = &stRecvBuffer;
    DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"GEN_SERCOMM:: got INIT+\n");
    if(m_roSerialHandler.registerSerialFB(stConnectionInformation) != 0){
      QO = true;
      STATUS = scm_sResponseTexts[0]; //OK
    }
    else{
      QO = false;
      STATUS = scm_sResponseTexts[2]; // TERMINATED
    }
  }
  else { //INIT-
    DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"GEN_SERCOMM:: got INIT-\n");
    //Disconnect the SIFB from the Serial Handler
    QO = false;
    if(m_roSerialHandler.unregisterSerialFB(this) == 0)
      STATUS = scm_sResponseTexts[0]; //OK
    else
      STATUS = scm_sResponseTexts[2]; //TERMINATED
  }
  if(0 != m_poQO)
    m_poQO->writeData(&QO);
  if(0 != m_poSTATUS)
    m_poSTATUS->writeData(&STATUS);
}

void GEN_SERCOMM::requestEvent(void){
  if(0 != m_poQI)
    m_poQI->readData(&QI);
  for(unsigned int i=0; i<m_nNumDataInput; ++i){
    if(0 != m_poSD[i] && SD[i] != 0)
      (m_poSD[i])->readData(SD[i]);
  }
  if(QI == 1)
    if(sendData()!= 0){
      DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"GEN_SERCOMM:: sendData failed\n");
    }
}

void GEN_SERCOMM::externalEvent(void){
  DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"GEN_SERCOMM:: got External Event\n");
  int nResult = receiveData();

  if(nResult == 0){
    QO = true;
    STATUS = scm_sResponseTexts[0]; //OK
  }
  else{
    QO = 0;
    STATUS = scm_sResponseTexts[2]; //TERMINATED
  }
  if(0 != m_poQO)
    m_poQO->writeData(&QO);
  if(0 != m_poSTATUS)
    m_poSTATUS->writeData(&STATUS);
  for(unsigned int i=0; i<m_nNumDataOutput; i++)
    if(0 != m_poRD[i] && RD[i] != 0)
     (m_poRD[i])->writeData(RD[i]);
}

GEN_SERCOMM::GEN_SERCOMM(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes):
          CSerCommFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec, m_apoEOConns){
  setEventChainExecutor(pa_poSrcRes->getResourceEventExecution());

  m_stEventSourceEventEntry.m_poFB = this;

  stSendBuffer.m_pcData = new char[scm_nSendBufferSize];
  if(stSendBuffer.m_pcData == 0){
    DEVLOG_ERROR(cg_nID_CFunctionBlock, 0x01, "GEN_SERCOMM:: unable to allocate memory\n");
  }
  stSendBuffer.m_nFilledSize = 0;
  stSendBuffer.m_nMaxSize = scm_nSendBufferSize;
  stSendBuffer.m_enConnectionState = CSerialHandler::e_Unconnected;
  stRecvBuffer.m_pcData = new char[scm_nRecvBufferSize];
  if(stRecvBuffer.m_pcData == 0){
    DEVLOG_ERROR(cg_nID_CFunctionBlock, 0x01, "GEN_SERCOMM:: unable to allocate memory\n");
  }
  stRecvBuffer.m_nFilledSize = 0;
  stRecvBuffer.m_nMaxSize = scm_nRecvBufferSize;
  stRecvBuffer.m_enConnectionState = CSerialHandler::e_Unconnected;

  stConnectionInformation.m_poSerialFB = this;
  stConnectionInformation.m_pstSendBuffer = &stSendBuffer;
  stConnectionInformation.m_pstRecvBuffer = &stRecvBuffer;
  stConnectionInformation.m_enServiceType = CSerialHandler::e_SerComm;


}

GEN_SERCOMM::~GEN_SERCOMM(){
  m_roSerialHandler.unregisterSerialFB(this);
  delete[] stSendBuffer.m_pcData;
  delete[] stRecvBuffer.m_pcData;
}

