/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *    20080425: ih, hegny@acin.tuwien.ac.at    - adopted behaviour of Server-FBs to meet the new Version of compliance profile for feasibility studies
 *    20071029: az, alil@users.sourceforge.net - fixed bug in delte of data in the destructor
 *******************************************************************************/
#include <string.h>
#include "GEN_SERVER.h"
#include "../../core/resource.h"
#include "../../arch/devlog.h"

DEFINE_GENERIC_FB(GEN_SERVER)

const TFBNameListEntry
    GEN_SERVER::scm_acEventInputNames[2] = { "INIT", "RSP" };
const TFBNameListEntry
    GEN_SERVER::scm_acEventOutputNames[2] = { "INITO", "IND" };
const TINT16 GEN_SERVER::scm_anEIWidthIndexes[2] = { -1, -1 };
const TINT16 GEN_SERVER::scm_anEOWidthIndexes[3] = { -1, -1, -1 };

const SFBInterfaceSpec
    GEN_SERVER::scm_stFBInterfaceSpec = { 2, scm_acEventInputNames, 0, scm_anEIWidthIndexes, 2, scm_acEventOutputNames, 0, scm_anEOWidthIndexes, 0, 0, 0, 0 };

const TEventID GEN_SERVER::scm_nEventINITID = 0;
const TEventID GEN_SERVER::scm_nEventRSPID = 1;
const TEventID GEN_SERVER::scm_nEventINITOID = 0;
const TEventID GEN_SERVER::scm_nEventINDID = 1;

const int GEN_SERVER::scm_nSendBufferSize = 1024; //TODO: Check this value!
const int GEN_SERVER::scm_nRecvBufferSize = 1024; //TODO: Check this value!

void GEN_SERVER::executeEvent(TEventID pa_nEIID) {
  // TODO: Check return values of the called functions and set correct Status Message
  switch (pa_nEIID) {
    case scm_nEventINITID:
      init();
      sendOutputEvent(scm_nEventINITOID);
      break;
    case scm_nEventRSPID:
      response();
      break;
    case cg_nExternalEventID:
      externalEvent();
      break;
    default:
      break;
      //TODO necessary for doing something here ??
  }
}

void GEN_SERVER::init(void) {
  if (0 != m_poQI)
    m_poQI->readData(&QI);
  if (0 != m_poID)
    m_poID->readData(&ID);
  QO = QI;
  if (QI == true) { //INIT+
    strncpy(stConnectionInformation.m_pcConnectionID, ID, 31);
    stConnectionInformation.m_pcConnectionID[31] = '\0';
    if (m_roSocketHandler.registerSocketFB(stConnectionInformation) != 0) {
      STATUS = scm_sResponseTexts[0]; //OK
    } else {
      QO = false;
      STATUS = scm_sResponseTexts[2]; // TERMINATED
    }
  } else { //INIT-
    DEVLOG_DEBUG(0x02,0x01,"GEN_SERVER::got INIT-\n");
    //disconnect the SIFB from the socket handler
    if (m_roSocketHandler.unregisterSocketFB(this) == 0)
      STATUS=scm_sResponseTexts[0]; //OK
    else
      STATUS=scm_sResponseTexts[2]; //TERMINATED
  }
  if (0 != m_poQO)
    m_poQO->writeData(&QO);
  if (0 != m_poSTATUS)
    m_poSTATUS->writeData(&STATUS);
}

void GEN_SERVER::response(void) {
  if (0 != m_poQI)
    m_poQI->readData(&QI);

  for (unsigned int i = 0; i < m_nNumDataInput; ++i) {
    if (0 != m_poSD[i] && SD[i] != 0)
      (m_poSD[i])->readData(SD[i]);
  }

  if (QI == 1)
    if (sendData()!= 0) {
      // TODO: do LOG this error
    }
}

void GEN_SERVER::externalEvent() {
  int result = receiveData();

  if (result == 0) {
    QO = true;
    STATUS=scm_sResponseTexts[0]; //OK
    if (0 != m_poQO)
      m_poQO->writeData(&QO);
    if (0 != m_poSTATUS)
      m_poSTATUS->writeData(&STATUS);
    for (unsigned int i = 0; i < m_nNumDataOutput; i++)
      if (0 != m_poRD[i] && RD[i] != 0)
        (m_poRD[i])->writeData(RD[i]);
    sendOutputEvent(scm_nEventINDID);
  } else {
    QO = false;
    if (0 != m_poQO)
      m_poQO->writeData(&QO);
    if (result == 1) {
      STATUS = scm_sResponseTexts[2]; //TERMINATED
      if (0 != m_poSTATUS)
        m_poSTATUS->writeData(&STATUS);
      sendOutputEvent(scm_nEventINITOID); //In the case of a client termination or other communication errors inito- needs to be issued
    } else {
      STATUS = scm_sResponseTexts[4]; //DATA_TYPE_ERROR
      if (0 != m_poSTATUS)
            m_poSTATUS->writeData(&STATUS);
      sendOutputEvent(scm_nEventINDID);
    }
  }
}

GEN_SERVER::GEN_SERVER(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) :
  CNetCommFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec, m_apoEOConns) {
  setEventChainExecutor(pa_poSrcRes->getResourceEventExecution());
  m_stEventSourceEventEntry.m_poFB = this;

  stSendBuffer.m_pcData = new TBYTE[scm_nSendBufferSize];
  if (stSendBuffer.m_pcData == 0)
    DEVLOG_DEBUG(cg_nID_Server, 0x01, "unable to allocate memory for sendbuffer\n");
  //TODO add some error handling. What to do in this case?
  stSendBuffer.m_nMaxSize = 0;
  stSendBuffer.m_nFilledSize = 0;
  stSendBuffer.m_nMaxSize = scm_nSendBufferSize;
  stSendBuffer.m_enConnectionState = CSocketHandler::e_Unconnected;
  stRecvBuffer.m_pcData = new TBYTE[scm_nRecvBufferSize];
  if (stRecvBuffer.m_pcData == 0)
    DEVLOG_DEBUG(cg_nID_Server, 0x01, "unable to allocate memory for recvbuffer\n");
  //TODO add some error handling. What to do in this case?
  stRecvBuffer.m_nMaxSize = 0;
  stRecvBuffer.m_nFilledSize = 0;
  stRecvBuffer.m_nMaxSize = scm_nRecvBufferSize;
  stRecvBuffer.m_enConnectionState = CSocketHandler::e_Unconnected;

  stConnectionInformation.m_poSocketFB = this;
  stConnectionInformation.m_pstSendBuffer = &stSendBuffer;
  stConnectionInformation.m_pstRecvBuffer = &stRecvBuffer;
  stConnectionInformation.m_enServiceType = CSocketHandler::e_Server;

  strncpy(stConnectionInformation.m_pcConnectionID, "0.0.0.0:61499", 31);
  stConnectionInformation.m_pcConnectionID[31] = '\0';
}

GEN_SERVER::~GEN_SERVER() {
  m_roSocketHandler.unregisterSocketFB(this);
  delete[] stSendBuffer.m_pcData;
  delete[] stRecvBuffer.m_pcData;
}

