/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: RT_E_CYCLE
 *** Description: Delayed propagation of an event-Cancellable
 *** Version: 
 ***     0.0: 2008-07-10/4DIAC-IDE - 4DIAC-Consortium
 *************************************************************************/
#include "RT_E_CYCLE.h"

DEFINE_FIRMWARE_FB(RT_E_CYCLE)

const TFBNameListEntry RT_E_CYCLE::scm_acDataInputNames[] = {"DT", "Deadline", "WCET"};
const TFBNameListEntry RT_E_CYCLE::scm_acDataOutputNames[] = {"QO"};

const TEventID RT_E_CYCLE::scm_nEventSTARTID = 0;
const TEventID RT_E_CYCLE::scm_nEventSTOPID = 1;
const TINT16 RT_E_CYCLE::scm_anEIWithIndexes[] = {0, -1};
const TDataIOID RT_E_CYCLE::scm_anEIWith[] = {0, 1, 2, 255};
const TFBNameListEntry RT_E_CYCLE::scm_acEventInputNames[] = {"START", "STOP"};

const TEventID RT_E_CYCLE::scm_nEventEOID = 0;
const TDataIOID RT_E_CYCLE::scm_anEOWith[] = {0, 255};
const TINT16 RT_E_CYCLE::scm_anEOWithIndexes[] = {-1, 0};
const TFBNameListEntry RT_E_CYCLE::scm_acEventOutputNames[] = {"EO"};

const SFBInterfaceSpec RT_E_CYCLE::scm_stFBInterfaceSpec = {
  2,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  3,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

RT_E_CYCLE::RT_E_CYCLE(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
            CEventSourceFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec, m_apoEOConns, m_apoDIConns, m_apoDIs, m_apoDOConns, m_apoDOs){
  m_apoDIs[0] = &DT;
  m_apoDIs[1] = &Deadline;
  m_apoDIs[2] = &WCET;
  m_apoDOs[0] = &QO;
  setEventChainExecutor(&m_oECEO);
  m_bActive = false;
  //Timout value is correctly initialized by the constructor m_stTimeListEntry.m_nTimeOut = 0;
  m_stTimeListEntry.m_nInterval = 0; 
  m_stTimeListEntry.m_pstNext = 0;
  m_stTimeListEntry.m_poTimedFB = this;
  m_stTimeListEntry.m_eType = e_Periodic;
  m_stEventSourceEventEntry.m_poFB = this;
  m_oECEO.start();
}

void RT_E_CYCLE::executeEvent(TEventID pa_nEIID){
  switch(pa_nEIID){
    case cg_nExternalEventID:
      sendOutputEvent(scm_nEventEOID);
      break;
    case scm_nEventSTOPID:
      if(m_bActive){
        m_oECEO.setDeadline(0);
        CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
        m_bActive = false;
      }
      break;
    case scm_nEventSTARTID:
      if(!m_bActive){
        m_oECEO.setDeadline(Deadline);
        m_stTimeListEntry.m_nInterval = DT;
        CTimerHandler::sm_poFORTETimer->registerTimedFB( &m_stTimeListEntry);
        m_bActive = true;
      }
      break;
    default:
      break;
  }
}






