/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: RT_E_DELAY
 *** Description: Delayed propagation of an event-Cancellable
 *** Version: 
 ***     0.0: 2008-07-10/4DIAC-IDE - 4DIAC-Consortium
 *************************************************************************/

#include "RT_E_DELAY.h"

DEFINE_FIRMWARE_FB(RT_E_DELAY)

const TFBNameListEntry RT_E_DELAY::scm_acDataInputNames[] = {"QI", "DT", "Tmin", "Deadline", "WCET"};

const TFBNameListEntry RT_E_DELAY::scm_acDataOutputNames[] = {"QO"};

const TEventID RT_E_DELAY::scm_nEventINITID = 0;
const TEventID RT_E_DELAY::scm_nEventSTARTID = 1;
const TEventID RT_E_DELAY::scm_nEventSTOPID = 2;
const TINT16 RT_E_DELAY::scm_anEIWithIndexes[] = {0, 5, -1};
const TDataIOID RT_E_DELAY::scm_anEIWith[] = {0, 2, 3, 4, 255, 1, 255};
const TFBNameListEntry RT_E_DELAY::scm_acEventInputNames[] = {"INIT", "START", "STOP"};

const TEventID RT_E_DELAY::scm_nEventINITOID = 0;
const TEventID RT_E_DELAY::scm_nEventEOID = 1;
const TDataIOID RT_E_DELAY::scm_anEOWith[] = {0, 255};
const TINT16 RT_E_DELAY::scm_anEOWithIndexes[] = {0, -1, -1};
const TFBNameListEntry RT_E_DELAY::scm_acEventOutputNames[] = {"INITO", "EO"};

const SFBInterfaceSpec RT_E_DELAY::scm_stFBInterfaceSpec = {
  3, 
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  2,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  5,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

RT_E_DELAY::RT_E_DELAY(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
            CEventSourceFB(pa_nAppl, pa_poSrcRes, &scm_stFBInterfaceSpec, m_apoEOConns, m_apoDIConns, m_apoDIs, m_apoDOConns, m_apoDOs){
  m_apoDIs[0] = &QI;
  m_apoDIs[1] = &DT;
  m_apoDIs[2] = &Tmin;
  m_apoDIs[3] = &Deadline;
  m_apoDIs[4] = &WCET;
  m_apoDOs[0] = &QO;

  setEventChainExecutor(&m_oECEO);
  m_bActive = false;
  m_bInitialized = false;
  //Timout value is correctly initialized by the constructor m_stTimeListEntry.m_nTimeOut = 0;
  m_stTimeListEntry.m_nInterval = 0; 
  m_stTimeListEntry.m_pstNext = 0;
  m_stTimeListEntry.m_poTimedFB = this;
  m_stTimeListEntry.m_eType = e_SingleShot;
  m_stEventSourceEventEntry.m_poFB = this; 
  m_oECEO.start();
}

void RT_E_DELAY::executeEvent(TEventID pa_nEIID){
  switch(pa_nEIID){
    case cg_nExternalEventID:
      sendOutputEvent(scm_nEventEOID);
      m_bActive = false;
      break;
    case scm_nEventSTOPID:
      if(m_bActive){
        CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
        m_bActive = false;
      }
      break;
    case scm_nEventSTARTID:
      if((!m_bActive)&&(m_bInitialized)){
        m_stTimeListEntry.m_nInterval = DT;
        CTimerHandler::sm_poFORTETimer->registerTimedFB( &m_stTimeListEntry);
        m_bActive = true;
      }
      break;
    case scm_nEventINITID:
      if(QI == true){
        if(!m_bInitialized){
          //m_oECEO.start(); 
          m_bInitialized = true;
        }
        m_oECEO.setDeadline(Deadline);
      }
      else{
       // m_oECEO.end();
        m_bInitialized = false;
        m_oECEO.setDeadline(0);
      }
      QO = QI;
      sendOutputEvent(scm_nEventINITOID);  
    default:
      break;
  }
}



