/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: RT_E_SWITCH
 *** Description: Split an event to two event chain branches
 *** Version: 
 ***     0.0: 2008-07-10/4DIAC-IDE - 4DIAC-Consortium
 *************************************************************************/

#include "RT_E_SWITCH.h"

DEFINE_FIRMWARE_FB(RT_E_SWITCH)

const TFBNameListEntry RT_E_SWITCH::scm_acDataInputNames[] = {"QI", "G", "Tmin", "Deadline_EO1", "WCET_EO1", "Deadline_EO2", "WCET_EO2"};

const TFBNameListEntry RT_E_SWITCH::scm_acDataOutputNames[] = {"QO"};

const TEventID RT_E_SWITCH::scm_nEventINITID = 0;
const TEventID RT_E_SWITCH::scm_nEventEIID = 1;
const TINT16 RT_E_SWITCH::scm_anEIWithIndexes[] = {0, 7};
const TDataIOID RT_E_SWITCH::scm_anEIWith[] = {0, 2, 3, 4, 5, 6, 255, 1, 255};
const TFBNameListEntry RT_E_SWITCH::scm_acEventInputNames[] = {"INIT", "EI"};

const TEventID RT_E_SWITCH::scm_nEventINITOID = 0;
const TEventID RT_E_SWITCH::scm_nEventEO1ID = 1;
const TEventID RT_E_SWITCH::scm_nEventEO2ID = 2;
const TDataIOID RT_E_SWITCH::scm_anEOWith[] = {0, 255};
const TINT16 RT_E_SWITCH::scm_anEOWithIndexes[] = {0, -1, -1, -1};
const TFBNameListEntry RT_E_SWITCH::scm_acEventOutputNames[] = {"INITO", "EO1", "EO2"};

const SFBInterfaceSpec RT_E_SWITCH::scm_stFBInterfaceSpec = {
  2,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  3,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  7,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

RT_E_SWITCH::RT_E_SWITCH(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
       CServiceInterfaceFB(pa_nAppl,
           pa_poSrcRes,
           &scm_stFBInterfaceSpec,
           m_apoEOConns,
           m_apoDIConns,
           m_apoDIs,
           m_apoDOConns,
           m_apoDOs) {
  m_apoDIs[0] = &QI;
  m_apoDIs[1] = &G;
  m_apoDIs[2] = &Tmin;
  m_apoDIs[3] = &Deadline_EO1;
  m_apoDIs[4] = &WCET_EO1;
  m_apoDIs[5] = &Deadline_EO2;
  m_apoDIs[6] = &WCET_EO2;
  m_apoDOs[0] = &QO;
  m_bInitialized = false;
}
 
void RT_E_SWITCH::executeEvent(TEventID pa_nEIID){
  if(scm_nEventEIID == pa_nEIID){
    if(m_bInitialized){
      if(G == false){
        if(0 != m_apoEOConns[scm_nEventEO1ID]){
      	  m_apoEOConns[scm_nEventEO1ID]->triggerEvent(m_oECEO1);
      	  m_oECEO1.resumeSelfSuspend();
        }
      }
      else{  
        if(0 != m_apoEOConns[scm_nEventEO2ID]){
      	  m_apoEOConns[scm_nEventEO2ID]->triggerEvent(m_oECEO2);
      	  m_oECEO2.resumeSelfSuspend();
        }
      }  
    } 
  }
  else
    if(scm_nEventINITID == pa_nEIID){
      if(QI == true){
        if(!m_bInitialized){
          m_oECEO1.start(); 
          m_oECEO2.start();
          m_bInitialized = true;
        }
        m_oECEO1.setDeadline(Deadline_EO1);
        m_oECEO2.setDeadline(Deadline_EO2);   
      }
      else{
        m_oECEO1.end();
        m_oECEO2.end();
        m_bInitialized = false;  
      }
      QO = QI;
      sendOutputEvent(scm_nEventINITOID);
    }
}




