/***************************************************************************
                                  _____                     
                                 / ____|                    
                           _   _| |     _ __ ___  _ __  ___ 
                          | | | | |    | '__/ _ \| '_ \/ __|
                          | |_| | |____| | | (_) | | | \__ \
                          | ._,_|\_____|_|  \___/|_| |_|___/
                          | |                               
                          |_|                               
                                                                                                     
                          rtesingle.h -  definition of the CEventChainExecutionThread class
                             -------------------
    begin                : 29.11.2006
    author               : (C) 2005 by AZ 
    email                : zoitl@acin.tuwien.ac.at
 ***************************************************************************/
#ifndef RTESINGLE_H_
#define RTESINGLE_H_

#include <sifb.h>
#include <ecet.h>
#include <forte_time.h>
#include <forte_bool.h>
#include <forte_array.h>


/*!\brief Base class for all single output real-time event function blocks.
 * 
 */
class CRTEventSingle: public CServiceInterfaceFB{
private:
  CEventChainExecutionThread m_oECEO;
  CSyncObject m_oSyncObj;              //!<Lock for protected the RT_E_FB from mutual events
  CIEC_TIME *m_poDeadline;
  bool m_bInitialized;
protected:
 /*! \brief check if the succeeding EC part needs to be activated.
  * 
  * This function makes the assumption that the input event with 
  * the id 0 is the INIT event input.
  * \param pa_nEIID the event id of the event that triggerd the FB
  * \return if true the succeding EC part will be activated.
  */
  virtual bool checkActivation(TEventID pa_nEIID) = 0; 
  virtual void executeEvent(TEventID pa_nEIID){
    if(0 != pa_nEIID){  //it is not the init event
      if(m_bInitialized){
        if(checkActivation(pa_nEIID)){
          if( 0 != m_apoEOConns[1]){
            m_apoEOConns[1]->triggerEvent(m_oECEO);
            m_oECEO.resumeSelfSuspend();
          }
        }  
      } 
    }
    else{ // we got init
      if(*((CIEC_BOOL*)m_apoDIs[0]) == true){
        if(!m_bInitialized){
          m_oECEO.start(); 
          m_bInitialized = true;
        }
        m_oECEO.setDeadline(*m_poDeadline);
      }
      else{
        m_oECEO.end();
        m_bInitialized = false;
      }
      *((CIEC_BOOL*)m_apoDIs[0]) = *((CIEC_BOOL*)m_apoDIs[0]);
      sendOutputEvent(0);
    }  
  }

public:
  CRTEventSingle(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes,  
           const SFBInterfaceSpec *pa_pstInterfaceSpec,
                 TEventConnectionPtr *pa_apoEOConns, 
                 TDataConnectionPtr *pa_apoDIConns,
                 TIEC_ANYPtr *pa_apoDIs,  
                 TDataConnectionPtr *pa_apoDOConns,  
                 TIEC_ANYPtr *pa_apoDOs,
                 CIEC_TIME *pa_poDeadline): 
      CServiceInterfaceFB(pa_nAppl, pa_poSrcRes, pa_pstInterfaceSpec, pa_apoEOConns, 
                        pa_apoDIConns, pa_apoDIs, pa_apoDOConns, pa_apoDOs){
      m_poDeadline = pa_poDeadline;
      m_bInitialized = false;   
  };
  virtual ~CRTEventSingle(){};
};

#endif /*RTESINGLE_H_*/
