/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include <string.h>
#include "./sercommfb.h"
#include "../arch/devlog.h"

const TFBNameListEntry CSerCommFB::scm_acDataInputNames[] = {"QI", "ID", "BAUD"};

int CSerCommFB::sendData(){
  stSendBuffer.m_nFilledSize = 0;
  //If we have no data inputs, we send only an event message
  if (m_nNumDataInput == 0){
    DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: No data inputs, just sending event message\n");
    stSendBuffer.m_pcData[0] = 5; //5 is event only code
    stSendBuffer.m_nFilledSize = 1;
  }
  else{
    for(unsigned int i=0; i < m_nNumDataInput; i++){
      if(SD[i]!=0) {
        if(packSendData(&stSendBuffer, SD[i])!= 0){
          DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: packSendData returned an error\n");
          return -1;
        }
      }
    }
  }

  DEVLOG_DEBUG_3(cg_nID_CFunctionBlock, 0x01, "CSerCommFB:: stSendBuffer.m_nFilledSize = %i on a maximum of %i\n", stSendBuffer.m_nFilledSize, stSendBuffer.m_nMaxSize);
  m_roSerialHandler.sendData(this);
  return 0;
}

int CSerCommFB::packSendData(CSerialHandler::SSharedData *pa_pstBuffer, CIEC_ANY *pa_poSD){
  DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: Entering packSendData\n");
  int nRetval = 0;
  int nFreeBytes = 0;
  char *cPos;
  cPos = &(pa_pstBuffer->m_pcData[pa_pstBuffer->m_nFilledSize]);
  nFreeBytes = pa_pstBuffer->m_nMaxSize - pa_pstBuffer->m_nFilledSize;
  //nRetval = pa_poSD->serialize((TBYTE*) cPos, nFreeBytes);
  nRetval = m_roSerDeser.serialize((TBYTE*) cPos, nFreeBytes, pa_poSD);
  if(nRetval == 0){
    //TODO : sendbuffer Full, some handling here ?
    DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: Sendbuffer full!!!\n");
    return -1;
  }
  pa_pstBuffer->m_nFilledSize += nRetval;
  return 0;
}

int CSerCommFB::receiveData(){
  int nRetval;
  m_nRDpos = 0;
  char *cBufPos;
  cBufPos = stRecvBuffer.m_pcData;
  if(m_nNumDataOutput == 0){
    if(stRecvBuffer.m_pcData[0] == 5 ) // we just send an output event
      m_roSerialHandler.receivedData(this);
    else{
      m_roSerialHandler.receivedData(this);
      DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: No Output events and received wrong ASN1 Code!\n");
      return -1;
    }
  }
  else
    while(1){ //Loop for going through all Outputs
      if(RD[m_nRDpos]==0) { //if RD[] is an ANY stop loop
        return -1; //TODO: mmm - Talk to alois about this error code
      }
      //nRetval = (RD[m_nRDpos])->deserialize((TBYTE*) cBufPos, stRecvBuffer.m_nFilledSize);
      nRetval = m_roSerDeser.deserialize((TBYTE*) cBufPos, stRecvBuffer.m_nFilledSize, (RD[m_nRDpos]));
      if(nRetval == 0){
        //TODO: Handle Partial Sends!
        DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: RECEIVED FRAGMENTED PACKET!-- not handled!\n");
        m_roSerialHandler.receivedData(this);
        return 0;
      }
      if(nRetval == -1){
        DEVLOG_DEBUG(cg_nID_CFunctionBlock,0x01,"CSerCommFB:: Deserialize returned an error\n");
        m_nRDpos = 0;
        return -1; // TODO : Handle this Error / Data for wrong Type received.
      }
      if(nRetval > 0){ //else
        stRecvBuffer.m_nFilledSize -= nRetval;
        cBufPos += nRetval;
      }
      if(++m_nRDpos == m_nNumDataOutput){
        m_nRDpos = 0;
        m_roSerialHandler.receivedData(this);
        break;
      }
    }
  return 0;
}

unsigned int CSerCommFB::getDIID(const char *pa_acDIName) const {
  char tempstring[7] = "SD_";
  if(!strcmp(pa_acDIName, scm_acDataInputNames[0])) {
    return scm_QI_ID;
  }
  if(!strcmp(pa_acDIName, scm_acDataInputNames[1])) {
    return scm_ID_ID;
  }
  if(!strcmp(pa_acDIName, scm_acDataInputNames[2])) {
    return scm_BAUD_ID;
  }
  for(unsigned int i = 0; i < m_nNumDataInput; i++) {
    snprintf(&(tempstring[3]), 7, "%i",i+1);
      if(!strcmp(pa_acDIName, tempstring))  {
        return i+3;
      }
  }
  return cg_nInvalidDataInputID;
}

CIEC_ANY *CSerCommFB::getDataInput(const unsigned int pa_nDIID){
  switch(pa_nDIID)  {
    case scm_QI_ID: {
      return &QI;
    }
    case scm_ID_ID: {
      return &ID;
    }
    default:  {
      if(pa_nDIID-3 < m_nNumDataInput)  {
        return SD[pa_nDIID-3];
      }
      else  {
        return 0;
      }
    }
  }
}

bool CSerCommFB::connectDI(const unsigned int pa_nDIID, CDataConnection* pa_poDataCon){
  switch(pa_nDIID){
    case scm_QI_ID: {
      m_poQI = pa_poDataCon;
      return true;
    }
    case scm_ID_ID: {
      m_poID = pa_poDataCon;
      return true;
    }
    default:  {
      if(pa_nDIID-3 < m_nNumDataInput)  {
        m_poSD[pa_nDIID-3] = pa_poDataCon;
        SD[pa_nDIID-3] = pa_poDataCon->getValue()->clone();
      return true;
      }
      else  {
        return false;
      }
    }
  }
}
