/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _SERCOMMFB_H_
#define _SERCOMMFB_H_

#include "./commfb.h"
#include "../arch/serhand.h"
#include "../core/datatypes/forte_udint.h"
#include "../core/serializer/ISerDeser.h"
#include "../core/serializer/CITASerDeser.h"

/*!\brief This class is the base class for serial communication FBs.
 *
 * Generic serial FBs can be derived from this class.
 *
 * @version 20090105/mmm - added CITASerDeser* m_pCSerDeser, changed constructor instructions for getting
 *  Serializer/Deserializer, changed packSendData and recieveData due to new Serializer/Deserializer
 * @version 20081121/mmm - added getDIID and const scm_BAUD_ID, changed connectDI and getDataInput
 *  added constant scm_BAUD_ID due to special block handling
 * @version 20061130/jbrunn - Changed Serial Handler
 * @version 20061128/jbrunn - First working version
 * @version 20061127/jbrunn - Initial version
 */
class CSerCommFB : public CCommFB {

private:

//!Variable holding the current position in the RD vector when receiving data.
  unsigned int m_nRDpos;
 /*!\brief Helper function for serializing all SD_x inputs to the sendbuffer.
 */
  int packSendData(CSerialHandler::SSharedData *pa_pstBuffer, CIEC_ANY *pa_poRD);

  //!< Serializer
  ISerDeser &m_roSerDeser;

protected:
  static const unsigned int scm_BAUD_ID=2; //Input ID 2 for BAUD Input
  static const TFBNameListEntry scm_acDataInputNames[];

  //Extra data inputs
  CDataConnection *m_poBAUD;
  //Extra Internal Datatypes
  CIEC_UDINT BAUD;

/*!Reference to the SerialHandler used by the network communication FBs.
 */
  CSerialHandler &m_roSerialHandler;
/*!Shared Memory between the FB and the Serialhandler.
 */
  CSerialHandler::SSharedData stSendBuffer;
/*!Shared Memory between the FB and the Serialhandler.
 */
  CSerialHandler::SSharedData stRecvBuffer;
/*!Structure holding the whole connection information of one FB.
 */
  CSerialHandler::SConnectionInformation stConnectionInformation;

/*!\brief Function for recieving data from the Serial Handler
 */
  int receiveData();
/*!\brief function for sending data via the Serial Handler
 */
  int sendData();
/*!\brief Overloading getDIID method defined in CComFB class
 *
 */
  virtual unsigned int getDIID(const char *pa_acDIName) const;
/*!\brief Overloading of the getDataInput method defined in the CCommFB class
 */
  CIEC_ANY* getDataInput (const unsigned int pa_nDIID);
/*!\brief Overloading of the connectDI method defined in the CCommFB class
 */
  bool connectDI(const unsigned int pa_nDIID, CDataConnection* pa_poDataCon);

public:
  CSerCommFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes, const SFBInterfaceSpec *pa_stFBInterfaceSpec, TEventConnectionPtr *pa_apoEOConns) : CCommFB(pa_nAppl, pa_poSrcRes, pa_stFBInterfaceSpec, pa_apoEOConns),
          m_roSerDeser(CITASerDeser::getInstance()),
          m_roSerialHandler(CSerialHandler::getInstance()){
    m_poBAUD = 0;
  }
  virtual ~CSerCommFB(){};
};

#endif //_SERCOMMFB_H_
