/*************************************************************************
 *** FORTE Library Element
 ***
 *** Name: CRYPT_1
 *** Description: Basic Function Block Type
 *** Version: 
 ***     1.0: 2006-11-14/TS - PROFACTOR GmbH - 
 *************************************************************************/

#include "CRYPT_1.h"

DEFINE_FIRMWARE_FB(CRYPT_1)

const TFBNameListEntry CRYPT_1::scm_acDataInputNames[] = {"INPUT"};

const TFBNameListEntry CRYPT_1::scm_acDataOutputNames[] = {"OUTPUT"};

const TEventID CRYPT_1::scm_nEventREQID = 0;
const TINT16 CRYPT_1::scm_anEIWithIndexes[] = {0};
const TDataIOID CRYPT_1::scm_anEIWith[] = {0, 255};
const TFBNameListEntry CRYPT_1::scm_acEventInputNames[] = {"REQ"};

const TEventID CRYPT_1::scm_nEventCNFID = 0;
const TDataIOID CRYPT_1::scm_anEOWith[] = {0, 255};
const TINT16 CRYPT_1::scm_anEOWithIndexes[] = {0, -1};
const TFBNameListEntry CRYPT_1::scm_acEventOutputNames[] = {"CNF"};

const SFBInterfaceSpec CRYPT_1::scm_stFBInterfaceSpec = {
  1,
  scm_acEventInputNames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEventOutputNames,
  scm_anEOWith,
  scm_anEOWithIndexes,
  1,
  scm_acDataInputNames,
  1,
  scm_acDataOutputNames
};

void CRYPT_1::alg_REQ(void){
  std::string buffer = "";
  char character = '\0';
  for(unsigned int i=0; i<INPUT.getValue().length(); i++){
    character = INPUT.getValue()[i];
    character++;
    buffer += character;
  }
  OUTPUT.setValue(buffer);
}

const TINT16 CRYPT_1::scm_nStateSTART = 0;
const TINT16 CRYPT_1::scm_nStateREQ = 1;

void CRYPT_1::enterStateSTART(){
  m_nECCState = scm_nStateSTART;
}

void CRYPT_1::enterStateREQ(){
  m_nECCState = scm_nStateREQ;
  alg_REQ();
  sendOutputEvent( scm_nEventCNFID);
}

bool CRYPT_1::executeECCOnce(TEventID pa_nEIID){
  bool bRetVal = true;
  switch(m_nECCState){
    case scm_nStateSTART:
      if(scm_nEventREQID == pa_nEIID)
      	enterStateREQ();
      else
        bRetVal = false; //no transition cleared
      break;
    case scm_nStateREQ:
      if(1)
      	enterStateSTART();
      else
        bRetVal = false; //no transition cleared
      break;
    default:
      DEVLOG_ERROR(cg_nID_CFunctionBlock, 0x01 , "The ECC state is not in the valid range!");
      m_nECCState = 0; //0 is allways the initial state
      break;
  }
  return bRetVal;
}

CRYPT_1::CRYPT_1(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes) : 
       CBasicFB(pa_nAppl,
           pa_poSrcRes,
           &scm_stFBInterfaceSpec,
           m_apoEOConns,
           m_apoDIConns,
           m_apoDIs,
           m_apoDOConns,
           m_apoDOs,
           0, 0, 0) {
  m_apoDIs[0] = &INPUT;
  m_apoDOs[0] = &OUTPUT;
}
 
CRYPT_1::~CRYPT_1(){
}



