/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#include "timedfb.h"
#include "../core/resource.h"

const TFBNameListEntry CTimedFB::scm_acEINames[] = {"START", "STOP"};
const TDataIOID CTimedFB::scm_anEIWith[] = {0, 255};
const TINT16 CTimedFB::scm_anEIWithIndexes[] = {0, -1};
const TFBNameListEntry CTimedFB::scm_acEONames[] = {"EO"}; 
const TINT16 CTimedFB::scm_anEOWithIndexes[] = {-1, -1};
const TFBNameListEntry CTimedFB::scm_acDINames[] = {"DT"};

const SFBInterfaceSpec CTimedFB::scm_stFBInterfaceSpec = {
  2, 
  scm_acEINames,
  scm_anEIWith,
  scm_anEIWithIndexes,
  1,
  scm_acEONames,
  0,
  scm_anEOWithIndexes,
  1,
  scm_acDINames,
  0,
  0
};

const TEventID CTimedFB::csm_nEventSTARTID = 0;
const TEventID CTimedFB::csm_nEventSTOPID = 1;

const TEventID CTimedFB::csm_nEOID = 0;

CTimedFB::CTimedFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes):
          CEventSourceFB(pa_nAppl, pa_poSrcRes,
          &scm_stFBInterfaceSpec, m_apoEOConns, m_apoDIConns, m_apoDIs, 0, 0){            
  setEventChainExecutor(pa_poSrcRes->getResourceEventExecution());
  m_apoDIs[0] = &DT;
  m_bActive = false;
  m_stTimeListEntry.m_stTimeOut.m_nLowerValue = 0;
  m_stTimeListEntry.m_stTimeOut.m_nUpperValue = 0;    
  m_stTimeListEntry.m_nInterval = 0; 
  m_stTimeListEntry.m_pstNext = 0;
  m_stEventSourceEventEntry.m_poFB = this;
  
  
}

void CTimedFB::executeEvent(TEventID pa_nEIID){
  switch(pa_nEIID){
    case cg_nExternalEventID:
      sendOutputEvent(csm_nEOID);
      break;
    case csm_nEventSTOPID:
      if(m_bActive){
        CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
        m_bActive = false;
      }
      break;
    case csm_nEventSTARTID:
      if(!m_bActive){
        m_stTimeListEntry.m_nInterval = DT;
        CTimerHandler::sm_poFORTETimer->registerTimedFB( &m_stTimeListEntry);
        m_bActive = true;
      }
      break;
    default:
      break;
  }
}

EMGMResponse CTimedFB::stopManagedObject(void) {
	if(m_bActive){
		CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
		m_bActive = false;
	}

	return CEventSourceFB::stopManagedObject();
}

EMGMResponse CTimedFB::killManagedObject(void) {
	if(m_bActive){
		CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
		m_bActive = false;
	}

	return CEventSourceFB::killManagedObject();
}

EMGMResponse CTimedFB::resetManagedObject(void) {
	if(m_bActive){
		CTimerHandler::sm_poFORTETimer->unregisterTimedFB(this);
		m_bActive = false;
	}

	return CEventSourceFB::resetManagedObject();
}  
