/*******************************************************************************
 * Copyright (c) 2007 4DIAC - consortium.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     <date>: <author>, <author email> - changes
 *******************************************************************************/
#ifndef _TIMEDFB_H_
#define _TIMEDFB_H_

#include "../core/esfb.h"
#include "../core/datatypes/forte_time.h"
#include "../arch/timerha.h"

/*!\brief Base class for timed function block like E_CYCLE or E_DELAY providing this interface
 * 
 * 
 * @author AZ
 * @version 20060206/AZ - Changed event connection handling
 * @version 20051203/AZ - First version as basis
 */
class CTimedFB : public CEventSourceFB{
private:
protected:  
  static const SFBInterfaceSpec scm_stFBInterfaceSpec;
  static const TFBNameListEntry scm_acEINames[];
  static const TDataIOID scm_anEIWith[];
  static const TINT16 scm_anEIWithIndexes[];
  static const TFBNameListEntry scm_acEONames[]; 
  static const TINT16 scm_anEOWithIndexes[];
  static const TFBNameListEntry scm_acDINames[];
  TEventConnectionPtr m_apoEOConns[1]; 
  TDataConnectionPtr m_apoDIConns[1]; 
  TIEC_ANYPtr m_apoDIs[1];
  
  static const TEventID csm_nEventSTARTID;
  static const TEventID csm_nEventSTOPID;
  
  static const TEventID csm_nEOID;
  
  CIEC_TIME DT;
    
  
  bool m_bActive; //!> flag to indicate that the timed fb is currently active
  STimedFBListEntry m_stTimeListEntry; //!> The Timer list entry of this timed FB 
/*!\brief execute the input events of timed FBs as far it is possible
 * 
 * Derived Timed FBs only normaly need only the start event es this is different for each timed FB type (e.g. periodic vs. onetimeshot)
 */ 
  virtual void executeEvent(TEventID pa_nEIID);

public:
  CTimedFB(CStringDictionary::TStringId pa_nAppl, CResource *pa_poSrcRes);
  virtual ~CTimedFB() {};
  

/*! \brief Stop the ManagedObject.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */  
  virtual EMGMResponse stopManagedObject(void);
/*! \brief Kill the ManagedO.
 *
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */  
  virtual EMGMResponse killManagedObject(void);
/*!\brief Reset the ManagedObject.
 * 
 * Derived objects will add aditional functionality to this function (e.g. E_RESTART)
 * @return response of the command execution as defined in IEC 61499
 */
  virtual EMGMResponse resetManagedObject(void);  
  
};

#endif /*TIMEDFB_H_*/
